﻿//-----------------------------------------------------------------------------
// <copyright file="GeneralTasks.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>
// Wrapper for general tasks of a USB interface.
// </summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using UsbInterfaceTesttool.SerialCommunication;

    /// <summary>
    /// Wrapper for general tasks of a USB interface.
    /// </summary>
    public class GeneralTasks
    {
        /// <summary>
        /// Initializes a new instance of the GeneralTasks class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public GeneralTasks(UsbInterface usbInterface)
        {
            this.UsbInterface = usbInterface;
        }

        /// <summary>
        /// Possible baud rates for a USB interface.
        /// </summary>
        public enum InterfaceBaudRate
        {
            /// <summary>
            /// Baud rate of 4.800 bit/s.
            /// </summary>
            _4800,

            /// <summary>
            /// Baud rate of 9.600 bit/s.
            /// </summary>
            _9600,

            /// <summary>
            /// Baud rate of 19.200 bit/s.
            /// </summary>
            _19200,

            /// <summary>
            /// Baud rate of 38.400 bit/s.
            /// </summary>
            _38400,

            /// <summary>
            /// Baud rate of 76.800 bit/s.
            /// </summary>
            _76800,

            /// <summary>
            /// Baud rate of 250.000 bit/s.
            /// </summary>
            _250000,

            /// <summary>
            /// Baud rate of 500.000 bit/s.
            /// </summary>
            _500000 
        }

        /// <summary>
        /// Possible SPI clock rates for a USB-SPI interface.
        /// </summary>
        public enum SpiClockRate
        {
            /// <summary>
            /// SPI clock rate of 62,5kHz.
            /// </summary>
            _62_5kHz,

            /// <summary>
            /// SPI clock rate of 125kHz.
            /// </summary>
            _125kHz,

            /// <summary>
            /// SPI clock rate of 250kHz.
            /// </summary>
            _250kHz,

            /// <summary>
            /// SPI clock rate of 500kHz.
            /// </summary>
            _500kHz,

            /// <summary>
            /// SPI clock rate of 1MHz.
            /// </summary>
            _1MHz,

            /// <summary>
            /// SPI clock rate of 2MHz.
            /// </summary>
            _2MHz,

            /// <summary>
            /// SPI clock rate of 4MHz.
            /// </summary>
            _4MHz
        }

        /// <summary>
        /// Gets or sets used USB-Interface for communication.
        /// </summary>
        /// <value>Used USB-Interface for communication.</value>
        public UsbInterface UsbInterface { get; set; }

        /// <summary>
        /// Converts value of the baud rate enum to a int value.
        /// </summary>
        /// <param name="baudRate">Value of the baud rate.</param>
        /// <returns>Baud rate as int.</returns>
        public static int GetBaudRate(GeneralTasks.InterfaceBaudRate baudRate)
        {
            switch (baudRate)
            {
                case InterfaceBaudRate._4800:
                    return 4800;
                case InterfaceBaudRate._9600:
                    return 9600;
                case InterfaceBaudRate._19200:
                    return 19200;
                case InterfaceBaudRate._38400:
                    return 38400;
                case InterfaceBaudRate._76800:
                    return 76800;
                case InterfaceBaudRate._250000:
                    return 250000;
                case InterfaceBaudRate._500000:
                    return 500000;
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Converts int baud rate to a value of the baud rate enum.
        /// </summary>
        /// <param name="baudRate">Int baud rate.</param>
        /// <returns>Value of the baud rate enum.</returns>
        public static GeneralTasks.InterfaceBaudRate GetIntegerBaudRate(int baudRate)
        {
            switch (baudRate)
            {
                case 4800:
                    return InterfaceBaudRate._4800;
                case 9600:
                    return InterfaceBaudRate._9600;
                case 19200:
                    return InterfaceBaudRate._19200;
                case 38400:
                    return InterfaceBaudRate._38400;
                case 76800:
                    return InterfaceBaudRate._76800;
                case 250000:
                    return InterfaceBaudRate._250000;
                case 500000:
                    return InterfaceBaudRate._500000;
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Sets a baud rate of a USB interface.
        /// </summary>
        /// <param name="baudRate">Baud rate to set.</param>
        public void SetBaudRate(GeneralTasks.InterfaceBaudRate baudRate)
        {
            const char ChangeBaudRate = 'X';

            string parameter;
            switch (baudRate)
            {
                case InterfaceBaudRate._4800:
                    parameter = "0048";
                    break;
                case InterfaceBaudRate._9600:
                    parameter = "0096";
                    break;
                case InterfaceBaudRate._19200:
                    parameter = "0192";
                    break;
                case InterfaceBaudRate._38400:
                    parameter = "0384";
                    break;
                case InterfaceBaudRate._76800:
                    parameter = "0768";
                    break;
                case InterfaceBaudRate._250000:
                    parameter = "2500";
                    break;
                case InterfaceBaudRate._500000:
                    parameter = "5000";
                    break;
                default:
                    throw new ArgumentException();
            }

            this.UsbInterface.Write(string.Format("{0} {1}", ChangeBaudRate, parameter));
        }

        /// <summary>
        /// Sets a SPI clock rate of a USB interface.
        /// </summary>
        /// <param name="spiClockRate">SPI clock rate to set.</param>
        public void SetSpiClockRate(GeneralTasks.SpiClockRate spiClockRate)
        {
            const char ChangeSpiClockRate = 'T';

            string parameter;
            switch (spiClockRate)
            {
                case SpiClockRate._62_5kHz:
                    parameter = "00625";
                    break;
                case SpiClockRate._125kHz:
                    parameter = "01250";
                    break;
                case SpiClockRate._250kHz:
                    parameter = "02500";
                    break;
                case SpiClockRate._500kHz:
                    parameter = "05000";
                    break;
                case SpiClockRate._1MHz:
                    parameter = "10000";
                    break;
                case SpiClockRate._2MHz:
                    parameter = "20000";
                    break;
                case SpiClockRate._4MHz:
                    parameter = "40000";
                    break;
                default:
                    throw new ArgumentException();
            }

            this.UsbInterface.Write(string.Format("{0} {1}", ChangeSpiClockRate, parameter));
        }
    }
}
