﻿//-----------------------------------------------------------------------------
// <copyright file="_3DBS.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper class for communicating with 3D-BS.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.Collections;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Wrapper class for communicating with 3D-BS.
    /// </summary>
    [Supports(new UsbInterface.Type[] { UsbInterface.Type.I2C, UsbInterface.Type.SPI })]
    public class _3DBS : Device
    {
        /// <summary>
        /// Position of the bit of a register address which indicates whether a read or write operation will be executed.
        /// </summary>
        public const int ReadWriteBit = 7;

        /// <summary>
        /// Amount of all documented registers.
        /// </summary>
        public const int AmountOfRegisters = 22;

        /// <summary>
        /// Current range of the 3D-BS.
        /// </summary>
        private Range currentRange;

        /// <summary>
        /// Initializes a new instance of the _3DBS class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public _3DBS(UsbInterface usbInterface)
            : base(usbInterface)
        {
        }

        /// <summary>
        /// Registers of the 3D-BS.
        /// </summary>
        public enum Registers
        {
            /// <summary>
            /// Flags of operational registers.
            /// </summary>
            OperationalRegistersFlags = 0x15,

            /// <summary>
            /// Register for range, bandwidth and critical sensor individual calibration data.
            /// </summary>
            RangeBandwidth = 0x14,

            /// <summary>
            /// Register for any_motion_dur and HG/LG_hyst.
            /// </summary>
            HystAndAnyMotionDur = 0x11,

            /// <summary>
            /// Register of any_motion_thres.
            /// </summary>
            AnyMotionThres = 0x10,

            /// <summary>
            /// Register of LG_dur.
            /// </summary>
            HgDur = 0x0F,

            /// <summary>
            /// Register of LG_thres.
            /// </summary>
            HgThres = 0x0E,

            /// <summary>
            /// Register of LG_dur.
            /// </summary>
            LgDur = 0x0D,

            /// <summary>
            /// Register of LG_thres.
            /// </summary>
            LgThres = 0x0C,

            /// <summary>
            /// Settings register for interrupt.
            /// </summary>
            InterruptSettings = 0x0B,

            /// <summary>
            /// Register for control settings.
            /// </summary>
            Control = 0x0A,

            /// <summary>
            /// Register for status settings.
            /// </summary>
            Status = 0x09,

            /// <summary>
            /// Register of lsb of x axis (acc data).
            /// </summary>
            AccXAxisLsb = 0x02,

            /// <summary>
            /// Register containing version info.
            /// </summary>
            Version = 0x01,

            /// <summary>
            /// Register containing chip ID info.
            /// </summary>
            ChipID = 0x00
        }

        /// <summary>
        /// Possible range of 3D-BS.
        /// </summary>
        public enum Range
        {
            /// <summary>
            /// Range of +/- 2g.
            /// </summary>
            _2g,

            /// <summary>
            /// Range of +/- 4g.
            /// </summary>
            _4g,

            /// <summary>
            /// Range of +/- 8g.
            /// </summary>
            _8g,

            /// <summary>
            /// Not authorised code.
            /// </summary>
            NotAuthorisedCode
        }

        /// <summary>
        /// Possible bandwidth of 3D-BS.
        /// </summary>
        public enum Bandwidth
        {
            /// <summary>
            /// Bandwidth of 25Hz.
            /// </summary>
            _25Hz,

            /// <summary>
            /// Bandwidth of 50Hz.
            /// </summary>
            _50Hz,

            /// <summary>
            /// Bandwidth of 100Hz.
            /// </summary>
            _100Hz,

            /// <summary>
            /// Bandwidth of 190Hz.
            /// </summary>
            _190Hz,

            /// <summary>
            /// Bandwidth of 375Hz.
            /// </summary>
            _375Hz,

            /// <summary>
            /// Bandwidth of 750Hz.
            /// </summary>
            _750Hz,

            /// <summary>
            /// Bandwidth of 1500Hz.
            /// </summary>
            _1500Hz,

            /// <summary>
            /// Not authorised code.
            /// </summary>
            NotAuthorisedCode
        }

        /// <summary>
        /// Possible wake up pause of 3D-BS.
        /// </summary>
        public enum WakeUpPause
        {
            /// <summary>
            /// Wake up pause of 20 ms.
            /// </summary>
            _20ms,

            /// <summary>
            /// Wake up pause of 80 ms.
            /// </summary>
            _80ms,

            /// <summary>
            /// Wake up pause of 320 ms.
            /// </summary>
            _320ms,

            /// <summary>
            /// Wake up pause of 2560 ms.
            /// </summary>
            _2560ms,
        }

        /// <summary>
        /// Possible counter values of 3D-BS.
        /// </summary>
        public enum Counter
        {
            /// <summary>
            /// Reset the counter.
            /// </summary>
            Reset,

            /// <summary>
            /// Count down by 1 LSB/ms.
            /// </summary>
            CountDownBy1LsbMs,

            /// <summary>
            /// Count down by 2 LSB/ms.
            /// </summary>
            CountDownBy2LsbMs,

            /// <summary>
            /// Count down by 3 LSB/ms.
            /// </summary>
            CountDownBy3LsbMs
        }

        /// <summary>
        /// Number of required consecutive conditions to set or reset the any motion interrupt.
        /// </summary>
        public enum AnyMotionDuration
        {
            /// <summary>
            /// 1 consecutive conditions.
            /// </summary>
            _1,

            /// <summary>
            /// 3 consecutive conditions.
            /// </summary>
            _3,

            /// <summary>
            /// 5 consecutive conditions.
            /// </summary>
            _5,

            /// <summary>
            /// 7 consecutive conditions.
            /// </summary>
            _7
        }

        /// <summary>
        /// Handles chaning addresses for the 6D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Passed new address.</param>
        public override void AddressChanged(object sender, AddressEvent e)
        {
            if (e.RelevantDevice != this.GetType())
            {
                return;
            }

            this.Address = e.GetFirstAddress();
        }

        /// <summary>
        /// Reads current settings of the 3D-BS.
        /// </summary>
        /// <returns>Current settings of the 3D-BS.</returns>
        public _3DBS_Settings ReadSettings()
        {
            byte[] registers = this.UsbInterface.Read(
                this.Address,
                0x00, 
                _3DBS.AmountOfRegisters,
                UsbInterface.Module._3DBS);

            _3DBS_Settings settings = new _3DBS_Settings();
            settings.OperationalSettings = this.ReadOperationalSettings(registers);
            settings.InterruptSettings = this.ReadInterruptSettings(registers);
            settings.ControlSettings = this.ReadControlSettings(registers);
            settings.StatusSettings = this.ReadStatusSettings(registers);
            settings.InfoSettings = this.ReadInfoSettings(registers);

            return settings;
        }

        /// <summary>
        /// Sets the current settings from the GUI to the 3D-BS.
        /// </summary>
        /// <param name="settings">Settings from the GUI.</param>
        public void SetSettings(_3DBS_Settings settings)
        {
            this.SetOperationalSettings(settings.OperationalSettings);
        }

        /// <summary>
        /// Reads current data (values of axis and new data flags) from the 3D-BS.
        /// </summary>
        /// <param name="accX">Current value of x axis.</param>
        /// <param name="accY">Current value of y axis.</param>
        /// <param name="accZ">Current value of z axis.</param>
        /// <param name="newDataX">A value indicating whether the new data flag for the x-axis is set or not.</param>
        /// <param name="newDataY">A value indicating whether the new data flag for the y-axis is set or not.</param>
        /// <param name="newDataZ">A value indicating whether the new data flag for the z-axis is set or not.</param>
        public void ReadData(
            out short accX,
            out short accY,
            out short accZ,
            out bool newDataX,
            out bool newDataY,
            out bool newDataZ)
        {
            const int NewDataBit = 0;

            byte[] accData = this.UsbInterface.Read(this.Address, (byte)_3DBS.Registers.AccXAxisLsb, 6, UsbInterface.Module._3DBS);

            accX = _3DBS.ConvertAxisValue(accData, 0, this.currentRange);
            accY = _3DBS.ConvertAxisValue(accData, 2, this.currentRange);
            accZ = _3DBS.ConvertAxisValue(accData, 4, this.currentRange);

            newDataX = BitUtils.GetBit(accData[0], NewDataBit);
            newDataY = BitUtils.GetBit(accData[2], NewDataBit);
            newDataZ = BitUtils.GetBit(accData[4], NewDataBit);
        }

        /// <summary>
        /// Reads current data (new data flags) from the 3D-BS.
        /// </summary>
        /// <param name="newDataX">A value indicating whether the new data flag for the x-axis is set or not.</param>
        /// <param name="newDataY">A value indicating whether the new data flag for the y-axis is set or not.</param>
        /// <param name="newDataZ">A value indicating whether the new data flag for the z-axis is set or not.</param>
        public void ReadData(
            out bool newDataX,
            out bool newDataY,
            out bool newDataZ)
        {
            short accX;
            short accY;
            short accZ;

            this.ReadData(out accX, out accY, out accZ, out newDataX, out newDataY, out newDataZ);
        }

        /// <summary>
        /// Converts read axes values from registers.
        /// </summary>
        /// <param name="accData">Read values.</param>
        /// <param name="startIndex">Start index of the two values of a single axis.</param>
        /// <param name="range">Adjusts converted value to the current range.</param>
        /// <returns>Converted axis value.</returns>
        private static short ConvertAxisValue(byte[] accData, int startIndex, _3DBS.Range range)
        {
            byte msb = accData[startIndex + 1];
            byte lsb = accData[startIndex];

            short result = (short)((msb << 8) ^ lsb);
            result >>= 6;

            /*
            switch (range)
            {
                case Range._2g:
                    result *= 2;
                    break;
                case Range._4g:
                    result *= 4;
                    break;
                case Range._8g:
                    result *= 8;
                    break;
                case Range.NotAuthorisedCode:
                    break;
                default:
                    throw new ArgumentException();
            }
             * */

            return result;
        }

        /// <summary>
        /// Reads operational settings of the 3D-BS.
        /// </summary>
        /// <param name="registers">Read registers.</param>
        /// <returns>Operational settings of the 3D-BS.</returns>
        private _3DBS_Settings.Operational ReadOperationalSettings(byte[] registers)
        {
            byte rangeBandwidthRegister = registers[(byte)_3DBS.Registers.RangeBandwidth];
            byte operationalRegistersFlags = registers[(byte)_3DBS.Registers.OperationalRegistersFlags];

            _3DBS_Settings.Operational operationalSettings = new _3DBS_Settings.Operational();
            if (!BitUtils.GetBit(rangeBandwidthRegister, 4) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 3))
            {
                operationalSettings.Range = Range._2g;
            }
            else if (
                !BitUtils.GetBit(rangeBandwidthRegister, 4) &&
                BitUtils.GetBit(rangeBandwidthRegister, 3))
            {
                operationalSettings.Range = Range._4g;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 4) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 3))
            {
                operationalSettings.Range = Range._8g;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 4) &&
                BitUtils.GetBit(rangeBandwidthRegister, 3))
            {
                operationalSettings.Range = Range.NotAuthorisedCode;
            }
            else
            {
                throw new ArgumentException();
            }

            if (!BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._25Hz;
            }
            else if (
                !BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._50Hz;
            }
            else if (
                !BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._100Hz;
            }
            else if (
                !BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._190Hz;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._375Hz;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._750Hz;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth._1500Hz;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1) &&
                BitUtils.GetBit(rangeBandwidthRegister, 0))
            {
                operationalSettings.Bandwidth = Bandwidth.NotAuthorisedCode;
            }
            else
            {
                throw new ArgumentException();
            }

            operationalSettings.Wake_up = BitUtils.GetBit(operationalRegistersFlags, 0);
            if (!BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1))
            {
                operationalSettings.Wake_up_pause = WakeUpPause._20ms;
            }
            else if (
                !BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1))
            {
                operationalSettings.Wake_up_pause = WakeUpPause._80ms;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                !BitUtils.GetBit(rangeBandwidthRegister, 1))
            {
                operationalSettings.Wake_up_pause = WakeUpPause._320ms;
            }
            else if (
                BitUtils.GetBit(rangeBandwidthRegister, 2) &&
                BitUtils.GetBit(rangeBandwidthRegister, 1))
            {
                operationalSettings.Wake_up_pause = WakeUpPause._2560ms;
            }
            else
            {
                throw new ArgumentException();
            }

            operationalSettings.Shadow_dis = BitUtils.GetBit(operationalRegistersFlags, 3);

            this.currentRange = operationalSettings.Range;
            return operationalSettings;
        }

        /// <summary>
        /// Sets operational settings to the 3D-BS.
        /// </summary>
        /// <param name="settings">Operational settings.</param>
        private void SetOperationalSettings(_3DBS_Settings.Operational settings)
        {
            byte rangeBandwidth = 0x00;
            byte operationalFlags = this.UsbInterface.Read(this.Address, (byte)_3DBS.Registers.OperationalRegistersFlags, UsbInterface.Module._3DBS);

            switch (settings.Range)
            {
                case Range._2g:
                    BitUtils.SetBit(ref rangeBandwidth, 4, false);
                    BitUtils.SetBit(ref rangeBandwidth, 3, false);
                    break;
                case Range._4g:
                    BitUtils.SetBit(ref rangeBandwidth, 4, false);
                    BitUtils.SetBit(ref rangeBandwidth, 3, true);
                    break;
                case Range._8g:
                    BitUtils.SetBit(ref rangeBandwidth, 4, true);
                    BitUtils.SetBit(ref rangeBandwidth, 3, false);
                    break;
                case Range.NotAuthorisedCode:
                    BitUtils.SetBit(ref rangeBandwidth, 4, true);
                    BitUtils.SetBit(ref rangeBandwidth, 3, true);
                    break;
            }

            switch (settings.Bandwidth)
            {
                case Bandwidth._25Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, false);
                    BitUtils.SetBit(ref rangeBandwidth, 1, false);
                    BitUtils.SetBit(ref rangeBandwidth, 0, false);
                    break;
                case Bandwidth._50Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, false);
                    BitUtils.SetBit(ref rangeBandwidth, 1, false);
                    BitUtils.SetBit(ref rangeBandwidth, 0, true);
                    break;
                case Bandwidth._100Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, false);
                    BitUtils.SetBit(ref rangeBandwidth, 1, true);
                    BitUtils.SetBit(ref rangeBandwidth, 0, false);
                    break;
                case Bandwidth._190Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, false);
                    BitUtils.SetBit(ref rangeBandwidth, 1, true);
                    BitUtils.SetBit(ref rangeBandwidth, 0, true);
                    break;
                case Bandwidth._375Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, true);
                    BitUtils.SetBit(ref rangeBandwidth, 1, false);
                    BitUtils.SetBit(ref rangeBandwidth, 0, false);
                    break;
                case Bandwidth._750Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, true);
                    BitUtils.SetBit(ref rangeBandwidth, 1, false);
                    BitUtils.SetBit(ref rangeBandwidth, 0, true);
                    break;
                case Bandwidth._1500Hz:
                    BitUtils.SetBit(ref rangeBandwidth, 2, true);
                    BitUtils.SetBit(ref rangeBandwidth, 1, true);
                    BitUtils.SetBit(ref rangeBandwidth, 0, false);
                    break;
                case Bandwidth.NotAuthorisedCode:
                    BitUtils.SetBit(ref rangeBandwidth, 2, true);
                    BitUtils.SetBit(ref rangeBandwidth, 1, true);
                    BitUtils.SetBit(ref rangeBandwidth, 0, true);
                    break;
            }

            BitUtils.SetBit(ref operationalFlags, 0, settings.Wake_up);
            switch (settings.Wake_up_pause)
            {
                case WakeUpPause._20ms:
                    BitUtils.SetBit(ref operationalFlags, 2, false);
                    BitUtils.SetBit(ref operationalFlags, 1, false);
                    break;
                case WakeUpPause._80ms:
                    BitUtils.SetBit(ref operationalFlags, 2, false);
                    BitUtils.SetBit(ref operationalFlags, 1, true);
                    break;
                case WakeUpPause._320ms:
                    BitUtils.SetBit(ref operationalFlags, 2, true);
                    BitUtils.SetBit(ref operationalFlags, 1, false);
                    break;
                case WakeUpPause._2560ms:
                    BitUtils.SetBit(ref operationalFlags, 2, true);
                    BitUtils.SetBit(ref operationalFlags, 1, true);
                    break;
            }

            BitUtils.SetBit(ref operationalFlags, 3, settings.Shadow_dis);

            this.UsbInterface.Write(this.Address, (byte)_3DBS.Registers.RangeBandwidth, rangeBandwidth, UsbInterface.Module._3DBS);
            this.UsbInterface.Write(this.Address, (byte)_3DBS.Registers.OperationalRegistersFlags, operationalFlags, UsbInterface.Module._3DBS);

            this.currentRange = settings.Range;
        }

        /// <summary>
        /// Reads interrupt settings of the 3D-BS.
        /// </summary>
        /// <param name="registers">Read registers.</param>
        /// <returns>Interrupt settings of the 3D-BS.</returns>
        private _3DBS_Settings.Interrupt ReadInterruptSettings(byte[] registers)
        {
            byte interruptSettingsRegister = registers[(byte)_3DBS.Registers.InterruptSettings];
            byte operationalRegistersFlags = registers[(byte)_3DBS.Registers.OperationalRegistersFlags];

            byte hystAndAnyMotionDurRegister = registers[(byte)_3DBS.Registers.HystAndAnyMotionDur];
            byte thresLg = registers[(byte)_3DBS.Registers.LgThres];
            byte durLg = registers[(byte)_3DBS.Registers.LgDur];
            byte thresHg = registers[(byte)_3DBS.Registers.HgThres];
            byte durHg = registers[(byte)_3DBS.Registers.HgDur];

            byte anyMotionThres = registers[(byte)_3DBS.Registers.AnyMotionThres];

            _3DBS_Settings.Interrupt interruptSettings = new _3DBS_Settings.Interrupt();

            // flags
            interruptSettings.EnableLG = BitUtils.GetBit(interruptSettingsRegister, 0);
            interruptSettings.EnableHG = BitUtils.GetBit(interruptSettingsRegister, 1);
            interruptSettings.EnableAdvINT = BitUtils.GetBit(operationalRegistersFlags, 6);
            interruptSettings.AnyMotion = BitUtils.GetBit(interruptSettingsRegister, 6);
            interruptSettings.Alert = BitUtils.GetBit(interruptSettingsRegister, 7);
            interruptSettings.LatchINT = BitUtils.GetBit(operationalRegistersFlags, 4);

            // lg
            interruptSettings.LGThres = new BitArray(new byte[] { thresLg });
            interruptSettings.LGHyst = new BitArray(new bool[]
            {
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 2),
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 1),
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 0)
            });

            interruptSettings.LGDur = new BitArray(new byte[] { durLg });

            if (!BitUtils.GetBit(interruptSettingsRegister, 3) &&
                !BitUtils.GetBit(interruptSettingsRegister, 2))
            {
                interruptSettings.CounterLG = Counter.Reset;
            }
            else if (
                !BitUtils.GetBit(interruptSettingsRegister, 3) &&
                BitUtils.GetBit(interruptSettingsRegister, 2))
            {
                interruptSettings.CounterLG = Counter.CountDownBy1LsbMs;
            }
            else if (
                !BitUtils.GetBit(interruptSettingsRegister, 3) &&
                BitUtils.GetBit(interruptSettingsRegister, 2))
            {
                interruptSettings.CounterLG = Counter.CountDownBy2LsbMs;
            }
            else if (
                BitUtils.GetBit(interruptSettingsRegister, 3) &&
                BitUtils.GetBit(interruptSettingsRegister, 2))
            {
                interruptSettings.CounterLG = Counter.CountDownBy3LsbMs;
            }
            else
            {
                throw new ArgumentException();
            }

            // hg
            interruptSettings.HGThres = new BitArray(new byte[] { thresHg });
            interruptSettings.HGHyst = new BitArray(new bool[]
            {
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 5),
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 4),
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 3)
            });

            interruptSettings.HGDur = new BitArray(new byte[] { durHg });

            if (!BitUtils.GetBit(interruptSettingsRegister, 5) &&
                !BitUtils.GetBit(interruptSettingsRegister, 4))
            {
                interruptSettings.CounterHG = Counter.Reset;
            }
            else if (
                !BitUtils.GetBit(interruptSettingsRegister, 5) &&
                BitUtils.GetBit(interruptSettingsRegister, 4))
            {
                interruptSettings.CounterHG = Counter.CountDownBy1LsbMs;
            }
            else if (
                !BitUtils.GetBit(interruptSettingsRegister, 5) &&
                BitUtils.GetBit(interruptSettingsRegister, 4))
            {
                interruptSettings.CounterHG = Counter.CountDownBy2LsbMs;
            }
            else if (
                BitUtils.GetBit(interruptSettingsRegister, 5) &&
                BitUtils.GetBit(interruptSettingsRegister, 4))
            {
                interruptSettings.CounterHG = Counter.CountDownBy3LsbMs;
            }
            else
            {
                throw new ArgumentException();
            }

            // any motion
            interruptSettings.AnyMotionThres = new BitArray(new byte[] { anyMotionThres });

            if (!BitUtils.GetBit(hystAndAnyMotionDurRegister, 7) &&
                !BitUtils.GetBit(hystAndAnyMotionDurRegister, 6))
            {
                interruptSettings.AnyMotionDuration = AnyMotionDuration._1;
            }
            else if (
                !BitUtils.GetBit(hystAndAnyMotionDurRegister, 7) &&
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 6))
            {
                interruptSettings.AnyMotionDuration = AnyMotionDuration._3;
            }
            else if (
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 7) &&
                !BitUtils.GetBit(hystAndAnyMotionDurRegister, 6))
            {
                interruptSettings.AnyMotionDuration = AnyMotionDuration._5;
            }
            else if (
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 7) &&
                BitUtils.GetBit(hystAndAnyMotionDurRegister, 6))
            {
                interruptSettings.AnyMotionDuration = AnyMotionDuration._7;
            }
            else
            {
                throw new ArgumentException();
            }

            // New_data_int
            interruptSettings.NewDataInt = BitUtils.GetBit(operationalRegistersFlags, 5);

            return interruptSettings;
        }

        /// <summary>
        /// Reads control settings of the 3D-BS.
        /// </summary>
        /// <param name="registers">Read registers.</param>
        /// <returns>Control settings of the 3D-BS.</returns>
        private _3DBS_Settings.Control ReadControlSettings(byte[] registers)
        {
            byte controlRegister = registers[(byte)_3DBS.Registers.Control];
            _3DBS_Settings.Control controlSettings = new _3DBS_Settings.Control();

            controlSettings.ResetINT = BitUtils.GetBit(controlRegister, 6);
            controlSettings.Selftest0 = BitUtils.GetBit(controlRegister, 2);
            controlSettings.Selftest1 = BitUtils.GetBit(controlRegister, 3);
            controlSettings.SoftReset = BitUtils.GetBit(controlRegister, 1);
            controlSettings.Sleep = BitUtils.GetBit(controlRegister, 0);

            return controlSettings;
        }

        /// <summary>
        /// Reads status settings of the 3D-BS.
        /// </summary>
        /// <param name="registers">Read registers.</param>
        /// <returns>Status settings of the 3D-BS.</returns>
        private _3DBS_Settings.Status ReadStatusSettings(byte[] registers)
        {
            byte statusRegister = registers[(byte)_3DBS.Registers.Status];
            _3DBS_Settings.Status statusSettings = new _3DBS_Settings.Status();

            statusSettings.StResult = BitUtils.GetBit(statusRegister, 7);
            statusSettings.AlertPhase = BitUtils.GetBit(statusRegister, 1);
            statusSettings.LGlatched = BitUtils.GetBit(statusRegister, 3);
            statusSettings.HGlatched = BitUtils.GetBit(statusRegister, 2);
            statusSettings.StatusLG = BitUtils.GetBit(statusRegister, 1);
            statusSettings.StatusHG = BitUtils.GetBit(statusRegister, 0);

            return statusSettings;
        }

        /// <summary>
        /// Reads info settings of the 3D-BS.
        /// </summary>
        /// <param name="registers">Read registers.</param>
        /// <returns>Info settings of the 3D-BS.</returns>
        private _3DBS_Settings.Info ReadInfoSettings(byte[] registers)
        {
            byte versionRegister = registers[(byte)_3DBS.Registers.Version];
            byte chipIDRegister = registers[(byte)_3DBS.Registers.ChipID];
            _3DBS_Settings.Info infoSettings = new _3DBS_Settings.Info();

            infoSettings.AiVersion = new BitArray(new bool[]
            {
                BitUtils.GetBit(versionRegister, 7),
                BitUtils.GetBit(versionRegister, 6),
                BitUtils.GetBit(versionRegister, 5),
                BitUtils.GetBit(versionRegister, 4)
            });

            infoSettings.MlVersion = new BitArray(new bool[]
            {
                BitUtils.GetBit(versionRegister, 3),
                BitUtils.GetBit(versionRegister, 2),
                BitUtils.GetBit(versionRegister, 1),
                BitUtils.GetBit(versionRegister, 0)
            });

            bool[] b = new bool[]
            {
                BitUtils.GetBit(chipIDRegister, 2),
                BitUtils.GetBit(chipIDRegister, 1),
                BitUtils.GetBit(chipIDRegister, 0)
            };

            infoSettings.ChipID = new BitArray(new bool[]
            {
                BitUtils.GetBit(chipIDRegister, 2),
                BitUtils.GetBit(chipIDRegister, 1),
                BitUtils.GetBit(chipIDRegister, 0)
            });

            return infoSettings;
        }
    }
}