﻿//-----------------------------------------------------------------------------
// <copyright file="_6DBS.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper class for communicating with 6D-BS.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using UsbInterfaceTesttool.SerialCommunication;

    /// <summary>
    /// Wrapper class for communicating with 6D-BS.
    /// </summary>
    [Supports(new UsbInterface.Type[] { UsbInterface.Type.I2C, UsbInterface.Type.SPI })]
    public class _6DBS : Device
    {
        /// <summary>
        /// Amount of position registers in order.
        /// </summary>
        public const int AmountOfPositionRegisters = 6;

        /// <summary>
        /// Position of the bit of a register address which indicates whether a read or write operation will be executed.
        /// </summary>
        public const int ReadWriteBit = 7;

        /// <summary>
        /// Position of the bit of a register address which indicates whether one or more bytes will be read/written.
        /// </summary>
        public const int OperationAmountBit = 6;

        /// <summary>
        /// Initializes a new instance of the _6DBS class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public _6DBS(UsbInterface usbInterface)
            : base(usbInterface)
        {
            this.Gyro = new _6DBS_Gyro(usbInterface);
            this.Acc = new _6DBS_Acc(usbInterface);
        }

        /// <summary>
        /// Used delegate for PositionRead-event.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Read position.</param>
        public delegate void PositionReadHandler(object sender, PositionArgs e);

        /// <summary>
        /// Registers of the gyro and acc.
        /// </summary>
        public enum Registers
        {
            /// <summary>
            /// Chip id register.
            /// </summary>
            ChipID = 0x0F,

            /// <summary>
            /// Settings register.
            /// </summary>
            Settings = 0x20,

            /// <summary>
            /// Filter register 1.
            /// </summary>
            FilterSettings1 = 0x21,

            /// <summary>
            /// Scale register.
            /// </summary>
            Scale = 0x23,

            /// <summary>
            /// Filter register 2.
            /// </summary>
            FilterSettings2 = 0x24,

            /// <summary>
            /// Temperature register.
            /// </summary>
            Temperature = 0x26,

            /// <summary>
            /// Status register.
            /// </summary>
            Status = 0x27,

            /// <summary>
            /// Begin of position registers.
            /// </summary>
            Position = 0x28
        }

        /// <summary>
        /// Choice of different Axis for charts.
        /// </summary>
        public enum Axis
        {
            /// <summary>
            /// X-axis of a sensor.
            /// </summary>
            X,

            /// <summary>
            /// Y-axis of a sensor.
            /// </summary>
            Y,

            /// <summary>
            /// Z-axis of a sensor.
            /// </summary>
            Z
        }

        /// <summary>
        /// Choice of filter mode.
        /// </summary>
        public enum FilterMode
        {
            /// <summary>
            /// Normal mode (reset reading HP_RESET_FILTER).
            /// </summary>
            NormalModeWithReset = 0,

            /// <summary>
            /// Reference signal for filtering.
            /// </summary>
            ReferenceSignal = 1,

            /// <summary>
            /// Normal mode.
            /// </summary>
            NormalMode = 2,

            /// <summary>
            /// Autoreset on interrupt event.
            /// </summary>
            AutoresetOnInterruptEvent = 3
        }

        /// <summary>
        /// Big/little endian selection.
        /// </summary>
        public enum EndianSelection
        {
            /// <summary>
            /// Data LSb @ lower address.
            /// </summary>
            DataLsbAtLowerAddress = 0,

            /// <summary>
            /// Data MSb @ lower address.
            /// </summary>
            DataMsbAtLowerAddress = 1,
        }

        /// <summary>
        /// Gyro or acc of the 6D-BS.
        /// </summary>
        public enum Type
        {
            /// <summary>
            /// 6D-BS Gyro is used.
            /// </summary>
            Gyro,

            /// <summary>
            /// 6D-BS Acc is used.
            /// </summary>
            Acc
        }

        /// <summary>
        /// Gets or sets wrapper for communicating with the gyro.
        /// </summary>
        /// <value>Wrapper for communicating with the gyro.</value>
        public _6DBS_Gyro Gyro { get; set; }

        /// <summary>
        /// Gets or sets wrapper for communicating with the acc.
        /// </summary>
        /// <value>Wrapper for communicating with the acc.</value>
        public _6DBS_Acc Acc { get; set; }

        /// <summary>
        /// Reverses the three byte pairs (used for msb at lower address).
        /// </summary>
        /// <param name="bytes">Three byte pairs.</param>
        /// <returns>Reversed byte array.</returns>
        public static byte[] ReversePositionBytes(byte[] bytes)
        {
            if (bytes.Length != _6DBS.AmountOfPositionRegisters)
            {
                throw new ArgumentException();
            }

            List<byte> list = new List<byte>(bytes);
            for (int i = 0; i < list.Count; i += 2)
            {
                list.Reverse(i, 2);
            }

            return list.ToArray();
        }

        /// <summary>
        /// Handles chaning addresses for the 6D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Passed new address.</param>
        public override void AddressChanged(object sender, AddressEvent e)
        {
            if (e.RelevantDevice != this.GetType())
            {
                return;
            }

            this.Acc.Address = e.Address[0];
            this.Gyro.Address = e.Address[1];
        }
    }
}