﻿//-----------------------------------------------------------------------------
// <copyright file="_6DBS_Acc.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper class for communication with the 6D-BS-Acc.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.ComponentModel;
    using System.Threading;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Wrapper class for communication with the 6D-BS-Acc.
    /// </summary>
    public class _6DBS_Acc
    {
        /// <summary>
        /// Used USB-Interface for communication.
        /// </summary>
        private UsbInterface usbInterface;

        /// <summary>
        /// Endian selection of the acc.
        /// </summary>
        private _6DBS.EndianSelection endian;

        /// <summary>
        /// A value indicates whether high resolution mode is enabled or not.
        /// </summary>
        private bool isHighResolutionModeEnabled;

        /// <summary>
        /// A value indicates whether low power mode is enabled or not.
        /// </summary>
        private bool isLowPowerModeEnabled;

        /// <summary>
        /// Initializes a new instance of the _6DBS_Acc class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public _6DBS_Acc(UsbInterface usbInterface)
        {
            this.usbInterface = usbInterface;
        }

        /// <summary>
        /// Triggered when a new position was read.
        /// </summary>
        public event _6DBS.PositionReadHandler PositionRead;

        /// <summary>
        /// Choice of data rate.
        /// </summary>
        public enum DataRate
        {
            /// <summary>
            /// Power-down mode.
            /// </summary>
            PowerDownMode = 0,

            /// <summary>
            /// Normal / Low power mode (1 Hz).
            /// </summary>
            Hz1 = 1,

            /// <summary>
            /// Normal / Low power mode (10 Hz).
            /// </summary>
            Hz10 = 2,

            /// <summary>
            /// Normal / Low power mode (25 Hz).
            /// </summary>
            Hz25 = 3,

            /// <summary>
            /// Normal / Low power mode (50 Hz).
            /// </summary>
            Hz50 = 4,

            /// <summary>
            /// Normal / Low power mode (100 Hz).
            /// </summary>
            Hz100 = 5,

            /// <summary>
            /// Normal / Low power mode (200 Hz).
            /// </summary>
            Hz200 = 6,

            /// <summary>
            /// Normal / Low power mode (400 Hz).
            /// </summary>
            Hz400 = 7,

            /// <summary>
            /// Low power mode (1.620 kHz).
            /// </summary>
            LowPowerMode = 8,

            /// <summary>
            /// Normal (1.344 kHz) / Low power mode (5.376 kHz).
            /// </summary>
            KHz1344 = 9
        }

        /// <summary>
        /// Choice of full scale.
        /// </summary>
        public enum FullScale
        {
            /// <summary>
            /// Full Scale of +/- 2G.
            /// </summary>
            TwoG,

            /// <summary>
            /// Full Scale of +/- 4G.
            /// </summary>
            FourG,

            /// <summary>
            /// Full Scale of +/- 8G.
            /// </summary>
            EightG,

            /// <summary>
            /// Full Scale of +/- 16G.
            /// </summary>
            SixteenG
        }

        /// <summary>
        /// Choice of spi-mode.
        /// </summary>
        public enum SpiMode
        {
            /// <summary>
            /// Three-wire interface.
            /// </summary>
            ThreeWireInterface,

            /// <summary>
            /// Four-wire interface.
            /// </summary>
            FourWireInterface
        }

        /// <summary>
        /// Gets or sets address for the acc.
        /// </summary>
        /// <value>Address for the acc.</value>
        public string Address { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether the low power mode of the acc is enabled or not.
        /// </summary>
        /// <value>A value indicating whether the low power mode of the acc is enabled or not.</value>
        public bool LowPowerModeEnabled { get; set; }

        /// <summary>
        /// Sets the general settings the acc.
        /// </summary>
        /// <param name="dataRate">Chosen data rate.</param>
        /// <param name="enablePowerDownMode">Enable or disable power down mode.</param>
        public void SetGeneralSettings(_6DBS_Acc.DataRate dataRate, bool enablePowerDownMode)
        {
            byte data = 0x00;

            switch (dataRate)
            {
                case DataRate.PowerDownMode:
                    break;

                case DataRate.Hz1:
                    BitUtils.SetBit(ref data, 4, true);
                    break;

                case DataRate.Hz10:
                    BitUtils.SetBit(ref data, 5, true);
                    break;

                case DataRate.Hz25:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 5, true);
                    break;

                case DataRate.Hz50:
                    BitUtils.SetBit(ref data, 6, true);
                    break;

                case DataRate.Hz100:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 6, true);
                    break;

                case DataRate.Hz200:
                    BitUtils.SetBit(ref data, 5, true);
                    BitUtils.SetBit(ref data, 6, true);
                    break;

                case DataRate.Hz400:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 5, true);
                    BitUtils.SetBit(ref data, 6, true);
                    break;

                case DataRate.LowPowerMode:
                    BitUtils.SetBit(ref data, 7, true);
                    break;

                case DataRate.KHz1344:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 7, true);
                    break;
            }

            if (enablePowerDownMode)
            {
                BitUtils.SetBit(ref data, 3, true);
            }

            if (dataRate == DataRate.LowPowerMode || dataRate == DataRate.PowerDownMode || enablePowerDownMode)
            {
                this.LowPowerModeEnabled = true;
            }
            else
            {
                this.LowPowerModeEnabled = false;
            }

            // activate x-, y- and z-axis
            BitUtils.SetBit(ref data, 2, true);
            BitUtils.SetBit(ref data, 1, true);
            BitUtils.SetBit(ref data, 0, true);

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.Settings, data, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Sets the scale options of the acc.
        /// </summary>
        /// <param name="endian">Choice of big/small endian.</param>
        /// <param name="fullScale">Choice of full scale.</param>
        /// <param name="spiMode">Choice of spi-mode.</param>
        /// <param name="highResolutionOutputMode">Enable high resolution output mode.</param>
        public void SetScale(
            _6DBS.EndianSelection endian,
            _6DBS_Acc.FullScale fullScale,
            _6DBS_Acc.SpiMode spiMode,
            bool highResolutionOutputMode)
        {
            byte scale = 0x00;

            switch (endian)
            {
                case _6DBS.EndianSelection.DataLsbAtLowerAddress:
                    this.endian = _6DBS.EndianSelection.DataLsbAtLowerAddress;
                    break;
                case _6DBS.EndianSelection.DataMsbAtLowerAddress:
                    BitUtils.SetBit(ref scale, 6, true);

                    this.endian = _6DBS.EndianSelection.DataMsbAtLowerAddress;
                    break;
            }

            switch (fullScale)
            {
                case _6DBS_Acc.FullScale.TwoG:
                    break;
                case _6DBS_Acc.FullScale.FourG:
                    BitUtils.SetBit(ref scale, 4, true);
                    break;
                case _6DBS_Acc.FullScale.EightG:
                    BitUtils.SetBit(ref scale, 5, true);
                    break;
                case _6DBS_Acc.FullScale.SixteenG:
                    BitUtils.SetBit(ref scale, 4, true);
                    BitUtils.SetBit(ref scale, 5, true);
                    break;
            }

            switch (spiMode)
            {
                case _6DBS_Acc.SpiMode.ThreeWireInterface:
                    BitUtils.SetBit(ref scale, 0, true);
                    break;
                case _6DBS_Acc.SpiMode.FourWireInterface:
                    break;
            }

            this.isHighResolutionModeEnabled = highResolutionOutputMode;

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.Scale, scale, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Sets the filter options of the acc.
        /// </summary>
        /// <param name="filterMode">Chosen filter mode.</param>
        /// <param name="filterCutOffFrequency1">Enable first bit of the filter cut-off frequency.</param>
        /// <param name="filterCutOffFrequency2">Enable second bit of the filter cut-off frequency.</param>
        /// <param name="filteredDataSelection">Enabled = data from internal filter sent to output register and FIFO; Disabled: internal filter bypassed.</param>
        public void SetFilter(
            _6DBS.FilterMode filterMode,
            bool filterCutOffFrequency1,
            bool filterCutOffFrequency2,
            bool filteredDataSelection)
        {
            byte filterByte = 0x00;

            switch (filterMode)
            {
                case _6DBS.FilterMode.NormalModeWithReset:
                    break;
                case _6DBS.FilterMode.ReferenceSignal:
                    BitUtils.SetBit(ref filterByte, 6, true);
                    break;
                case _6DBS.FilterMode.NormalMode:
                    BitUtils.SetBit(ref filterByte, 7, true);
                    break;
                case _6DBS.FilterMode.AutoresetOnInterruptEvent:
                    BitUtils.SetBit(ref filterByte, 6, true);
                    BitUtils.SetBit(ref filterByte, 7, true);
                    break;
            }

            if (filterCutOffFrequency1)
            {
                BitUtils.SetBit(ref filterByte, 5, true);
            }

            if (filterCutOffFrequency2)
            {
                BitUtils.SetBit(ref filterByte, 4, true);
            }

            if (filteredDataSelection)
            {
                BitUtils.SetBit(ref filterByte, 3, true);
            }

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.FilterSettings1, filterByte, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Gets the basic settings from the acc.
        /// </summary>
        /// <param name="dataRate">Read data rate.</param>
        /// <param name="enableLowPowerMode">Read flag for enabling power down mode.</param>
        public void GetGeneralSettings(
            out DataRate dataRate,
            out bool enableLowPowerMode)
        {
            byte settings = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Settings, UsbInterface.Module._6DBS);

            if (!BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                !BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.PowerDownMode;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz1;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                BitUtils.GetBit(settings, 5) &&
                !BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz10;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                BitUtils.GetBit(settings, 5) &&
                BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz25;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                !BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz50;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz100;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                BitUtils.GetBit(settings, 6) &&
                BitUtils.GetBit(settings, 5) &&
                !BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz200;
            }
            else if (!BitUtils.GetBit(settings, 7) &&
                BitUtils.GetBit(settings, 6) &&
                BitUtils.GetBit(settings, 5) &&
                BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.Hz400;
            }
            else if (BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                !BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.LowPowerMode;
            }
            else if (BitUtils.GetBit(settings, 7) &&
                !BitUtils.GetBit(settings, 6) &&
                !BitUtils.GetBit(settings, 5) &&
                BitUtils.GetBit(settings, 4))
            {
                dataRate = DataRate.KHz1344;
            }
            else
            {
                throw new ArgumentException();
            }

            if (BitUtils.GetBit(settings, 3))
            {
                enableLowPowerMode = true;
            }
            else
            {
                enableLowPowerMode = false;
            }

            this.isLowPowerModeEnabled = enableLowPowerMode;
        }

        /// <summary>
        /// Gets the scale from the acc.
        /// </summary>
        /// <param name="endian">Read endian selection.</param>
        /// <param name="fullScale">Read full scale.</param>
        /// <param name="highResolutionOutputMode">Read flag for enabling high resolution mode.</param>
        /// <param name="spiMode">Read spi mode.</param>
        public void GetScale(
            out _6DBS.EndianSelection endian,
            out _6DBS_Acc.FullScale fullScale,
            out bool highResolutionOutputMode,
            out _6DBS_Acc.SpiMode spiMode)
        {
            byte scale = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Scale, UsbInterface.Module._6DBS);

            if (BitUtils.GetBit(scale, 6))
            {
                endian = _6DBS.EndianSelection.DataMsbAtLowerAddress;
            }
            else
            {
                endian = _6DBS.EndianSelection.DataLsbAtLowerAddress;
            }

            if (!BitUtils.GetBit(scale, 5) && !BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale.TwoG;
            }
            else if (!BitUtils.GetBit(scale, 5) && BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale.FourG;
            }
            else if (BitUtils.GetBit(scale, 5) && !BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale.EightG;
            }
            else if (BitUtils.GetBit(scale, 5) && BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale.SixteenG;
            }
            else
            {
                throw new ArgumentException();
            }

            highResolutionOutputMode = BitUtils.GetBit(scale, 3);
            this.isHighResolutionModeEnabled = highResolutionOutputMode;

            spiMode = BitUtils.GetBit(scale, 0) ? SpiMode.ThreeWireInterface : SpiMode.FourWireInterface;
        }

        /// <summary>
        /// Gets the filter from the acc.
        /// </summary>
        /// <param name="filterMode">Read filter mode.</param>
        /// <param name="cutOffFrequency1">Read cut-off frequency (first bit).</param>
        /// <param name="cutOffFrequency2">Read cut-off frequency (second bit).</param>
        /// <param name="filteredDataSelection">Read filtered data selection.</param>
        public void GetFilter(
            out _6DBS.FilterMode filterMode,
            out bool cutOffFrequency1,
            out bool cutOffFrequency2,
            out bool filteredDataSelection)
        {
            byte filter = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.FilterSettings1, UsbInterface.Module._6DBS);

            if (!BitUtils.GetBit(filter, 7) && !BitUtils.GetBit(filter, 6))
            {
                filterMode = _6DBS.FilterMode.NormalModeWithReset;
            }
            else if (!BitUtils.GetBit(filter, 7) && BitUtils.GetBit(filter, 6))
            {
                filterMode = _6DBS.FilterMode.ReferenceSignal;
            }
            else if (BitUtils.GetBit(filter, 7) && !BitUtils.GetBit(filter, 6))
            {
                filterMode = _6DBS.FilterMode.NormalMode;
            }
            else if (BitUtils.GetBit(filter, 7) && BitUtils.GetBit(filter, 6))
            {
                filterMode = _6DBS.FilterMode.AutoresetOnInterruptEvent;
            }
            else
            {
                throw new ArgumentException();
            }

            cutOffFrequency2 = BitUtils.GetBit(filter, 5);
            cutOffFrequency1 = BitUtils.GetBit(filter, 4) ? true : false;
            filteredDataSelection = BitUtils.GetBit(filter, 3) ? true : false;
        }

        /// <summary>
        /// Gets the current status of the acc.
        /// </summary>
        /// <param name="overrunX">Overrun flag for x is set.</param>
        /// <param name="overrunY">Overrun flag for y is set.</param>
        /// <param name="overrunZ">Overrun flag for z is set.</param>
        /// <param name="newDataX">New data flag for x is set.</param>
        /// <param name="newDataY">New data flag for y is set.</param>
        /// <param name="newDataZ">New data flag for z is set.</param>
        public void GetStatus(
            out bool overrunX,
            out bool overrunY,
            out bool overrunZ,
            out bool newDataX,
            out bool newDataY,
            out bool newDataZ)
        {
            byte status = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Status, UsbInterface.Module._6DBS);

            overrunX = BitUtils.GetBit(status, 6);
            overrunY = BitUtils.GetBit(status, 5);
            overrunZ = BitUtils.GetBit(status, 4);

            newDataX = BitUtils.GetBit(status, 2);
            newDataY = BitUtils.GetBit(status, 1);
            newDataZ = BitUtils.GetBit(status, 0);
        }

        /// <summary>
        /// Reads the current position of the acc.
        /// </summary>
        public void ReadCurrentPosition()
        {
            byte[] bytes;
            try
            {
                bytes = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Position, _6DBS.AmountOfPositionRegisters, UsbInterface.Module._6DBS);

                if (bytes.Length != _6DBS.AmountOfPositionRegisters)
                {
                    throw new Exception();
                }
            }
            catch (Exception)
            {
                throw new InvalidOperationException("Die Position des Accs konnte nicht ausgelesen werden.");
            }

            if (this.endian == _6DBS.EndianSelection.DataMsbAtLowerAddress)
            {
                bytes = _6DBS.ReversePositionBytes(bytes);
            }

            if (this.PositionRead != null)
            {
                int x, y, z;
                x = this.ScaleReadValue(BitConverter.ToInt16(bytes, 0));
                y = this.ScaleReadValue(BitConverter.ToInt16(bytes, 2));
                z = this.ScaleReadValue(BitConverter.ToInt16(bytes, 4));

                this.PositionRead.BeginInvoke(this, new PositionArgs(x, y, z), null, null);
            }
        }

        /// <summary>
        /// Scales read value according to the scale settings.
        /// </summary>
        /// <param name="readValue">Read value to scale.</param>
        /// <returns>Scales value.</returns>
        private int ScaleReadValue(short readValue)
        {
            sbyte highByte = (sbyte)(readValue / 256);
            sbyte lowByte = (sbyte)(readValue % 256);
            if (!this.isLowPowerModeEnabled && this.isHighResolutionModeEnabled)
            {
                return ((highByte * 256) + lowByte) >> 4;
            }
            else if (!this.isLowPowerModeEnabled && !this.isHighResolutionModeEnabled)
            {
                return ((highByte * 256) + lowByte) >> 6;
            }
            else
            {
                return highByte;
            }
        }
    }
}
