﻿//-----------------------------------------------------------------------------
// <copyright file="_6DBS_Gyro.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper class for communication with the 6D-BS-Gyro.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.ComponentModel;
    using System.Threading;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Wrapper class for communication with the 6D-BS-Gyro.
    /// </summary>
    public class _6DBS_Gyro
    {
        /// <summary>
        /// Used USB-Interface for communication.
        /// </summary>
        private UsbInterface usbInterface;

        /// <summary>
        /// Used endian for position reading.
        /// </summary>
        private _6DBS.EndianSelection endian;

        /// <summary>
        /// Initializes a new instance of the _6DBS_Gyro class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public _6DBS_Gyro(UsbInterface usbInterface)
        {
            this.usbInterface = usbInterface;

            this.IsLowPowerModeEnabled = false;
        }

        /// <summary>
        /// Triggered when a new position was read.
        /// </summary>
        public event _6DBS.PositionReadHandler PositionRead;

        /// <summary>
        /// Choice of mode.
        /// </summary>
        public enum Mode
        {
            /// <summary>
            /// Normal mode or Sleep mode.
            /// </summary>
            NormalOrSleepMode,

            /// <summary>
            /// Power-down mode.
            /// </summary>
            PowerDownMode
        }

        /// <summary>
        /// Choice of block data update.
        /// </summary>
        public enum BlockDataUpdate
        {
            /// <summary>
            /// Continuous update.
            /// </summary>
            ContinuousUpdate,

            /// <summary>
            /// Output registers not updated until MSb and LSb reading.
            /// </summary>
            OutputRegistersNotUpdatedUntilMSbAndLSbReading
        }

        /// <summary>
        /// Choice of full scale.
        /// </summary>
        public enum FullScale
        {
            /// <summary>
            /// Scale of 250 dps.
            /// </summary>
            _250dps = 0,

            /// <summary>
            /// Scale of 500 dps.
            /// </summary>
            _500dps = 1,

            /// <summary>
            /// Scale of 1000 dps.
            /// </summary>
            _1000dps = 3,

            /// <summary>
            /// Scale of 2000 dps.
            /// </summary>
            _2000dps = 2
        }

        /// <summary>
        /// Choice of GyroSim.
        /// </summary>
        public enum Sim
        {
            /// <summary>
            /// Three-wire SPI Serial interface read mode diabled.
            /// </summary>
            ThreeWireReadModeDisabled = 0,

            /// <summary>
            /// Three-wire SPI Serial interface read mode enabled.
            /// </summary>
            ThreeWireReadModeEnabled = 1,
        }

        /// <summary>
        /// List of possible data filter.
        /// </summary>
        public enum DataFilter
        {
            /// <summary>
            /// Normal: no additional filter activated.
            /// </summary>
            Normal,

            /// <summary>
            /// Use filter option hpf.
            /// </summary>
            Hpf,

            /// <summary>
            /// Use filter option lpf.
            /// </summary>
            Lpf2,

            /// <summary>
            /// Use filter option hpf and lpf.
            /// </summary>
            HpfLpf2
        }

        /// <summary>
        /// Choice of ODR.
        /// </summary>
        public enum Odr
        {
            /// <summary>
            /// Frequence of 95 Hz.
            /// </summary>
            _95 = 0,

            /// <summary>
            /// Frequence of 190 Hz.
            /// </summary>
            _190 = 1,

            /// <summary>
            /// Frequence of 380 Hz.
            /// </summary>
            _380 = 2,

            /// <summary>
            /// Frequence of 760 Hz.
            /// </summary>
            _760 = 3
        }

        /// <summary>
        /// Gets or sets address for the acc.
        /// </summary>
        /// <value>Address for the acc.</value>
        public string Address { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether low power mode is enabled or not.
        /// </summary>
        /// <value>A value indicating whether low power mode is enabled or not.</value>
        public bool IsLowPowerModeEnabled { get; set; }

        /// <summary>
        /// Sets the general settings of the angular rate sensor.
        /// </summary>
        /// <param name="odr">Choice of odr.</param>
        /// <param name="cutOff">Choice of cut-off frequency.</param>
        /// <param name="enablePowerDownMode">Choice of mode.</param>
        public void SetSettings(
            _6DBS_Gyro.Odr odr,
            int cutOff,
            bool enablePowerDownMode)
        {
            byte data = 0x00;

            switch (odr)
            {
                case Odr._95:
                    break;
                case Odr._190:
                    BitUtils.SetBit(ref data, 6, true);
                    break;
                case Odr._380:
                    BitUtils.SetBit(ref data, 7, true);
                    break;
                case Odr._760:
                    BitUtils.SetBit(ref data, 6, true);
                    BitUtils.SetBit(ref data, 7, true);
                    break;
            }

            switch (cutOff)
            {
                case 0:
                    break;
                case 1:
                    BitUtils.SetBit(ref data, 4, true);
                    break;
                case 2:
                    BitUtils.SetBit(ref data, 5, true);
                    break;
                case 3:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 5, true);
                    break;
            }

            if (!enablePowerDownMode)
            {
                BitUtils.SetBit(ref data, 3, true);
            }

            this.IsLowPowerModeEnabled = enablePowerDownMode;

            // activate x-, y- and z-axis
            BitUtils.SetBit(ref data, 2, true);
            BitUtils.SetBit(ref data, 1, true);
            BitUtils.SetBit(ref data, 0, true);

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.Settings, data, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Sets the filter options of the angular rate sensor.
        /// </summary>
        /// <param name="isEnabledEdgeSensitiveTrigger">Enable edge-sensitive trigger.</param>
        /// <param name="isEnabledLevelSensitiveTrigger">Enable level-sensitive trigger.</param>
        /// <param name="filterMode">Choice of filter-mode.</param>
        /// <param name="cutOffFrequencySelectionIndex">Index of cut-off frequency.</param>
        /// <param name="dataFilter">Data filter choice.</param>
        public void SetFilter(
            bool isEnabledEdgeSensitiveTrigger,
            bool isEnabledLevelSensitiveTrigger,
            _6DBS.FilterMode filterMode,
            int cutOffFrequencySelectionIndex,
            _6DBS_Gyro.DataFilter dataFilter)
        {
            byte data = 0x00;
            if (isEnabledEdgeSensitiveTrigger)
            {
                BitUtils.SetBit(ref data, 7, true);
            }

            if (isEnabledLevelSensitiveTrigger)
            {
                BitUtils.SetBit(ref data, 6, true);
            }

            switch (filterMode)
            {
                case _6DBS.FilterMode.NormalModeWithReset:
                    break;
                case _6DBS.FilterMode.ReferenceSignal:
                    BitUtils.SetBit(ref data, 4, true);
                    break;
                case _6DBS.FilterMode.NormalMode:
                    BitUtils.SetBit(ref data, 5, true);
                    break;
                case _6DBS.FilterMode.AutoresetOnInterruptEvent:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 5, true);
                    break;
            }

            switch (cutOffFrequencySelectionIndex)
            {
                case 0:
                    break;
                case 1:
                    BitUtils.SetBit(ref data, 0, true);
                    break;
                case 2:
                    BitUtils.SetBit(ref data, 1, true);
                    break;
                case 3:
                    BitUtils.SetBit(ref data, 0, true);
                    BitUtils.SetBit(ref data, 1, true);
                    break;
                case 4:
                    BitUtils.SetBit(ref data, 2, true);
                    break;
                case 5:
                    BitUtils.SetBit(ref data, 0, true);
                    BitUtils.SetBit(ref data, 2, true);
                    break;
                case 6:
                    BitUtils.SetBit(ref data, 1, true);
                    BitUtils.SetBit(ref data, 2, true);
                    break;
                case 7:
                    BitUtils.SetBit(ref data, 0, true);
                    BitUtils.SetBit(ref data, 1, true);
                    BitUtils.SetBit(ref data, 2, true);
                    break;
                case 8:
                    BitUtils.SetBit(ref data, 3, true);
                    break;
                case 9:
                    BitUtils.SetBit(ref data, 0, true);
                    BitUtils.SetBit(ref data, 3, true);
                    break;
            }

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.FilterSettings1, data, UsbInterface.Module._6DBS);

            data = 0x00;
            byte formerValue = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.FilterSettings2, UsbInterface.Module._6DBS);
            if (BitUtils.GetBit(formerValue, 2))
            {
                BitUtils.SetBit(ref data, 2, true);
            }

            if (BitUtils.GetBit(formerValue, 3))
            {
                BitUtils.SetBit(ref data, 3, true);
            }

            if (BitUtils.GetBit(formerValue, 6))
            {
                BitUtils.SetBit(ref data, 6, true);
            }

            if (BitUtils.GetBit(formerValue, 7))
            {
                BitUtils.SetBit(ref data, 7, true);
            }

            switch (dataFilter)
            {
                case DataFilter.Normal:
                    break;
                case DataFilter.Hpf:
                    BitUtils.SetBit(ref data, 0, true);
                    break;
                case DataFilter.Lpf2:
                    BitUtils.SetBit(ref data, 1, true);
                    break;
                case DataFilter.HpfLpf2:
                    BitUtils.SetBit(ref data, 1, true);
                    BitUtils.SetBit(ref data, 5, true);
                    break;
            }

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.FilterSettings2, data, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Sets the scale options of the angular rate sensor.
        /// </summary>
        /// <param name="blockDataUpdate">Choice of block data update.</param>
        /// <param name="endianSelection">Choice of endian selection.</param>
        /// <param name="fullScale">Choice of full scale.</param>
        /// <param name="spiMode">Choice of spi-mode.</param>
        public void SetScale(
            _6DBS_Gyro.BlockDataUpdate blockDataUpdate,
            _6DBS.EndianSelection endianSelection,
            _6DBS_Gyro.FullScale fullScale,
            _6DBS_Gyro.Sim spiMode)
        {
            byte data = 0x00;

            switch (blockDataUpdate)
            {
                case BlockDataUpdate.ContinuousUpdate:
                    break;
                case BlockDataUpdate.OutputRegistersNotUpdatedUntilMSbAndLSbReading:
                    BitUtils.SetBit(ref data, 7, true);
                    break;
            }

            switch (endianSelection)
            {
                case _6DBS.EndianSelection.DataLsbAtLowerAddress:
                    this.endian = _6DBS.EndianSelection.DataLsbAtLowerAddress;
                    break;
                case _6DBS.EndianSelection.DataMsbAtLowerAddress:
                    BitUtils.SetBit(ref data, 6, true);

                    this.endian = _6DBS.EndianSelection.DataMsbAtLowerAddress;
                    break;
            }

            switch (fullScale)
            {
                case FullScale._250dps:
                    break;
                case FullScale._500dps:
                    BitUtils.SetBit(ref data, 4, true);
                    break;
                case FullScale._1000dps:
                    BitUtils.SetBit(ref data, 5, true);
                    break;
                case FullScale._2000dps:
                    BitUtils.SetBit(ref data, 4, true);
                    BitUtils.SetBit(ref data, 5, true);
                    break;
            }

            switch (spiMode)
            {
                case _6DBS_Gyro.Sim.ThreeWireReadModeDisabled:
                    break;
                case _6DBS_Gyro.Sim.ThreeWireReadModeEnabled:
                    BitUtils.SetBit(ref data, 0, true);
                    break;
            }

            this.usbInterface.Write(this.Address, (byte)_6DBS.Registers.Scale, data, UsbInterface.Module._6DBS);
        }

        /// <summary>
        /// Gets the basic settings from the device.
        /// </summary>
        /// <param name="odr">Read odr from the device.</param>
        /// <param name="bandwithIndex">Read bandwith.</param>
        /// <param name="enablePowerDownMode">Read flag for enabling power down mode.</param>
        public void GetGeneralSettings(
            out _6DBS_Gyro.Odr odr,
            out int bandwithIndex,
            out bool enablePowerDownMode)
        {
            byte generalSettings = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Settings, UsbInterface.Module._6DBS);

            if (!BitUtils.GetBit(generalSettings, 7) && !BitUtils.GetBit(generalSettings, 6))
            {
                odr = Odr._95;
            }
            else if (!BitUtils.GetBit(generalSettings, 7) && BitUtils.GetBit(generalSettings, 6))
            {
                odr = Odr._190;
            }
            else if (BitUtils.GetBit(generalSettings, 7) && !BitUtils.GetBit(generalSettings, 6))
            {
                odr = Odr._380;
            }
            else if (BitUtils.GetBit(generalSettings, 7) && BitUtils.GetBit(generalSettings, 6))
            {
                odr = Odr._760;
            }
            else
            {
                throw new ArgumentException();
            }

            if (!BitUtils.GetBit(generalSettings, 5) && !BitUtils.GetBit(generalSettings, 6))
            {
                bandwithIndex = 0;
            }
            else if (!BitUtils.GetBit(generalSettings, 5) && BitUtils.GetBit(generalSettings, 6))
            {
                bandwithIndex = 1;
            }
            else if (BitUtils.GetBit(generalSettings, 5) && !BitUtils.GetBit(generalSettings, 6))
            {
                bandwithIndex = 2;
            }
            else if (BitUtils.GetBit(generalSettings, 5) && BitUtils.GetBit(generalSettings, 6))
            {
                bandwithIndex = 3;
            }
            else
            {
                throw new ArgumentException();
            }

            enablePowerDownMode = !BitUtils.GetBit(generalSettings, 3);
        }

        /// <summary>
        /// Gets the scale from the device.
        /// </summary>
        /// <param name="endian">Read endian.</param>
        /// <param name="fullScale">Read full scale option.</param>
        /// <param name="blockDataUpdate">Read block data update option.</param>
        /// <param name="sim">Read sim from the device.</param>
        public void GetScale(
            out _6DBS.EndianSelection endian,
            out _6DBS_Gyro.FullScale fullScale,
            out _6DBS_Gyro.BlockDataUpdate blockDataUpdate,
            out _6DBS_Gyro.Sim sim)
        {
            byte scale = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Scale, UsbInterface.Module._6DBS);

            if (BitUtils.GetBit(scale, 7))
            {
                blockDataUpdate = _6DBS_Gyro.BlockDataUpdate.OutputRegistersNotUpdatedUntilMSbAndLSbReading;
            }
            else
            {
                blockDataUpdate = _6DBS_Gyro.BlockDataUpdate.ContinuousUpdate;
            }

            if (BitUtils.GetBit(scale, 6))
            {
                endian = _6DBS.EndianSelection.DataMsbAtLowerAddress;
            }
            else
            {
                endian = _6DBS.EndianSelection.DataLsbAtLowerAddress;
            }

            if (!BitUtils.GetBit(scale, 5) && !BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale._250dps;
            }
            else if (!BitUtils.GetBit(scale, 5) && BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale._500dps;
            }
            else if (BitUtils.GetBit(scale, 5) && !BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale._1000dps;
            }
            else if (BitUtils.GetBit(scale, 5) && BitUtils.GetBit(scale, 4))
            {
                fullScale = FullScale._2000dps;
            }
            else
            {
                throw new ArgumentException();
            }

            if (BitUtils.GetBit(scale, 0))
            {
                sim = Sim.ThreeWireReadModeEnabled;
            }
            else
            {
                sim = Sim.ThreeWireReadModeDisabled;
            }
        }

        /// <summary>
        /// Gets the filter from the device.
        /// </summary>
        /// <param name="edgeSensitiveTrigger">Read flag for enabling edge sensitive trigger.</param>
        /// <param name="levelSensitiveTrigger">Read flag for enabling level sensitive trigger.</param>
        /// <param name="modeOfHp">Read hp mode.</param>
        /// <param name="cutOffFrequencyIndex">Read cut-off frequency.</param>
        /// <param name="dataFilter">Read data filter.</param>
        public void GetFilter(
            out bool edgeSensitiveTrigger,
            out bool levelSensitiveTrigger,
            out _6DBS.FilterMode modeOfHp,
            out int cutOffFrequencyIndex,
            out DataFilter dataFilter)
        {
            byte filterSettings1 = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.FilterSettings1, UsbInterface.Module._6DBS);
            byte filterSettings2 = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.FilterSettings2, UsbInterface.Module._6DBS);

            if (BitUtils.GetBit(filterSettings1, 7))
            {
                edgeSensitiveTrigger = true;
            }
            else
            {
                edgeSensitiveTrigger = false;
            }

            if (BitUtils.GetBit(filterSettings1, 6))
            {
                levelSensitiveTrigger = true;
            }
            else
            {
                levelSensitiveTrigger = false;
            }

            if (!BitUtils.GetBit(filterSettings1, 5) && !BitUtils.GetBit(filterSettings1, 4))
            {
                modeOfHp = _6DBS.FilterMode.NormalModeWithReset;
            }
            else if (!BitUtils.GetBit(filterSettings1, 5) && !BitUtils.GetBit(filterSettings1, 4))
            {
                modeOfHp = _6DBS.FilterMode.ReferenceSignal;
            }
            else if (!BitUtils.GetBit(filterSettings1, 5) && !BitUtils.GetBit(filterSettings1, 4))
            {
                modeOfHp = _6DBS.FilterMode.NormalMode;
            }
            else if (!BitUtils.GetBit(filterSettings1, 5) && !BitUtils.GetBit(filterSettings1, 4))
            {
                modeOfHp = _6DBS.FilterMode.AutoresetOnInterruptEvent;
            }
            else
            {
                throw new ArgumentException();
            }

            if (!BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 0;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 1;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 2;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 3;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 4;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 5;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                BitUtils.GetBit(filterSettings1, 2) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 6;
            }
            else if (!BitUtils.GetBit(filterSettings1, 3) &&
                BitUtils.GetBit(filterSettings1, 2) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 7;
            }
            else if (BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 8;
            }
            else if (BitUtils.GetBit(filterSettings1, 3) &&
                !BitUtils.GetBit(filterSettings1, 2) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                cutOffFrequencyIndex = 9;
            }
            else
            {
                throw new ArgumentException();
            }

            if (!BitUtils.GetBit(filterSettings1, 5) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                dataFilter = DataFilter.Normal;
            }
            else if (!BitUtils.GetBit(filterSettings1, 5) &&
                !BitUtils.GetBit(filterSettings1, 1) &&
                BitUtils.GetBit(filterSettings1, 0))
            {
                dataFilter = DataFilter.Hpf;
            }
            else if (!BitUtils.GetBit(filterSettings1, 5) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                dataFilter = DataFilter.Lpf2;
            }
            else if (BitUtils.GetBit(filterSettings1, 5) &&
                BitUtils.GetBit(filterSettings1, 1) &&
                !BitUtils.GetBit(filterSettings1, 0))
            {
                dataFilter = DataFilter.HpfLpf2;
            }
            else
            {
                throw new ArgumentException();
            }
        }

        /// <summary>
        /// Gets the Chip-ID and status of the gyro.
        /// </summary>
        /// <param name="chipID">Chip-ID of the gyro.</param>
        /// <param name="overrunX">X overrun.</param>
        /// <param name="overrunY">Y overrun.</param>
        /// <param name="overrunZ">Z overrun.</param>
        /// <param name="newDataX">X new data.</param>
        /// <param name="newDataY">Y new data.</param>
        /// <param name="newDataZ">Z new data.</param>
        public void GetStatus(
            out byte chipID,
            out bool overrunX,
            out bool overrunY,
            out bool overrunZ,
            out bool newDataX,
            out bool newDataY,
            out bool newDataZ)
        {
            chipID = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.ChipID, UsbInterface.Module._6DBS);

            byte status = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Status, UsbInterface.Module._6DBS);
            overrunZ = BitUtils.GetBit(status, 6);
            overrunY = BitUtils.GetBit(status, 5);
            overrunX = BitUtils.GetBit(status, 4);
            newDataZ = BitUtils.GetBit(status, 2);
            newDataY = BitUtils.GetBit(status, 1);
            newDataX = BitUtils.GetBit(status, 0);
        }

        /// <summary>
        /// Reads the current position of the acc.
        /// </summary>
        public void ReadCurrentPosition()
        {
            byte[] bytes;
            try
            {
                bytes = this.usbInterface.Read(this.Address, (byte)_6DBS.Registers.Position, _6DBS.AmountOfPositionRegisters, UsbInterface.Module._6DBS);

                if (bytes.Length != _6DBS.AmountOfPositionRegisters)
                {
                    throw new Exception();
                }
            }
            catch
            {
                throw new InvalidOperationException("Die Position des Gyros konnte nicht ausgelesen werden.");
            }

            if (this.endian == _6DBS.EndianSelection.DataMsbAtLowerAddress)
            {
                bytes = _6DBS.ReversePositionBytes(bytes);
            }

            if (this.PositionRead != null)
            {
                int x, y, z;
                x = BitConverter.ToInt16(bytes, 0);
                y = BitConverter.ToInt16(bytes, 2);
                z = BitConverter.ToInt16(bytes, 4);

                this.PositionRead.BeginInvoke(this, new PositionArgs(x, y, z), null, null);
            }
        }
    }
}