﻿//-----------------------------------------------------------------------------
// <copyright file="_RTC.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper class for communication with the RTC.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Devices
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Wrapper class for communication with the RTC.
    /// </summary>
    [Supports(UsbInterface.Type.SPI)]
    public class _RTC : Device
    {
        /// <summary>
        /// Amount of bytes used for the time.
        /// </summary>
        public const int TimeBytesAmount = 7;

        /// <summary>
        /// Bit of a register which indicates whether address burst (auto increment) should be enabled or not.
        /// </summary>
        public const int SingleBurstBit = 7;

        /// <summary>
        /// Bit of a register which indicates whether a read or write operation should be done.
        /// </summary>
        public const int ReadWriteBit = 6;

        /// <summary>
        /// Initializes a new instance of the _RTC class.
        /// </summary>
        /// <param name="usbInterface">Used USB-Interface for communication.</param>
        public _RTC(UsbInterface usbInterface)
            : base(usbInterface)
        {
        }

        /// <summary>
        /// Registers of the RTC-DCF.
        /// </summary>
        public enum Registers
        {
            /// <summary>
            /// 0x00 - 0x06 (_RTC.TimeBytesAmount) represent the saved time.
            /// </summary>
            CurrentTime = 0x00,

            /// <summary>
            /// Minute value of saved alarm.
            /// </summary>
            AlarmMinute = 0x07,

            /// <summary>
            /// Hour value of saved alarm.
            /// </summary>
            AlarmHour = 0x08,

            /// <summary>
            /// Weekday value of saved alarm.
            /// </summary>
            AlarmWeekdays = 0x09,

            /// <summary>
            /// Saved periodical interrupt.
            /// </summary>
            PeriodicalInterrupt = 0x0A,

            /// <summary>
            /// Saved alarm config.
            /// </summary>
            AlarmConfig = 0x0B,

            /// <summary>
            /// Saved periodical interrupt config.
            /// </summary>
            PeriodicalInterruptConfig = 0x0C,

            /// <summary>
            /// Saved DCF config.
            /// </summary>
            DcfConfig = 0x0D,

            /// <summary>
            /// Saved status.
            /// </summary>
            Status = 0x0E
        }

        /// <summary>
        /// Different possible settings for the periodical interrupt frequency.
        /// </summary>
        public enum PeriodicalInterruptFrequency
        {
            /// <summary>
            /// Periodical interrupt with 1 Hz.
            /// </summary>
            _1Hz = 0x00,

            /// <summary>
            /// Periodical interrupt with 2 Hz.
            /// </summary>
            _2Hz = 0x01,

            /// <summary>
            /// Periodical interrupt each second.
            /// </summary>
            EachSecond = 0x02,

            /// <summary>
            /// Periodical interrupt each minute.
            /// </summary>
            EachMinute = 0x03,

            /// <summary>
            /// Periodical interrupt each hour.
            /// </summary>
            EachHour = 0x04,

            /// <summary>
            /// Periodical interrupt each day.
            /// </summary>
            EachDay = 0x05,

            /// <summary>
            /// Periodical interrupt each first of a month.
            /// </summary>
            EachMonth = 0x06
        }

        /// <summary>
        /// Adjusts a four digit year to a two digit year (f.e. 1992 => 92, 2013 => 13).
        /// </summary>
        /// <param name="time">DateTime to adjust.</param>
        /// <returns>Adjusted DateTime.</returns>
        public static DateTime GetAdjustedYear(DateTime time)
        {
            if (time.Year > 80)
            {
                return time.AddYears(1900);
            }
            else
            {
                return time.AddYears(2000);
            }
        }

        /// <summary>
        /// Handles chaning addresses for the RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        public override void AddressChanged(object sender, AddressEvent e)
        {
            if (e.RelevantDevice != this.GetType())
            {
                return;
            }

            this.Address = e.GetFirstAddress();
        }

        /// <summary>
        /// Reads the current time from the RTC.
        /// </summary>
        /// <returns>Read time.</returns>
        public DateTime ReadTime()
        {
            byte[] timeBytes = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.CurrentTime, _RTC.TimeBytesAmount, UsbInterface.Module._RTC);
            DateTime time = BcdUtils.ConvertBytesToDateTime(timeBytes);

            return _RTC.GetAdjustedYear(time);
        }

        /// <summary>
        /// Sets a specfic time on the RTC.
        /// </summary>
        /// <param name="time">Time to save.</param>
        public void SetTime(DateTime time)
        {
            byte[] timeBytes = BcdUtils.ConvertDateTimeToBytes(time);
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.CurrentTime, timeBytes, UsbInterface.Module._RTC);
        }

        /// <summary>
        /// Read and resets interrupt flags of the RTC.
        /// </summary>
        /// <param name="interruptFlagAIF">AIF-flag to set.</param>
        /// <param name="interruptFlagPIF">PIF-flag to set.</param>
        /// <param name="interruptFlagDCFIF">DCFIF-flag to set.</param>
        public void ReadAndResetInterruptFlags(
            out bool interruptFlagAIF,
            out bool interruptFlagPIF,
            out bool interruptFlagDCFIF)
        {
            const byte StatusResetValue = 0x07;

            // read interrupt flags
            byte status = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.Status, UsbInterface.Module._RTC);
            interruptFlagAIF = BitUtils.GetBit(status, 2);
            interruptFlagPIF = BitUtils.GetBit(status, 1);
            interruptFlagDCFIF = BitUtils.GetBit(status, 0);

            // reset interrupt flags
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.Status, StatusResetValue, UsbInterface.Module._RTC);
        }

        /// <summary>
        /// Sets alarm settings on the RTC.
        /// </summary>
        /// <param name="alarmSettings">Alarm settings to set.</param>
        public void SetAlarmSettings(_RTC.AlarmSettings alarmSettings)
        {
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.AlarmMinute, BcdUtils.ConvertIntToBcd(alarmSettings.Minute), UsbInterface.Module._RTC);
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.AlarmHour, BcdUtils.ConvertIntToBcd(alarmSettings.Hour), UsbInterface.Module._RTC);

            byte weekdays = 0x00;
            BitUtils.SetBit(ref weekdays, 0, alarmSettings.Mo);
            BitUtils.SetBit(ref weekdays, 1, alarmSettings.Di);
            BitUtils.SetBit(ref weekdays, 2, alarmSettings.Mi);
            BitUtils.SetBit(ref weekdays, 3, alarmSettings.Do);
            BitUtils.SetBit(ref weekdays, 4, alarmSettings.Fr);
            BitUtils.SetBit(ref weekdays, 5, alarmSettings.Sa);
            BitUtils.SetBit(ref weekdays, 6, alarmSettings.So);
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.AlarmWeekdays, weekdays, UsbInterface.Module._RTC);

            byte alarmConfig = 0x00;
            BitUtils.SetBit(ref alarmConfig, 1, alarmSettings.AlarmAIE);
            BitUtils.SetBit(ref alarmConfig, 2, alarmSettings.AlarmAILED);
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.AlarmConfig, alarmConfig, UsbInterface.Module._RTC);
        }

        /// <summary>
        /// Reads the current alarm settings of the RTC.
        /// </summary>
        /// <returns>Read alarm settings.</returns>
        public _RTC.AlarmSettings ReadAlarmSettings()
        {
            _RTC.AlarmSettings alarmSettings = new AlarmSettings();
            byte alarmMinute = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.AlarmMinute, UsbInterface.Module._RTC);
            byte alarmHour = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.AlarmHour, UsbInterface.Module._RTC);
            byte alarmWeekdays = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.AlarmWeekdays, UsbInterface.Module._RTC);
            byte alarmConfig = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.AlarmConfig, UsbInterface.Module._RTC);

            alarmSettings.Minute = BcdUtils.ConvertBcdToInt(alarmMinute);
            alarmSettings.Hour = BcdUtils.ConvertBcdToInt(alarmHour);

            alarmSettings.Mo = BitUtils.GetBit(alarmWeekdays, 0);
            alarmSettings.Di = BitUtils.GetBit(alarmWeekdays, 1);
            alarmSettings.Mi = BitUtils.GetBit(alarmWeekdays, 2);
            alarmSettings.Do = BitUtils.GetBit(alarmWeekdays, 3);
            alarmSettings.Fr = BitUtils.GetBit(alarmWeekdays, 4);
            alarmSettings.Sa = BitUtils.GetBit(alarmWeekdays, 5);
            alarmSettings.So = BitUtils.GetBit(alarmWeekdays, 6);

            alarmSettings.AlarmAIE = BitUtils.GetBit(alarmConfig, 1);
            alarmSettings.AlarmAILED = BitUtils.GetBit(alarmConfig, 2);

            return alarmSettings;
        }

        /// <summary>
        /// Sets periodical interrupt on the RTC.
        /// </summary>
        /// <param name="periodicalInterruptFrequency">Periodical interrupt frequency to set.</param>
        /// <param name="alarmPILED">PILED-flag to set.</param>
        /// <param name="alarmPIE">PIE-flag to set.</param>
        public void SetPeriodicalInterrupt(
            _RTC.PeriodicalInterruptFrequency periodicalInterruptFrequency,
            bool alarmPILED,
            bool alarmPIE)
        {
            byte periodicalInterrupt = (byte)periodicalInterruptFrequency;
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.PeriodicalInterrupt, periodicalInterrupt);

            byte periodicalInterruptConfig = 0x00;
            BitUtils.SetBit(ref periodicalInterruptConfig, 2, alarmPILED);
            BitUtils.SetBit(ref periodicalInterruptConfig, 1, alarmPIE);
            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.PeriodicalInterruptConfig, periodicalInterruptConfig, UsbInterface.Module._RTC);
        }

        /// <summary>
        /// Reads periodical interrupt from the RTC.
        /// </summary>
        /// <param name="periodicalInterruptFrequency">Read periodical interrupt frequency.</param>
        /// <param name="alarmPILED">Read PILED-flag.</param>
        /// <param name="alarmPIE">Read PIE-flag.</param>
        public void ReadPeriodicalInterrupt(
            out _RTC.PeriodicalInterruptFrequency periodicalInterruptFrequency,
            out bool alarmPILED,
            out bool alarmPIE)
        {
            byte periodicalInterrupt = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.PeriodicalInterrupt, UsbInterface.Module._RTC);
            byte periodicalInterruptConfig = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.PeriodicalInterruptConfig, UsbInterface.Module._RTC);

            periodicalInterruptFrequency = (_RTC.PeriodicalInterruptFrequency)periodicalInterrupt;
            alarmPILED = BitUtils.GetBit(periodicalInterruptConfig, 2);
            alarmPIE = BitUtils.GetBit(periodicalInterruptConfig, 1);
        }

        /// <summary>
        /// Sets DCF settings on the RTC.
        /// </summary>
        /// <param name="enableDcf">Enable or disable DCF.</param>
        /// <param name="dcfLed">DCFLED-flag to set.</param>
        /// <param name="dcfIe">DCFIE-flag to set.</param>
        public void SetDcf(
            bool enableDcf,
            bool dcfLed,
            bool dcfIe)
        {
            if (!enableDcf)
            {
                dcfLed = false;
                dcfIe = false;
            }

            byte dcfConfig = 0x00;
            BitUtils.SetBit(ref dcfConfig, 0, enableDcf);
            BitUtils.SetBit(ref dcfConfig, 1, dcfIe);
            BitUtils.SetBit(ref dcfConfig, 2, dcfLed);

            this.UsbInterface.Write(this.Address, (byte)_RTC.Registers.DcfConfig, dcfConfig, UsbInterface.Module._RTC);
        }

        /// <summary>
        /// Reads the current DCF settings of the RTC.
        /// </summary>
        /// <param name="enableDcf">Read value which indicates whether DCF is enabled or not.</param>
        /// <param name="dcfLed">Read DCFLED-flag.</param>
        /// <param name="dcfIe">Read DCFIE-flag.</param>
        public void ReadDcf(
            out bool enableDcf,
            out bool dcfLed,
            out bool dcfIe)
        {
            byte dcfConfig = this.UsbInterface.Read(this.Address, (byte)_RTC.Registers.DcfConfig, UsbInterface.Module._RTC);

            enableDcf = BitUtils.GetBit(dcfConfig, 0);
            dcfIe = BitUtils.GetBit(dcfConfig, 1);
            dcfLed = BitUtils.GetBit(dcfConfig, 2);
        }

        /// <summary>
        /// Wrapper for alarm settings.
        /// </summary>
        public struct AlarmSettings
        {
            /// <summary>
            /// Gets or sets minute value of alarm.
            /// </summary>
            /// <value>Minute value of alarm.</value>
            public int Minute { get; set; }

            /// <summary>
            /// Gets or sets hour value of alarm.
            /// </summary>
            /// <value>Hour value of alarm.</value>
            public int Hour { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for monday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for monday or not.</value>
            public bool Mo { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for tuesday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for tuesday or not.</value>
            public bool Di { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for wednesday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for wednesday or not.</value>
            public bool Mi { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for thursday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for thursday or not.</value>
            public bool Do { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for friday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for friday or not.</value>
            public bool Fr { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for saturday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for saturday or not.</value>
            public bool Sa { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether alarm should be enabled for sunday or not.
            /// </summary>
            /// <value>A value indicates whether alarm should be enabled for sunday or not.</value>
            public bool So { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether the AILED-flag is set or not.
            /// </summary>
            /// <value>A value indicating whether the AILED-flag is set or not.</value>
            public bool AlarmAILED { get; set; }

            /// <summary>
            /// Gets or sets a value indicating whether the AIE-flag is set or not.
            /// </summary>
            /// <value>A value indicating whether the AILED-flag is set or not.</value>
            public bool AlarmAIE { get; set; }
        }
    }
}