﻿//-----------------------------------------------------------------------------
// <copyright file="TabContent_3DBS.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>GUI-Components of the 3D-BS.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.GUI
{
    using System;
    using System.Collections.Generic;
    using System.Timers;
    using System.Windows;
    using System.Windows.Controls;
    using UsbInterfaceTesttool.Charts;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.Utils;
    
    /// <summary>
    /// GUI-Components of the 3D-BS.
    /// </summary>
    public partial class TabContent_3DBS : UserControl, IStoppable
    {
        /// <summary>
        /// Intervall for automatic reading of 3D-BS axes values.
        /// </summary>
        public const int AutoReadIntervall = 500;

        /// <summary>
        /// Gets or sets timer for triggering the 3D-BS automatically to read its current time and interrupt flags.
        /// </summary>
        private Timer readDataTimer;

        /// <summary>
        /// Initializes a new instance of the TabContent_3DBS class.
        /// </summary>
        public TabContent_3DBS()
        {
            this.InitializeComponent();
            this.InitChart(_3DBS.Range._2g);

            this.DataContext = this;
        }

        /// <summary>
        /// Gets 3D-BS device wrapper.
        /// </summary>
        /// <value>3D-BS device wrapper.</value>
        public _3DBS Device3DBS
        {
            get
            {
                return MainWindow.Device3DBS;
            }
        }

        #region IStoppable Member
        /// <summary>
        /// Stopps device communication before closing the ComPort-connection.
        /// </summary>
        public void StoppDeviceCommunication()
        {
            if (this.readDataTimer != null && this.readDataTimer.Enabled)
            {
                this.ToggleAutomaticallyReadingAxesValues_Click(this.btnToggleAutoRead, null);
            }
        }
        #endregion

        /// <summary>
        /// Reads settings from 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadSettings(object sender, RoutedEventArgs e)
        {
            _3DBS_Settings settings;
            try
            {
                settings = this.Device3DBS.ReadSettings();
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen des 3D-BS konnten nicht ausgelesen werden.");
                return;
            }

            this.InitChart(settings.OperationalSettings.Range);

            this.ParseSettingsWrapper(settings);
        }

        /// <summary>
        /// Initializes chart of 3D-BS axes.
        /// </summary>
        /// <param name="range">Range of the 3D-BS for adjusting the amplitude of chart.</param>
        private void InitChart(_3DBS.Range range)
        {
            range = _3DBS.Range._2g;

            /*
            switch (range)
            {
                case _3DBS.Range._2g:
                    ChartUtils.InitChart(chart, 2048);
                    break;
                case _3DBS.Range._4g:
                    ChartUtils.InitChart(chart, 4096);
                    break;
                case _3DBS.Range._8g:
                    ChartUtils.InitChart(chart, 8192);
                    break;
                case _3DBS.Range.NotAuthorisedCode:
                    break;
                default:
                    throw new ArgumentException();
            }
            */

            ChartUtils.InitChart(chart, 300);
        }

        /// <summary>
        /// Parses settings of a 3D-BS and sets the GUI.
        /// </summary>
        /// <param name="settings">Settings to parse.</param>
        private void ParseSettingsWrapper(_3DBS_Settings settings)
        {
            // operational settings
            this.Range = settings.OperationalSettings.Range;
            this.Bandwidth = settings.OperationalSettings.Bandwidth;
            this.WakeUp = settings.OperationalSettings.Wake_up;
            this.WakeUpPause = settings.OperationalSettings.Wake_up_pause;
            this.ShadowDis = settings.OperationalSettings.Shadow_dis;

            // interrupt settings
            this.EnableLG = settings.InterruptSettings.EnableLG;
            this.EnableHG = settings.InterruptSettings.EnableHG;
            this.EnableAdvINT = settings.InterruptSettings.EnableAdvINT;
            this.AnyMotion = settings.InterruptSettings.AnyMotion;
            this.Alert = settings.InterruptSettings.Alert;
            this.LatchINT = settings.InterruptSettings.LatchINT;
            this.LGthres = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.LGThres);
            this.LGhyst = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.LGHyst);
            this.LGdur = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.LGDur);
            this.CounterLG = settings.InterruptSettings.CounterLG;
            this.HGthres = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.HGThres);
            this.HGhyst = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.HGHyst);
            this.HGdur = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.HGDur);
            this.CounterHG = settings.InterruptSettings.CounterHG;
            this.AnyMotionThres = BitUtils.ConvertBitArrayToString(settings.InterruptSettings.AnyMotionThres);
            this.AnyMotionDuration = settings.InterruptSettings.AnyMotionDuration;
            this.NewDataInt = settings.InterruptSettings.NewDataInt;

            // control settings
            this.ResetINT = settings.ControlSettings.ResetINT;
            this.Selftest0 = settings.ControlSettings.Selftest0;
            this.Selftest1 = settings.ControlSettings.Selftest1;
            this.SoftReset = settings.ControlSettings.SoftReset;
            this.Sleep = settings.ControlSettings.Sleep;

            // status settings
            this.StResult = settings.StatusSettings.StResult;
            this.AlertPhase = settings.StatusSettings.AlertPhase;
            this.LGlatched = settings.StatusSettings.LGlatched;
            this.HGlatched = settings.StatusSettings.HGlatched;
            this.StatusLG = settings.StatusSettings.StatusLG;
            this.StatusHG = settings.StatusSettings.StatusHG;

            // info
            this.AiVersion = BitUtils.ConvertBitArrayToString(settings.InfoSettings.AiVersion);
            this.MlVersion = BitUtils.ConvertBitArrayToString(settings.InfoSettings.MlVersion);
            this.ChipID = BitUtils.ConvertBitArrayToString(settings.InfoSettings.ChipID);
        }

        /// <summary>
        /// Sets settings from the GUI to the 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetSettings_Click(object sender, RoutedEventArgs e)
        {
            _3DBS_Settings settings = this.GetSettingsFromGUI();

            try
            {
                this.Device3DBS.SetSettings(settings);
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen für den 3D-BS konnten nicht gesetzt werden.");
                return;
            }

            this.InitChart(settings.OperationalSettings.Range);
        }

        /// <summary>
        /// Gets current settings from the GUI.
        /// </summary>
        /// <returns>Current settings from the GUI.</returns>
        private _3DBS_Settings GetSettingsFromGUI()
        {
            _3DBS_Settings.Operational operationalSettings = new _3DBS_Settings.Operational();
            operationalSettings.Bandwidth = this.Bandwidth;
            operationalSettings.Range = this.Range;
            operationalSettings.Shadow_dis = this.ShadowDis;
            operationalSettings.Wake_up = this.WakeUp;
            operationalSettings.Wake_up_pause = this.WakeUpPause;

            _3DBS_Settings settings = new _3DBS_Settings();
            settings.OperationalSettings = operationalSettings;
            return settings;
        }

        /// <summary>
        /// Gets current axes values and new data flags.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GetAllData_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GetAllData();
            }
            catch
            {
                ErrorUtils.Show("Die aktuellen Achsenwerte bzw. Flags für neue Daten konnten nicht ausgelesen werden.");
                return;
            }
        }

        /// <summary>
        /// Gets current axes values and new data flags.
        /// </summary>
        private void GetAllData()
        {
            short accX, accY, accZ;
            bool newDataX, newDataY, newDataZ;

            this.Device3DBS.ReadData(out accX, out accY, out accZ, out newDataX, out newDataY, out newDataZ);

            this.CurrentX = accX;
            this.CurrentY = accY;
            this.CurrentZ = accZ;
            this.NewDataX = newDataX;
            this.NewDataY = newDataY;
            this.NewDataZ = newDataZ;
        }

        /// <summary>
        /// Gets current new data flags from the 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GetNewData_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GetNewData();
            }
            catch
            {
                ErrorUtils.Show("Die Flags für neue Daten des 3D-BS konnten nicht ausgelesen werden.");
                return;
            }
        }

        /// <summary>
        /// Gets current new data flags from the 3D-BS.
        /// </summary>
        private void GetNewData()
        {
            bool newDataX, newDataY, newDataZ;

            this.Device3DBS.ReadData(out newDataX, out newDataY, out newDataZ);

            this.NewDataX = newDataX;
            this.NewDataY = newDataY;
            this.NewDataZ = newDataZ;
        }

        /// <summary>
        /// Toggles automatic reading of current axes values and new data flags of 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ToggleAutomaticallyReadingAxesValues_Click(object sender, RoutedEventArgs e)
        {
            Button btn = (Button)sender;

            if (this.readDataTimer == null || !this.readDataTimer.Enabled)
            {
                this.readDataTimer = new Timer(AutoReadIntervall);
                this.readDataTimer.Elapsed += new ElapsedEventHandler(this.ReadDataTimer_Elapsed);
                this.readDataTimer.Start();

                btn.Content = "beenden";
            }
            else
            {
                this.readDataTimer.Stop();

                btn.Content = "starten";
            }
        }

        /// <summary>
        /// Tick-/Elapsed-method for reading current axes values and new data flags of 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadDataTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.uiElement.Dispatcher.BeginInvoke(
                    new Action<object, ElapsedEventArgs>(this.ReadDataTimer_Elapsed),
                    new object[] { sender, e });
                return;
            }

            try
            {
                this.GetAllData();
                this.UpdateChart();
            }
            catch
            {
                ErrorUtils.Show("Die aktuellen Achsenwerte bzw. Flags für neue Daten konnten nicht ausgelesen werden.");
            }
        }

        /// <summary>
        /// Updates the chart with read axes values.
        /// </summary>
        private void UpdateChart()
        {
            List<int> data = new List<int>(new int[] { this.CurrentX, this.CurrentY, this.CurrentZ });            
            
            ChartUtils.MoveSeriesPointsToRight(chart.Series);

            for (int i = 0; i < this.chart.Series.Count; i++)
            {
                this.chart.Series[i].Points[0].YValues = new double[] { data[i] };
            }
        }

        /// <summary>
        /// Resets settings of 3D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ResetSettings_Click(object sender, RoutedEventArgs e)
        {
            _3DBS_Settings.Operational operationalSettings = new _3DBS_Settings.Operational();
            operationalSettings.Bandwidth = _3DBS.Bandwidth._25Hz;
            operationalSettings.Range = _3DBS.Range._2g;
            operationalSettings.Shadow_dis = false;
            operationalSettings.Wake_up = false;
            operationalSettings.Wake_up_pause = _3DBS.WakeUpPause._20ms;

            _3DBS_Settings settings = new _3DBS_Settings();
            settings.OperationalSettings = operationalSettings;

            this.ParseSettingsWrapper(settings);
            try
            {
                this.Device3DBS.SetSettings(settings);
            }
            catch
            {
                ErrorUtils.Show("Das Zurücksetzen der Einstellungen konnte nicht durchgeführt werden.");
            }
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class TabContent_3DBS
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for Range.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty RangeProperty =
            DependencyProperty.Register("Range", typeof(_3DBS.Range), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.Range._2g));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Bandwith.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty BandwidthProperty =
            DependencyProperty.Register("Bandwidth", typeof(_3DBS.Bandwidth), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.Bandwidth._25Hz));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Wake_up.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty WakeUpProperty =
            DependencyProperty.Register("WakeUp", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for WakeUpPause.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty WakeUpPauseProperty =
            DependencyProperty.Register("WakeUpPause", typeof(_3DBS.WakeUpPause), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.WakeUpPause._20ms));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ShadowDis.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ShadowDisProperty =
            DependencyProperty.Register("ShadowDis", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for EnableLG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty EnableLGProperty =
            DependencyProperty.Register("EnableLG", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for EnableHG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty EnableHGProperty =
            DependencyProperty.Register("EnableHG", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for EnableAdvINT.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty EnableAdvINTProperty =
            DependencyProperty.Register("EnableAdvINT", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AnyMotion.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AnyMotionProperty =
            DependencyProperty.Register("AnyMotion", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Alert.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlertProperty =
            DependencyProperty.Register("Alert", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for LatchINT.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty LatchINTProperty =
            DependencyProperty.Register("LatchINT", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for LGthres.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty LGthresProperty =
            DependencyProperty.Register("LGthres", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for LGhyst.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty LGhystProperty =
            DependencyProperty.Register("LGhyst", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for LGdur.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty LGdurProperty =
            DependencyProperty.Register("LGdur", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CounterLG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CounterLGProperty =
            DependencyProperty.Register("CounterLG", typeof(_3DBS.Counter), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.Counter.Reset));

        /// <summary>
        /// Using a DependencyProperty as the backing store for HGthres.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty HGthresProperty =
            DependencyProperty.Register("HGthres", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for HGhyst.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty HGhystProperty =
            DependencyProperty.Register("HGhyst", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for HGdur.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty HGdurProperty =
            DependencyProperty.Register("HGdur", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CounterHG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CounterHGProperty =
            DependencyProperty.Register("CounterHG", typeof(_3DBS.Counter), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.Counter.Reset));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AnyMotionThres.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AnyMotionThresProperty =
            DependencyProperty.Register("AnyMotionThres", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AnyMotionDuration.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AnyMotionDurationProperty =
            DependencyProperty.Register("AnyMotionDuration", typeof(_3DBS.AnyMotionDuration), typeof(TabContent_3DBS), new UIPropertyMetadata(_3DBS.AnyMotionDuration._1));

        /// <summary>
        /// Using a DependencyProperty as the backing store for NewDataInt.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty NewDataIntProperty =
            DependencyProperty.Register("NewDataInt", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ResetINT.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ResetINTProperty =
            DependencyProperty.Register("ResetINT", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Selftest0.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty Selftest0Property =
            DependencyProperty.Register("Selftest0", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Selftest1.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty Selftest1Property =
            DependencyProperty.Register("Selftest1", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SoftReset.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SoftResetProperty =
            DependencyProperty.Register("SoftReset", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Sleep.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SleepProperty =
            DependencyProperty.Register("Sleep", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for StResult.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty StResultProperty =
            DependencyProperty.Register("StResult", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlertPhase.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlertPhaseProperty =
            DependencyProperty.Register("AlertPhase", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for StatusLG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty StatusLGProperty =
            DependencyProperty.Register("StatusLG", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for StatusHG.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty StatusHGProperty =
            DependencyProperty.Register("StatusHG", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for LGlatched.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty LGlatchedProperty =
            DependencyProperty.Register("LGlatched", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for HGlatched.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty HGlatchedProperty =
            DependencyProperty.Register("HGlatched", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AiVersion.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AiVersionProperty =
            DependencyProperty.Register("AiVersion", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for MlVersion.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty MlVersionProperty =
            DependencyProperty.Register("MlVersion", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ChipID.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ChipIDProperty =
            DependencyProperty.Register("ChipID", typeof(string), typeof(TabContent_3DBS), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for NewDataX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty NewDataXProperty =
            DependencyProperty.Register("NewDataX", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for NewDataY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty NewDataYProperty =
            DependencyProperty.Register("NewDataY", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for NewDataZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty NewDataZProperty =
            DependencyProperty.Register("NewDataZ", typeof(bool), typeof(TabContent_3DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentXProperty =
            DependencyProperty.Register("CurrentX", typeof(short), typeof(TabContent_3DBS), new UIPropertyMetadata((short)0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentYProperty =
            DependencyProperty.Register("CurrentY", typeof(short), typeof(TabContent_3DBS), new UIPropertyMetadata((short)0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentZProperty =
            DependencyProperty.Register("CurrentZ", typeof(short), typeof(TabContent_3DBS), new UIPropertyMetadata((short)0));

        /// <summary>
        /// Gets or sets range of the 3D-BS (operational registers)..
        /// </summary>
        /// <value>Range of the 3D-BS.</value>
        public _3DBS.Range Range
        {
            get { return (_3DBS.Range)GetValue(RangeProperty); }
            set { SetValue(RangeProperty, value); }
        }

        /// <summary>
        /// Gets or sets bandwith of the 3D-BS (operational registers)..
        /// </summary>
        /// <value>Bandwith of the 3D-BS.</value>
        public _3DBS.Bandwidth Bandwidth
        {
            get { return (_3DBS.Bandwidth)GetValue(BandwidthProperty); }
            set { SetValue(BandwidthProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Wake_up flag is set or not (operational registers).
        /// </summary>
        /// <value>A value indicating whether the Wake_up flag is set or not (operational registers).</value>
        public bool WakeUp
        {
            get { return (bool)GetValue(WakeUpProperty); }
            set { SetValue(WakeUpProperty, value); }
        }

        /// <summary>
        /// Gets or sets wake up pause (operational registers).
        /// </summary>
        /// <value>Wake up pause (operational registers).</value>
        public _3DBS.WakeUpPause WakeUpPause
        {
            get { return (_3DBS.WakeUpPause)GetValue(WakeUpPauseProperty); }
            set { SetValue(WakeUpPauseProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the ShadowDis flag is set or not (operational registers).
        /// </summary>
        /// <value>A value indicating whether the ShadowDis flag is set or not (operational registers).</value>
        public bool ShadowDis
        {
            get { return (bool)GetValue(ShadowDisProperty); }
            set { SetValue(ShadowDisProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the EnableLG flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the EnableLG flag is set or not (operational registers).</value>
        public bool EnableLG
        {
            get { return (bool)GetValue(EnableLGProperty); }
            set { SetValue(EnableLGProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the EnableHG flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the EnableHG flag is set or not (operational registers).</value>
        public bool EnableHG
        {
            get { return (bool)GetValue(EnableHGProperty); }
            set { SetValue(EnableHGProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the EnableAdvINT flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the EnableAdvINT flag is set or not (operational registers).</value>
        public bool EnableAdvINT
        {
            get { return (bool)GetValue(EnableAdvINTProperty); }
            set { SetValue(EnableAdvINTProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the AnyMotion flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the AnyMotion flag is set or not (operational registers).</value>
        public bool AnyMotion
        {
            get { return (bool)GetValue(AnyMotionProperty); }
            set { SetValue(AnyMotionProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Alert flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the Alert flag is set or not (operational registers).</value>
        public bool Alert
        {
            get { return (bool)GetValue(AlertProperty); }
            set { SetValue(AlertProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the LatchINT flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the LatchINT flag is set or not (operational registers).</value>
        public bool LatchINT
        {
            get { return (bool)GetValue(LatchINTProperty); }
            set { SetValue(LatchINTProperty, value); }
        }

        /// <summary>
        /// Gets or sets LG_thres value (interrupt registers).
        /// </summary>
        /// <value>LG_thres value (interrupt registers).</value>
        public string LGthres
        {
            get { return (string)GetValue(LGthresProperty); }
            set { SetValue(LGthresProperty, value); }
        }

        /// <summary>
        /// Gets or sets LG_hyst value (interrupt registers).
        /// </summary>
        /// <value>LG_thres value (interrupt registers).</value>
        public string LGhyst
        {
            get { return (string)GetValue(LGhystProperty); }
            set { SetValue(LGhystProperty, value); }
        }

        /// <summary>
        /// Gets or sets LG_dur value (interrupt registers).
        /// </summary>
        /// <value>LG_thres value (interrupt registers).</value>
        public string LGdur
        {
            get { return (string)GetValue(LGdurProperty); }
            set { SetValue(LGdurProperty, value); }
        }
        
        /// <summary>
        /// Gets or sets counter_LG value (interrupt registers).
        /// </summary>
        /// <value>Counter_LG value (interrupt registers).</value>
        public _3DBS.Counter CounterLG
        {
            get { return (_3DBS.Counter)GetValue(CounterLGProperty); }
            set { SetValue(CounterLGProperty, value); }
        }

        /// <summary>
        /// Gets or sets HG_thres value (interrupt registers).
        /// </summary>
        /// <value>HG_thres value (interrupt registers).</value>
        public string HGthres
        {
            get { return (string)GetValue(HGthresProperty); }
            set { SetValue(HGthresProperty, value); }
        }

        /// <summary>
        /// Gets or sets HG_hyst value (interrupt registers).
        /// </summary>
        /// <value>HG_thres value (interrupt registers).</value>
        public string HGhyst
        {
            get { return (string)GetValue(HGhystProperty); }
            set { SetValue(HGhystProperty, value); }
        }

        /// <summary>
        /// Gets or sets HG_dur value (interrupt registers).
        /// </summary>
        /// <value>HG_thres value (interrupt registers).</value>
        public string HGdur
        {
            get { return (string)GetValue(HGdurProperty); }
            set { SetValue(HGdurProperty, value); }
        }

        /// <summary>
        /// Gets or sets counter_HG value (interrupt registers).
        /// </summary>
        /// <value>Counter_LG value (interrupt registers).</value>
        public _3DBS.Counter CounterHG
        {
            get { return (_3DBS.Counter)GetValue(CounterHGProperty); }
            set { SetValue(CounterHGProperty, value); }
        }

        /// <summary>
        /// Gets or sets Any_motion_thres value (interrupt registers).
        /// </summary>
        /// <value>Any_motion_thres value (interrupt registers).</value>
        public string AnyMotionThres
        {
            get { return (string)GetValue(AnyMotionThresProperty); }
            set { SetValue(AnyMotionThresProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the any_motion_dur flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the any_motion_dur flag is set or not (interrupt registers).</value>
        public _3DBS.AnyMotionDuration AnyMotionDuration
        {
            get { return (_3DBS.AnyMotionDuration)GetValue(AnyMotionDurationProperty); }
            set { SetValue(AnyMotionDurationProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the New_data_int flag is set or not (interrupt registers).
        /// </summary>
        /// <value>A value indicating whether the New_data_int flag is set or not (interrupt registers).</value>
        public bool NewDataInt
        {
            get { return (bool)GetValue(NewDataIntProperty); }
            set { SetValue(NewDataIntProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Reset_INT flag is set or not (control registers).
        /// </summary>
        /// <value>A value indicating whether the Reset_INT flag is set or not (control registers).</value>
        public bool ResetINT
        {
            get { return (bool)GetValue(ResetINTProperty); }
            set { SetValue(ResetINTProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Selftest_0 flag is set or not (control registers).
        /// </summary>
        /// <value>A value indicating whether the Selftest_0 flag is set or not (control registers).</value>
        public bool Selftest0
        {
            get { return (bool)GetValue(Selftest0Property); }
            set { SetValue(Selftest0Property, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Selftest_1 flag is set or not (control registers).
        /// </summary>
        /// <value>A value indicating whether the Selftest_2 flag is set or not (control registers).</value>
        public bool Selftest1
        {
            get { return (bool)GetValue(Selftest1Property); }
            set { SetValue(Selftest1Property, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Soft_reset flag is set or not (control registers).
        /// </summary>
        /// <value>A value indicating whether the Soft_reset flag is set or not (control registers).</value>
        public bool SoftReset
        {
            get { return (bool)GetValue(SoftResetProperty); }
            set { SetValue(SoftResetProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Sleep flag is set or not (control registers).
        /// </summary>
        /// <value>A value indicating whether the Sleep flag is set or not (control registers).</value>
        public bool Sleep
        {
            get { return (bool)GetValue(SleepProperty); }
            set { SetValue(SleepProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the St_result flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the St_result flag is set or not (status registers).</value>
        public bool StResult
        {
            get { return (bool)GetValue(StResultProperty); }
            set { SetValue(StResultProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Alert_phase flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the Alert_phase flag is set or not (status registers).</value>
        public bool AlertPhase
        {
            get { return (bool)GetValue(AlertPhaseProperty); }
            set { SetValue(AlertPhaseProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Status_LG flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the Status_LG flag is set or not (status registers).</value>
        public bool StatusLG
        {
            get { return (bool)GetValue(StatusLGProperty); }
            set { SetValue(StatusLGProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the Status_HG flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the Status_HG flag is set or not (status registers).</value>
        public bool StatusHG
        {
            get { return (bool)GetValue(StatusHGProperty); }
            set { SetValue(StatusHGProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the LG_latched flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the LG_latched flag is set or not (status registers).</value>
        public bool LGlatched
        {
            get { return (bool)GetValue(LGlatchedProperty); }
            set { SetValue(LGlatchedProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the HG_latched flag is set or not (status registers).
        /// </summary>
        /// <value>A value indicating whether the HG_latched flag is set or not (status registers).</value>
        public bool HGlatched
        {
            get { return (bool)GetValue(HGlatchedProperty); }
            set { SetValue(HGlatchedProperty, value); }
        }

        /// <summary>
        /// Gets or sets Al_version value (info registers).
        /// </summary>
        /// <value>Al_version value (info registers).</value>
        public string AiVersion
        {
            get { return (string)GetValue(AiVersionProperty); }
            set { SetValue(AiVersionProperty, value); }
        }

        /// <summary>
        /// Gets or sets ml_version value (info registers).
        /// </summary>
        /// <value>Ml_version value (info registers).</value>
        public string MlVersion
        {
            get { return (string)GetValue(MlVersionProperty); }
            set { SetValue(MlVersionProperty, value); }
        }

        /// <summary>
        /// Gets or sets ChipID value (info registers).
        /// </summary>
        /// <value>ChipID value (info registers).</value>
        public string ChipID
        {
            get { return (string)GetValue(ChipIDProperty); }
            set { SetValue(ChipIDProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the new data flag for the x-axis is set or not.
        /// </summary>
        /// <value>A value indicating whether the new data flag for the x-axis is set or not.</value>
        public bool NewDataX
        {
            get { return (bool)GetValue(NewDataXProperty); }
            set { SetValue(NewDataXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the new data flag for the y-axis is set or not.
        /// </summary>
        /// <value>A value indicating whether the new data flag for the y-axis is set or not.</value>
        public bool NewDataY
        {
            get { return (bool)GetValue(NewDataYProperty); }
            set { SetValue(NewDataYProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the new data flag for the z-axis is set or not.
        /// </summary>
        /// <value>A value indicating whether the new data flag for the z-axis is set or not.</value>
        public bool NewDataZ
        {
            get { return (bool)GetValue(NewDataZProperty); }
            set { SetValue(NewDataZProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of x axis.
        /// </summary>
        /// <value>Current value of x axis.</value>
        public short CurrentX
        {
            get { return (short)GetValue(CurrentXProperty); }
            set { SetValue(CurrentXProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of y axis.
        /// </summary>
        /// <value>Current value of y axis.</value>
        public short CurrentY
        {
            get { return (short)GetValue(CurrentYProperty); }
            set { SetValue(CurrentYProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of z axis.
        /// </summary>
        /// <value>Current value of z axis.</value>
        public short CurrentZ
        {
            get { return (short)GetValue(CurrentZProperty); }
            set { SetValue(CurrentZProperty, value); }
        }
    }
}