﻿//-----------------------------------------------------------------------------
// <copyright file="TabContent_6DBS.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>GUI-Components of the 6D-BS.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.GUI
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.Drawing;
    using System.Threading;
    using System.Timers;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Forms.DataVisualization.Charting;
    using UsbInterfaceTesttool.Charts;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// GUI-Components of the 6D-BS.
    /// </summary>
    public partial class TabContent_6DBS : UserControl, IStoppable
    {
        /// <summary>
        /// List of all possible octal data rates.
        /// </summary>
        private List<List<string>> odrList;

        /// <summary>
        /// Timer for triggering the Gyro automatically to read its current position.
        /// </summary>
        private System.Timers.Timer readGyro;

        /// <summary>
        /// Timer for triggering the Acc automatically to read its current position.
        /// </summary>
        private System.Timers.Timer readAcc;

        /// <summary>
        /// Initializes a new instance of the TabContent_6DBS class.
        /// </summary>
        public TabContent_6DBS()
        {
            this.InitializeComponent();

            this.DataContext = this;

            this.InitOwnComponents();
        }

        /// <summary>
        /// Gets 6D-BS device wrapper.
        /// </summary>
        /// <value>6D-BS device wrapper.</value>
        public _6DBS Device6DBS
        {
            get
            {
                return MainWindow.Device6DBS;
            }
        }

        #region IStoppable Member
        /// <summary>
        /// Stopps device communication before closing the ComPort-connection.
        /// </summary>
        public void StoppDeviceCommunication()
        {
            if (this.readGyro != null && this.readGyro.Enabled)
            {
                this.GyroToggleReadPosition(this.btnGyroToggleAutoRead, null);
            }

            if (this.readAcc != null && this.readAcc.Enabled)
            {
                this.AccToggleReadPosition(this.btnAccToggleAutoRead, null);
            }
        }
        #endregion

        #region init stuff
        /// <summary>
        /// Initialize the charts and the list of odr's.
        /// </summary>
        private void InitOwnComponents()
        {
            this.InitCharts();
            this.InitOdr();
        }

        /// <summary>
        /// Initialize the charts for the gyro and the acc.
        /// </summary>
        private void InitCharts()
        {
            List<Chart> charts = new List<Chart>();
            charts.AddRange(
                new Chart[]
                {
                    this.chartGyro,
                    this.chartAcc
                });

            foreach (Chart chart in charts)
            {
                ChartUtils.InitChart(chart, 35000);
            }
        }

        /// <summary>
        /// Initialize the list of odr's.
        /// </summary>
        private void InitOdr()
        {
            this.odrList = new List<List<string>>();
            this.odrList.Add(new List<string>(new string[]
                {
                    "7.2",
                    "3.5",
                    "1.8",
                    "0.9",
                    "0.45",
                    "0.18",
                    "0.09",
                    "0.045",
                    "0.018",
                    "0.009"
                }));
            this.odrList.Add(new List<string>(new string[]
                {
                    "13.5",
                    "7.2",
                    "3.5",
                    "1.8",
                    "0.9",
                    "0.45",
                    "0.18",
                    "0.09",
                    "0.045",
                    "0.018",
                }));
            this.odrList.Add(new List<string>(new string[]
                {
                    "27",
                    "13.5",
                    "7.2",
                    "3.5",
                    "1.8",
                    "0.9",
                    "0.45",
                    "0.18",
                    "0.09",
                    "0.045",
                }));
            this.odrList.Add(new List<string>(new string[]
                {
                    "51.4",
                    "27",
                    "13.5",
                    "7.2",
                    "3.5",
                    "1.8",
                    "0.9",
                    "0.45",
                    "0.18",
                    "0.09",
                }));
        }
        #endregion

        #region gyro
        /// <summary>
        /// Refreshes odr.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.OdrContent.Clear();
            foreach (var str in this.odrList[(int)this.GyroGetGyroOdr()])
            {
                this.OdrContent.Add(str);
            }

            this.GyroCutOffFrequencyIndex = 0;
        }

        /// <summary>
        /// Saves the current settings from GUI on the gyro.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GyroSetSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GyroSetGeneralSettings();
                this.GyroSetFilterSettings();
                this.GyroSetScaleSettings();
            }
            catch
            {
                ErrorUtils.Show("Das Setzen der Einstellungen des 6D-BS konnte nicht durchgeführt werden.");
            }
        }

        /// <summary>
        /// Saves the current general settings from GUI on the gyro.
        /// </summary>
        private void GyroSetGeneralSettings()
        {
            _6DBS_Gyro.Odr odr;
            int cutOff;

            odr = this.GyroGetGyroOdr();

            switch (this.GyroSelectOdrIndex)
            {
                case 0:
                    cutOff = 0;
                    break;

                case 1:
                    cutOff = 1;
                    break;

                default:
                    cutOff = (this.GyroSelectOdrIndex - 2) % 4;
                    break;
            }

            this.Device6DBS.Gyro.SetSettings(odr, cutOff, this.GyroEnablePowerDownMode);
        }

        /// <summary>
        /// Gets the current odr rate from the GUI.
        /// </summary>
        /// <returns>Current odr rate from the GUI.</returns>
        private _6DBS_Gyro.Odr GyroGetGyroOdr()
        {
            switch (this.GyroSelectOdrIndex)
            {
                case 0:
                case 1:
                    return _6DBS_Gyro.Odr._95;

                case 2:
                case 3:
                case 4:
                case 5:
                    return _6DBS_Gyro.Odr._190;

                case 6:
                case 7:
                case 8:
                case 9:
                    return _6DBS_Gyro.Odr._380;

                case 10:
                case 11:
                case 12:
                case 13:
                    return _6DBS_Gyro.Odr._760;

                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Saves the current filter settings from GUI on the gyro.
        /// </summary>
        private void GyroSetFilterSettings()
        {
            this.Device6DBS.Gyro.SetFilter(
                this.GyroEnableEdgeSensitiveTrigger,
                this.GyroEnableLevelSensitiveTrigger,
                this.GyroFilterMode,
                this.GyroCutOffFrequencyIndex,
                this.GyroDataFilter);
        }

        /// <summary>
        /// Saves the current scale settings from GUI on the gyro.
        /// </summary>
        private void GyroSetScaleSettings()
        {
            this.Device6DBS.Gyro.SetScale(
                this.GyroBlockDataUpdate,
                this.GyroEndianSelection,
                this.GyroFullScale,
                this.GyroSim);
        }

        /// <summary>
        /// Reads the settings from the 6D-BS.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GyroReadSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GyroReadGeneralSettings();
                this.GyroReadScaleSettings();
                this.GyroReadFilterSettings();
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen des 6D-BS konnten nicht ausgelesen werden.");
            }
        }

        /// <summary>
        /// Reads the general settings from the gyro.
        /// </summary>
        private void GyroReadGeneralSettings()
        {
            _6DBS_Gyro.Odr odr;
            int bandwidthIndex;
            bool enablePowerDownMode;

            this.Device6DBS.Gyro.GetGeneralSettings(
                out odr,
                out bandwidthIndex,
                out enablePowerDownMode);

            this.GyroSelectOdrIndex = (int)odr;
            this.GyroSelectOdrIndex += 4;
            this.GyroSelectOdrIndex += bandwidthIndex;
            
            this.GyroEnablePowerDownMode = enablePowerDownMode;
        }

        /// <summary>
        /// Reads the scale settings from the gyro.
        /// </summary>
        private void GyroReadScaleSettings()
        {
            _6DBS.EndianSelection endian;
            _6DBS_Gyro.FullScale fullScale;
            _6DBS_Gyro.BlockDataUpdate blockDataUpdate;
            _6DBS_Gyro.Sim sim;

            this.Device6DBS.Gyro.GetScale(
                out endian,
                out fullScale,
                out blockDataUpdate,
                out sim);

            this.GyroEndianSelection = endian;
            this.GyroFullScale = fullScale;
            this.GyroBlockDataUpdate = blockDataUpdate;
            this.GyroSim = sim;
        }

        /// <summary>
        /// Reads the filter settings from the gyro.
        /// </summary>
        private void GyroReadFilterSettings()
        {
            bool edgeSensitiveTrigger;
            bool levelSensitiveTrigger;
            _6DBS.FilterMode filterMode;
            int cutOffFrequencyIndex;
            _6DBS_Gyro.DataFilter dataFilter;

            this.Device6DBS.Gyro.GetFilter(
                out edgeSensitiveTrigger,
                out levelSensitiveTrigger,
                out filterMode,
                out cutOffFrequencyIndex,
                out dataFilter);

            this.GyroEnableEdgeSensitiveTrigger = edgeSensitiveTrigger;
            this.GyroEnableLevelSensitiveTrigger = levelSensitiveTrigger;
            this.GyroFilterMode = filterMode;
            this.GyroCutOffFrequencyIndex = cutOffFrequencyIndex;
            this.GyroDataFilter = dataFilter;
        }

        /// <summary>
        /// Resets the settings of the gyro.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GyroResetSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GyroSelectOdrIndex = 0;
                this.GyroSelectOdrIndex = 0;
                this.GyroEnablePowerDownMode = false;
                this.GyroSetGeneralSettings();

                this.GyroEnableEdgeSensitiveTrigger = false;
                this.GyroEnableLevelSensitiveTrigger = false;
                this.GyroFilterMode = _6DBS.FilterMode.NormalModeWithReset;
                this.GyroCutOffFrequencyIndex = 0;
                this.GyroDataFilter = _6DBS_Gyro.DataFilter.Normal;
                this.GyroSetFilterSettings();

                this.GyroBlockDataUpdate = _6DBS_Gyro.BlockDataUpdate.ContinuousUpdate;
                this.GyroEndianSelection = _6DBS.EndianSelection.DataLsbAtLowerAddress;
                this.GyroFullScale = _6DBS_Gyro.FullScale._250dps;
                this.GyroSim = _6DBS_Gyro.Sim.ThreeWireReadModeDisabled;
                this.GyroSetScaleSettings();
            }
            catch
            {
                ErrorUtils.Show("Das Zurücksetzen der Einstellungen des 6D-BS konnte nicht durchgeführt werden.");
            }
        }

        /// <summary>
        /// Starts or stops automatic reading of the position of the gyro.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GyroToggleReadPosition(object sender, RoutedEventArgs e)
        {
            const int GyroTriggerIntervallMin = 500; // ms

            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(
                    new Action<object, RoutedEventArgs>(this.GyroToggleReadPosition),
                    new object[] { sender, e });
                return;
            }

            Button btn = (Button)sender;

            if (this.readGyro == null || !this.readGyro.Enabled)
            {
                if (this.GyroTriggerIntervall < GyroTriggerIntervallMin)
                {
                    ErrorUtils.Show(string.Format("Das Intervall zum automatischen Auslesen der Position des Gyros muss mindestens {0} ms betragen", GyroTriggerIntervallMin));
                    return;
                }

                this.Device6DBS.Gyro.PositionRead += new _6DBS.PositionReadHandler(this.Gyro_ReadPosition);

                this.readGyro = new System.Timers.Timer(this.GyroTriggerIntervall);
                this.readGyro.Elapsed += new ElapsedEventHandler(this.ReadGyro_Elapsed);
                this.readGyro.Start();

                btn.Content = "automatische Abfrage beenden";
                this.GyroEnableInputTriggerIntervall = false;
            }
            else
            {
                this.Device6DBS.Gyro.PositionRead -= new _6DBS.PositionReadHandler(this.Gyro_ReadPosition);

                this.readGyro.Stop();

                btn.Content = "automatische Abfrage starten";
                this.GyroEnableInputTriggerIntervall = true;
            }
        }

        /// <summary>
        /// ReadPosition event of the gyro gets handled.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="args">Standard parameter args.</param>
        private void Gyro_ReadPosition(object sender, PositionArgs args)
        {
            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(
                    new Action<object, PositionArgs>(this.Gyro_ReadPosition),
                    new object[] { sender, args });
                return;
            }

            this.GyroCurrentX = args.X;
            this.GyroCurrentY = args.Y;
            this.GyroCurrentZ = args.Z;

            this.UpdateChart(_6DBS.Type.Gyro);
        }

        /// <summary>
        /// Elapsed-method of the timer for triggering the gyro.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadGyro_Elapsed(object sender, ElapsedEventArgs e)
        {
            new Thread(new ThreadStart(this.ReadGyroPosition)).Start();
        }

        /// <summary>
        /// Reads the current position of the gyro.
        /// </summary>
        private void ReadGyroPosition()
        {
            try
            {
                this.Device6DBS.Gyro.ReadCurrentPosition();
            }
            catch (InvalidOperationException ex)
            {
                this.GyroToggleReadPosition(this.btnGyroToggleAutoRead, null);
                ErrorUtils.Show(ex);
            }
        }

        /// <summary>
        /// Gets the chip ID and current status from the 6D-BS gyro.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void GyroGetStatus_Click(object sender, RoutedEventArgs e)
        {
            byte chipID;
            bool overrunX, overrunY, overrunZ, newDataX, newDataY, newDataZ;

            try
            {
                this.Device6DBS.Gyro.GetStatus(out chipID, out overrunX, out overrunY, out overrunZ, out newDataX, out newDataY, out newDataZ);
            }
            catch
            {
                ErrorUtils.Show("Die Statusflags des 6D-BS konnte nicht ausgelesen werden.");
                return;
            }

            this.GyroChipID = chipID;

            this.GyroOverrunX = overrunX;
            this.GyroOverrunY = overrunY;
            this.GyroOverrunZ = overrunZ;
            this.GyroNewDataX = newDataX;
            this.GyroNewDataY = newDataY;
            this.GyroNewDataZ = newDataZ;
        }
        #endregion

        #region acc
        /// <summary>
        /// Saves the current settings from GUI on the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void AccSetSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.AccSetGeneralSettings();
                this.AccSetScaleSettings();
                this.AccSetFilterSettings();
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen des 6D-BS konnten nicht gesetzt werden.");
            }
        }

        /// <summary>
        /// Saves the general settings from GUI on the acc.
        /// </summary>
        private void AccSetGeneralSettings()
        {
            this.Device6DBS.Acc.SetGeneralSettings(
                this.AccDataRate,
                this.AccEnableLowPowerMode);
        }

        /// <summary>
        /// Saves the scale settings from GUI on the acc.
        /// </summary>
        private void AccSetScaleSettings()
        {
            this.Device6DBS.Acc.SetScale(
                this.AccEndianSelection,
                this.AccFullScale,
                this.AccSpiMode,
                this.AccEnableHighResolutionOutputMode);
        }

        /// <summary>
        /// Saves the filter settings from GUI on the acc.
        /// </summary>
        private void AccSetFilterSettings()
        {
            bool cutOffFrequency1, cutOffFrequency2;
            this.ConvertCutOffFrequency(this.AccCutOffFrequencyIndex, out cutOffFrequency1, out cutOffFrequency2);

            this.Device6DBS.Acc.SetFilter(
                this.AccFilterMode,
                cutOffFrequency1,
                cutOffFrequency2,
                this.AccEnableFilteredDataSelection);
        }

        /// <summary>
        /// Reads the current settings from GUI on the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void AccReadSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.AccReadGeneralSettings();
                this.AccReadScaleSettings();
                this.AccReadFilterSettings();
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen des 6D-BS konnten nicht ausgelesen werden.");
            }
        }

        /// <summary>
        /// Reads the general settings from GUI on the acc.
        /// </summary>
        private void AccReadGeneralSettings()
        {
            _6DBS_Acc.DataRate dataRate;
            bool enableLowPowerMode;

            this.Device6DBS.Acc.GetGeneralSettings(
                out dataRate,
                out enableLowPowerMode);

            this.AccDataRate = dataRate;
            this.AccEnableLowPowerMode = enableLowPowerMode;
        }

        /// <summary>
        /// Reads the scale settings from GUI on the acc.
        /// </summary>
        private void AccReadScaleSettings()
        {
            _6DBS.EndianSelection endian;
            _6DBS_Acc.FullScale fullScale;
            bool highResolutionOutputMode;
            _6DBS_Acc.SpiMode spiMode;

            this.Device6DBS.Acc.GetScale(
                out endian,
                out fullScale,
                out highResolutionOutputMode,
                out spiMode);

            this.AccEndianSelection = endian;
            this.AccFullScale = fullScale;
            this.AccEnableHighResolutionOutputMode = highResolutionOutputMode;
            this.AccSpiMode = spiMode;
        }

        /// <summary>
        /// Reads the filter settings from GUI on the acc.
        /// </summary>
        private void AccReadFilterSettings()
        {
            _6DBS.FilterMode filterMode;
            bool cutOffFrequency1;
            bool cutOffFrequency2;
            bool filteredDataSelection;

            this.Device6DBS.Acc.GetFilter(
                out filterMode,
                out cutOffFrequency1,
                out cutOffFrequency2,
                out filteredDataSelection);

            this.AccFilterMode = filterMode;
            this.AccCutOffFrequencyIndex = this.ConvertCutOffFrequency(cutOffFrequency1, cutOffFrequency2);
            this.AccEnableFilteredDataSelection = filteredDataSelection;
        }

        /// <summary>
        /// Resets the settings of the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void AccResetSettings_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.AccDataRate = _6DBS_Acc.DataRate.KHz1344;
                this.AccEnableLowPowerMode = false;
                this.AccSetGeneralSettings();

                this.AccEndianSelection = _6DBS.EndianSelection.DataLsbAtLowerAddress;
                this.AccFullScale = _6DBS_Acc.FullScale.TwoG;
                this.AccSpiMode = _6DBS_Acc.SpiMode.FourWireInterface;
                this.AccEnableHighResolutionOutputMode = false;
                this.AccSetScaleSettings();

                this.AccFilterMode = _6DBS.FilterMode.NormalModeWithReset;
                this.AccCutOffFrequencyIndex = 0;
                this.AccEnableFilteredDataSelection = false;
                this.AccSetFilterSettings();
            }
            catch
            {
                ErrorUtils.Show("Das Zurücksetzen der Einstellungen des 6D-BS konnte nicht durchgeführt werden.");
            }
        }

        /// <summary>
        /// Reads the current status of the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void AccGetStatus_Click(object sender, RoutedEventArgs e)
        {
            bool overrunX, overrunY, overrunZ, newDataX, newDataY, newDataZ;

            this.Device6DBS.Acc.GetStatus(out overrunX, out overrunY, out overrunZ, out newDataX, out newDataY, out newDataZ);

            this.AccOverrunX = overrunX;
            this.AccOverrunY = overrunY;
            this.AccOverrunZ = overrunZ;
            this.AccNewDataX = newDataX;
            this.AccNewDataY = newDataY;
            this.AccNewDataZ = newDataZ;
        }

        /// <summary>
        /// Converts two bool values to the index for the combo box for selecting the cut-off frequency.
        /// </summary>
        /// <param name="cutOffFrequency1">First bit.</param>
        /// <param name="cutOffFrequency2">Second bit.</param>
        /// <returns>Converted value.</returns>
        private int ConvertCutOffFrequency(bool cutOffFrequency1, bool cutOffFrequency2)
        {
            if (!cutOffFrequency1 && !cutOffFrequency2)
            {
                return 0;
            }
            else if (!cutOffFrequency1 && cutOffFrequency2)
            {
                return 1;
            }
            else if (cutOffFrequency1 && !cutOffFrequency2)
            {
                return 2;
            }
            else if (cutOffFrequency1 && cutOffFrequency2)
            {
                return 3;
            }
            else
            {
                throw new ArgumentException();
            }
        }

        /// <summary>
        /// Converts the index for the combo box for selecting the cut-off frequency to two bool values.
        /// </summary>
        /// <param name="selectedIndex">Index for the combo box for selecting the cut-off frequency.</param>
        /// <param name="cutOffFrequency1">First bit.</param>
        /// <param name="cutOffFrequency2">Second bit.</param>
        private void ConvertCutOffFrequency(int selectedIndex, out bool cutOffFrequency1, out bool cutOffFrequency2)
        {
            switch (selectedIndex)
            {
                case 0:
                    cutOffFrequency1 = false;
                    cutOffFrequency2 = false;
                    break;
                case 1:
                    cutOffFrequency1 = false;
                    cutOffFrequency2 = true;
                    break;
                case 2:
                    cutOffFrequency1 = true;
                    cutOffFrequency2 = false;
                    break;
                case 3:
                    cutOffFrequency1 = true;
                    cutOffFrequency2 = true;
                    break;
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Starts or stops automatic reading of the position of the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void AccToggleReadPosition(object sender, RoutedEventArgs e)
        {
            const int AccTriggerIntervallMin = 500; // ms

            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(
                    new Action<object, RoutedEventArgs>(this.GyroToggleReadPosition),
                    new object[] { sender, e });
                return;
            }

            Button btn = (Button)sender;

            if (this.readAcc == null || !this.readAcc.Enabled)
            {
                if (this.AccTriggerIntervall < AccTriggerIntervallMin)
                {
                    ErrorUtils.Show(string.Format("Das Intervall zum automatischen Auslesen der Position des Accs muss mindestens {0} ms betragen", AccTriggerIntervallMin));
                    return;
                }

                this.Device6DBS.Acc.PositionRead += new _6DBS.PositionReadHandler(this.Acc_PositionRead);

                this.readAcc = new System.Timers.Timer(this.AccTriggerIntervall);
                this.readAcc.Elapsed += new ElapsedEventHandler(this.ReadAcc_Elapsed);
                this.readAcc.Start();

                btn.Content = "automatische Abfrage beenden";
                this.AccEnableInputTriggerIntervall = false;
            }
            else
            {
                this.Device6DBS.Acc.PositionRead -= new _6DBS.PositionReadHandler(this.Acc_PositionRead);

                this.readAcc.Stop();

                btn.Content = "automatische Abfrage starten";
                this.AccEnableInputTriggerIntervall = true;
            }
        }

        /// <summary>
        /// Elapsed-method of the timer for triggering the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadAcc_Elapsed(object sender, ElapsedEventArgs e)
        {
            new Thread(new ThreadStart(this.ReadAccPosition)).Start();
        }

        /// <summary>
        /// Reads the current position of the acc.
        /// </summary>
        private void ReadAccPosition()
        {
            try
            {
                this.Device6DBS.Acc.ReadCurrentPosition();
            }
            catch (InvalidOperationException ex)
            {
                this.AccToggleReadPosition(this.btnGyroToggleAutoRead, null);
                ErrorUtils.Show(ex);
            }
        }

        /// <summary>
        /// Manages new read position from the acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void Acc_PositionRead(object sender, PositionArgs e)
        {
            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(
                    new Action<object, PositionArgs>(this.Acc_PositionRead),
                    new object[] { sender, e });
                return;
            }

            this.AccCurrentX = e.X;
            this.AccCurrentY = e.Y;
            this.AccCurrentZ = e.Z;

            this.UpdateChart(_6DBS.Type.Acc);
        }
        #endregion

        /// <summary>
        /// Updates the gyro or acc chart.
        /// </summary>
        /// <param name="type">Refresh the chart of gyro or acc.</param>
        private void UpdateChart(_6DBS.Type type)
        {
            Chart chart;
            List<int> data;
            switch (type)
            {
                case _6DBS.Type.Gyro:
                    chart = this.chartGyro;
                    data = new List<int>(new int[]
                    {
                        this.GyroCurrentX,
                        this.GyroCurrentY,
                        this.GyroCurrentZ
                    });

                    break;

                case _6DBS.Type.Acc:
                    chart = this.chartAcc;
                    data = new List<int>(new int[]
                    {
                        this.AccCurrentX,
                        this.AccCurrentY,
                        this.AccCurrentZ
                    });

                    break;

                default:
                    throw new ArgumentException();
            }

            ChartUtils.MoveSeriesPointsToRight(chart.Series);
            if (type == _6DBS.Type.Acc)
            {
                ChartUtils.UpdateChartAxes(chart, this.AccEnableHighResolutionOutputMode);
            }

            for (int i = 0; i < this.chartGyro.Series.Count; i++)
            {
                chart.Series[i].Points[0].YValues = new double[] { data[i] };
            }
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class TabContent_6DBS
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroFullScale.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroFullScaleProperty =
            DependencyProperty.Register("GyroFullScale", typeof(_6DBS_Gyro.FullScale), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Gyro.FullScale._250dps));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroEnablePowerDownMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroEnablePowerDownModeProperty =
            DependencyProperty.Register("GyroEnablePowerDownMode", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroEndianSelection.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroEndianSelectionProperty =
            DependencyProperty.Register("GyroEndianSelection", typeof(_6DBS.EndianSelection), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS.EndianSelection.DataLsbAtLowerAddress));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroBlockDataUpdate.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroBlockDataUpdateProperty =
            DependencyProperty.Register("GyroBlockDataUpdate", typeof(_6DBS_Gyro.BlockDataUpdate), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Gyro.BlockDataUpdate.ContinuousUpdate));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroSim.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroSimProperty =
            DependencyProperty.Register("GyroSim", typeof(_6DBS_Gyro.Sim), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Gyro.Sim.ThreeWireReadModeDisabled));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroDataFilter.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroDataFilterProperty =
            DependencyProperty.Register("GyroDataFilter", typeof(_6DBS_Gyro.DataFilter), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Gyro.DataFilter.Normal));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroFilterMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroFilterModeProperty =
            DependencyProperty.Register("GyroFilterMode", typeof(_6DBS.FilterMode), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS.FilterMode.NormalModeWithReset));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroEnableEdgeSensitiveTrigger.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroEnableEdgeSensitiveTriggerProperty =
            DependencyProperty.Register("GyroEnableEdgeSensitiveTrigger", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroEnableLevelSensitiveTrigger.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroEnableLevelSensitiveTriggerProperty =
            DependencyProperty.Register("GyroEnableLevelSensitiveTrigger", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroSelectOdrIndex.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroSelectOdrIndexProperty =
            DependencyProperty.Register("GyroSelectOdrIndex", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for OdrContent.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty OdrContentProperty =
            DependencyProperty.Register("OdrContent", typeof(ObservableCollection<string>), typeof(TabContent_6DBS), new UIPropertyMetadata(new ObservableCollection<string>()));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroCutOffFrequencyIndex.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroCutOffFrequencyIndexProperty =
            DependencyProperty.Register("GyroCutOffFrequencyIndex", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroCurrentX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroCurrentXProperty =
            DependencyProperty.Register("GyroCurrentX", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroCurrentY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroCurrentYProperty =
            DependencyProperty.Register("GyroCurrentY", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroCurrentZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroCurrentZProperty =
            DependencyProperty.Register("GyroCurrentZ", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroChipID.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroChipIDProperty =
            DependencyProperty.Register("GyroChipID", typeof(byte), typeof(TabContent_6DBS), new UIPropertyMetadata((byte)0xFF));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroOverrunX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroOverrunXProperty =
            DependencyProperty.Register("GyroOverrunX", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroOverrunY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroOverrunYProperty =
            DependencyProperty.Register("GyroOverrunY", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroOverrunZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroOverrunZProperty =
            DependencyProperty.Register("GyroOverrunZ", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroNewDataX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroNewDataXProperty =
            DependencyProperty.Register("GyroNewDataX", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroNewDataY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroNewDataYProperty =
            DependencyProperty.Register("GyroNewDataY", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroNewDataZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroNewDataZProperty =
            DependencyProperty.Register("GyroNewDataZ", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccDataRate.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccDataRateProperty =
            DependencyProperty.Register("AccDataRate", typeof(_6DBS_Acc.DataRate), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Acc.DataRate.KHz1344));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccEnableLowPowerMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccEnableLowPowerModeProperty =
            DependencyProperty.Register("AccEnableLowPowerMode", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccEnableHighResolutionOutputMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccEnableHighResolutionOutputModeProperty =
            DependencyProperty.Register("AccEnableHighResolutionOutputMode", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccFullScale.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccFullScaleProperty =
            DependencyProperty.Register("AccFullScale", typeof(_6DBS_Acc.FullScale), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Acc.FullScale.TwoG));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccSpiMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccSpiModeProperty =
            DependencyProperty.Register("AccSpiMode", typeof(_6DBS_Acc.SpiMode), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS_Acc.SpiMode.FourWireInterface));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccFilterMode.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccFilterModeProperty =
            DependencyProperty.Register("AccFilterMode", typeof(_6DBS.FilterMode), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS.FilterMode.NormalModeWithReset));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccCutOffFrequencyIndex.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccCutOffFrequencyIndexProperty =
            DependencyProperty.Register("AccCutOffFrequencyIndex", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccEnableFilteredDataSelection.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccEnableFilteredDataSelectionProperty =
            DependencyProperty.Register("AccEnableFilteredDataSelection", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccEndianSelection.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccEndianSelectionProperty =
            DependencyProperty.Register("AccEndianSelection", typeof(_6DBS.EndianSelection), typeof(TabContent_6DBS), new UIPropertyMetadata(_6DBS.EndianSelection.DataLsbAtLowerAddress));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccCurrentX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccCurrentXProperty =
            DependencyProperty.Register("AccCurrentX", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccCurrentY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccCurrentYProperty =
            DependencyProperty.Register("AccCurrentY", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccCurrentZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccCurrentZProperty =
            DependencyProperty.Register("AccCurrentZ", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccOverrunX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccOverrunXProperty =
            DependencyProperty.Register("AccOverrunX", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccOverrunY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccOverrunYProperty =
            DependencyProperty.Register("AccOverrunY", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccOverrunZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccOverrunZProperty =
            DependencyProperty.Register("AccOverrunZ", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccNewDataX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccNewDataXProperty =
            DependencyProperty.Register("AccNewDataX", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccNewDataY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccNewDataYProperty =
            DependencyProperty.Register("AccNewDataY", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccNewDataZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccNewDataZProperty =
            DependencyProperty.Register("AccNewDataZ", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentAccX.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentAccXProperty =
            DependencyProperty.Register("CurrentAccX", typeof(int), typeof(TabContent_3DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentAccY.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentAccYProperty =
            DependencyProperty.Register("CurrentAccY", typeof(int), typeof(TabContent_3DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for CurrentAccZ.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CurrentAccZProperty =
            DependencyProperty.Register("CurrentAccZ", typeof(int), typeof(TabContent_3DBS), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroTriggerIntervall.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroTriggerIntervallProperty =
            DependencyProperty.Register("GyroTriggerIntervall", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(500));

        /// <summary>
        /// Using a DependencyProperty as the backing store for GyroEnableInputTriggerIntervall.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty GyroEnableInputTriggerIntervallProperty =
            DependencyProperty.Register("GyroEnableInputTriggerIntervall", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(true));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccTriggerIntervall.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccTriggerIntervallProperty =
            DependencyProperty.Register("AccTriggerIntervall", typeof(int), typeof(TabContent_6DBS), new UIPropertyMetadata(500));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AccEnableInputTriggerIntervall.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AccEnableInputTriggerIntervallProperty =
            DependencyProperty.Register("AccEnableInputTriggerIntervall", typeof(bool), typeof(TabContent_6DBS), new UIPropertyMetadata(true));

        /// <summary>
        /// Gets or sets full scale of the gyro.
        /// </summary>
        /// <value>Full scale of the gyro.</value>
        public _6DBS_Gyro.FullScale GyroFullScale
        {
            get { return (_6DBS_Gyro.FullScale)GetValue(GyroFullScaleProperty); }
            set { SetValue(GyroFullScaleProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether power down mode should be enabled or not on the gyro.
        /// </summary>
        /// <value>Enabling of power down mode of the gyro.</value>
        public bool GyroEnablePowerDownMode
        {
            get { return (bool)GetValue(GyroEnablePowerDownModeProperty); }
            set { SetValue(GyroEnablePowerDownModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets endian selection of the gyro.
        /// </summary>
        /// <value>Endian selection of the gyro.</value>
        public _6DBS.EndianSelection GyroEndianSelection
        {
            get { return (_6DBS.EndianSelection)GetValue(GyroEndianSelectionProperty); }
            set { SetValue(GyroEndianSelectionProperty, value); }
        }

        /// <summary>
        /// Gets or sets block data update of the gyro.
        /// </summary>
        /// <value>Block data update of the gyro.</value>
        public _6DBS_Gyro.BlockDataUpdate GyroBlockDataUpdate
        {
            get { return (_6DBS_Gyro.BlockDataUpdate)GetValue(GyroBlockDataUpdateProperty); }
            set { SetValue(GyroBlockDataUpdateProperty, value); }
        }

        /// <summary>
        /// Gets or sets sim of the gyro.
        /// </summary>
        /// <value>Sim of the gyro.</value>
        public _6DBS_Gyro.Sim GyroSim
        {
            get { return (_6DBS_Gyro.Sim)GetValue(GyroSimProperty); }
            set { SetValue(GyroSimProperty, value); }
        }

        /// <summary>
        /// Gets or sets data filter of the gyro.
        /// </summary>
        /// <value>Data filter of the gyro.</value>
        public _6DBS_Gyro.DataFilter GyroDataFilter
        {
            get { return (_6DBS_Gyro.DataFilter)GetValue(GyroDataFilterProperty); }
            set { SetValue(GyroDataFilterProperty, value); }
        }

        /// <summary>
        /// Gets or sets filter mode of the gyro.
        /// </summary>
        /// <value>Filter mode of the gyro.</value>
        public _6DBS.FilterMode GyroFilterMode
        {
            get { return (_6DBS.FilterMode)GetValue(GyroFilterModeProperty); }
            set { SetValue(GyroFilterModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether edge sensitivity should be enabled or not on the gyro.
        /// </summary>
        /// <value>Enabling of edge sensitivity of the gyro.</value>
        public bool GyroEnableEdgeSensitiveTrigger
        {
            get { return (bool)GetValue(GyroEnableEdgeSensitiveTriggerProperty); }
            set { SetValue(GyroEnableEdgeSensitiveTriggerProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether level sensitivity should be enabled or not on the gyro.
        /// </summary>
        /// <value>Enabling of level sensitivity of the gyro.</value>
        public bool GyroEnableLevelSensitiveTrigger
        {
            get { return (bool)GetValue(GyroEnableLevelSensitiveTriggerProperty); }
            set { SetValue(GyroEnableLevelSensitiveTriggerProperty, value); }
        }

        /// <summary>
        /// Gets or sets selected index of chosable odr.
        /// </summary>
        /// <value>Selected index of chosable odr.</value>
        public int GyroSelectOdrIndex
        {
            get { return (int)GetValue(GyroSelectOdrIndexProperty); }
            set { SetValue(GyroSelectOdrIndexProperty, value); }
        }

        /// <summary>
        /// Gets or sets current content of the GUI-element for chosing odr.
        /// </summary>
        /// <value>Current content of the GUI-element for chosing odr.</value>
        public ObservableCollection<string> OdrContent
        {
            get { return (ObservableCollection<string>)GetValue(OdrContentProperty); }
            set { SetValue(OdrContentProperty, value); }
        }

        /// <summary>
        /// Gets or sets current cut-off frequency index of the Gyro.
        /// </summary>
        /// <value>Current cut-off frequency index of the Gyro.</value>
        public int GyroCutOffFrequencyIndex
        {
            get { return (int)GetValue(GyroCutOffFrequencyIndexProperty); }
            set { SetValue(GyroCutOffFrequencyIndexProperty, value); }
        }

        /// <summary>
        /// Gets or sets current x-position of the gyro.
        /// </summary>
        /// <value>Current x-position of the gyro.</value>
        public int GyroCurrentX
        {
            get { return (int)GetValue(GyroCurrentXProperty); }
            set { SetValue(GyroCurrentXProperty, value); }
        }

        /// <summary>
        /// Gets or sets current y-position of the gyro.
        /// </summary>
        /// <value>Current y-position of the gyro.</value>
        public int GyroCurrentY
        {
            get { return (int)GetValue(GyroCurrentYProperty); }
            set { SetValue(GyroCurrentYProperty, value); }
        }

        /// <summary>
        /// Gets or sets current z-position of the gyro.
        /// </summary>
        /// <value>Current z-position of the gyro.</value>
        public int GyroCurrentZ
        {
            get { return (int)GetValue(GyroCurrentZProperty); }
            set { SetValue(GyroCurrentZProperty, value); }
        }

        /// <summary>
        /// Gets or sets chip ID of the gyro.
        /// </summary>
        /// <value>Chip ID of the gyro.</value>
        public byte GyroChipID
        {
            get { return (byte)GetValue(GyroChipIDProperty); }
            set { SetValue(GyroChipIDProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the x overrun flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the x overrun flag of the gyro is set or not.</value>
        public bool GyroOverrunX
        {
            get { return (bool)GetValue(GyroOverrunXProperty); }
            set { SetValue(GyroOverrunXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the y overrun flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the y overrun flag of the gyro is set or not.</value>
        public bool GyroOverrunY
        {
            get { return (bool)GetValue(GyroOverrunYProperty); }
            set { SetValue(GyroOverrunYProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the z overrun flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the z overrun flag of the gyro is set or not.</value>
        public bool GyroOverrunZ
        {
            get { return (bool)GetValue(GyroOverrunZProperty); }
            set { SetValue(GyroOverrunZProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the x new data flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the x new data flag of the gyro is set or not.</value>
        public bool GyroNewDataX
        {
            get { return (bool)GetValue(GyroNewDataXProperty); }
            set { SetValue(GyroNewDataXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the y new data flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the y new data flag of the gyro is set or not.</value>
        public bool GyroNewDataY
        {
            get { return (bool)GetValue(GyroNewDataYProperty); }
            set { SetValue(GyroNewDataYProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the z new data flag of the gyro is set or not.
        /// </summary>
        /// <value>A value indicating whether the z new data flag of the gyro is set or not.</value>
        public bool GyroNewDataZ
        {
            get { return (bool)GetValue(GyroNewDataZProperty); }
            set { SetValue(GyroNewDataZProperty, value); }
        }

        /// <summary>
        /// Gets or sets data rate of the acc.
        /// </summary>
        /// <value>Data rate of the acc.</value>
        public _6DBS_Acc.DataRate AccDataRate
        {
            get { return (_6DBS_Acc.DataRate)GetValue(AccDataRateProperty); }
            set { SetValue(AccDataRateProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the low power mode of the acc is enabled or not.
        /// </summary>
        /// <value>A value indicating whether the low power mode of the acc is enabled or not.</value>
        public bool AccEnableLowPowerMode
        {
            get { return (bool)GetValue(AccEnableLowPowerModeProperty); }
            set { SetValue(AccEnableLowPowerModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the high resolution mode of the acc is enabled or not.
        /// </summary>
        /// <value>A value indicating whether the high resolution mode of the acc is enabled or not.</value>
        public bool AccEnableHighResolutionOutputMode
        {
            get { return (bool)GetValue(AccEnableHighResolutionOutputModeProperty); }
            set { SetValue(AccEnableHighResolutionOutputModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets full scale of the acc.
        /// </summary>
        /// <value>Full scale of the acc.</value>
        public _6DBS_Acc.FullScale AccFullScale
        {
            get { return (_6DBS_Acc.FullScale)GetValue(AccFullScaleProperty); }
            set { SetValue(AccFullScaleProperty, value); }
        }

        /// <summary>
        /// Gets or sets spi mode of the acc.
        /// </summary>
        /// <value>Spi mode of the acc.</value>
        public _6DBS_Acc.SpiMode AccSpiMode
        {
            get { return (_6DBS_Acc.SpiMode)GetValue(AccSpiModeProperty); }
            set { SetValue(AccSpiModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets filter mode of the acc.
        /// </summary>
        /// <value>Filter mode of the acc.</value>
        public _6DBS.FilterMode AccFilterMode
        {
            get { return (_6DBS.FilterMode)GetValue(AccFilterModeProperty); }
            set { SetValue(AccFilterModeProperty, value); }
        }

        /// <summary>
        /// Gets or sets cut-off frequency of the acc.
        /// </summary>
        /// <value>Cut-off frequency of the acc.</value>
        public int AccCutOffFrequencyIndex
        {
            get { return (int)GetValue(AccCutOffFrequencyIndexProperty); }
            set { SetValue(AccCutOffFrequencyIndexProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the filtered data selection of the acc is enabled or not.
        /// </summary>
        /// <value>A value indicating whether the filtered data selection of the acc is enabled or not.</value>
        public bool AccEnableFilteredDataSelection
        {
            get { return (bool)GetValue(AccEnableFilteredDataSelectionProperty); }
            set { SetValue(AccEnableFilteredDataSelectionProperty, value); }
        }

        /// <summary>
        /// Gets or sets endian selection of the acc.
        /// </summary>
        /// <value>Endian selection of the acc.</value>
        public _6DBS.EndianSelection AccEndianSelection
        {
            get { return (_6DBS.EndianSelection)GetValue(AccEndianSelectionProperty); }
            set { SetValue(AccEndianSelectionProperty, value); }
        }

        /// <summary>
        /// Gets or sets current x-position of the acc.
        /// </summary>
        /// <value>Current x-position of the acc.</value>
        public int AccCurrentY
        {
            get { return (int)GetValue(AccCurrentYProperty); }
            set { SetValue(AccCurrentYProperty, value); }
        }

        /// <summary>
        /// Gets or sets current y-position of the acc.
        /// </summary>
        /// <value>Current y-position of the acc.</value>
        public int AccCurrentZ
        {
            get { return (int)GetValue(AccCurrentZProperty); }
            set { SetValue(AccCurrentZProperty, value); }
        }

        /// <summary>
        /// Gets or sets current z-position of the acc.
        /// </summary>
        /// <value>Current z-position of the acc.</value>
        public int AccCurrentX
        {
            get { return (int)GetValue(AccCurrentXProperty); }
            set { SetValue(AccCurrentXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the x overrun flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the x overrun flag of the acc is set or not.</value>
        public bool AccOverrunX
        {
            get { return (bool)GetValue(AccOverrunXProperty); }
            set { SetValue(AccOverrunXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the y overrun flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the y overrun flag of the acc is set or not.</value>
        public bool AccOverrunY
        {
            get { return (bool)GetValue(AccOverrunYProperty); }
            set { SetValue(AccOverrunYProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the z overrun flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the z overrun flag of the acc is set or not.</value>
        public bool AccOverrunZ
        {
            get { return (bool)GetValue(AccOverrunZProperty); }
            set { SetValue(AccOverrunZProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the x new data flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the x new data flag of the acc is set or not.</value>
        public bool AccNewDataX
        {
            get { return (bool)GetValue(AccNewDataXProperty); }
            set { SetValue(AccNewDataXProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the y new data flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the y new data flag of the acc is set or not.</value>
        public bool AccNewDataY
        {
            get { return (bool)GetValue(AccNewDataYProperty); }
            set { SetValue(AccNewDataYProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the z new data flag of the acc is set or not.
        /// </summary>
        /// <value>A value indicating whether the z new data flag of the acc is set or not.</value>
        public bool AccNewDataZ
        {
            get { return (bool)GetValue(AccNewDataZProperty); }
            set { SetValue(AccNewDataZProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of the x axis.
        /// </summary>
        /// <value>Current value of the x axis.</value>
        public int CurrentAccX
        {
            get { return (int)GetValue(CurrentAccXProperty); }
            set { SetValue(CurrentAccXProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of the y axis.
        /// </summary>
        /// <value>Current value of the y axis.</value>
        public int CurrentAccY
        {
            get { return (int)GetValue(CurrentAccYProperty); }
            set { SetValue(CurrentAccYProperty, value); }
        }

        /// <summary>
        /// Gets or sets current value of the z axis.
        /// </summary>
        /// <value>Current value of the z axis.</value>
        public int CurrentAccZ
        {
            get { return (int)GetValue(CurrentAccZProperty); }
            set { SetValue(CurrentAccZProperty, value); }
        }

        /// <summary>
        /// Gets or sets intervall in ms for triggering the gyro.
        /// </summary>
        /// <value>Intervall in ms for triggering the gyro.</value>
        public int GyroTriggerIntervall
        {
            get { return (int)GetValue(GyroTriggerIntervallProperty); }
            set { SetValue(GyroTriggerIntervallProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the textbox for entering a intervall to trigger the gyro should be enabled or not.
        /// </summary>
        /// <value>A value indicating whether the textbox for entering a intervall to trigger the gyro should be enabled or not.</value>
        public bool GyroEnableInputTriggerIntervall
        {
            get { return (bool)GetValue(GyroEnableInputTriggerIntervallProperty); }
            set { SetValue(GyroEnableInputTriggerIntervallProperty, value); }
        }

        /// <summary>
        /// Gets or sets intervall in ms for triggering the acc.
        /// </summary>
        /// <value>Intervall in ms for triggering the acc.</value>
        public int AccTriggerIntervall
        {
            get { return (int)GetValue(AccTriggerIntervallProperty); }
            set { SetValue(AccTriggerIntervallProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the textbox for entering a intervall to trigger the acc should be enabled or not.
        /// </summary>
        /// <value>A value indicating whether the textbox for entering a intervall to trigger the acc should be enabled or not.</value>
        public bool AccEnableInputTriggerIntervall
        {
            get { return (bool)GetValue(AccEnableInputTriggerIntervallProperty); }
            set { SetValue(AccEnableInputTriggerIntervallProperty, value); }
        }
    }
}