﻿//-----------------------------------------------------------------------------
// <copyright file="TabContent_Allgemein.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>GUI-elements for general operations for a USB-interface.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.GUI
{
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// GUI-elements for general operations for a USB-interface.
    /// </summary>
    public partial class TabContent_Allgemein : UserControl
    {
        /// <summary>
        /// Wrapper for general tasks of a USB interface.
        /// </summary>
        private GeneralTasks generalTasks;

        /// <summary>
        /// Initializes a new instance of the TabContent_Allgemein class.
        /// </summary>
        public TabContent_Allgemein()
        {
            this.InitializeComponent();

            this.DataContext = this;
        }

        /// <summary>
        /// Gets UsbInterface from the parent (MainWindow).
        /// </summary>
        /// <value>UsbInterface from the parent (MainWindow).</value>
        public UsbInterface UsbInterface
        {
            get
            {
                return MainWindow.GetDataContext(this).UsbInterface;
            }
        }

        /// <summary>
        /// Gets or sets wrapper for general tasks of a USB interface.
        /// </summary>
        /// <value>Wrapper for general tasks of a USB interface.</value>
        public GeneralTasks GeneralTasks
        {
            get
            {
                if (this.generalTasks == null)
                {
                    this.generalTasks = new GeneralTasks(this.UsbInterface);
                }

                return this.generalTasks;
            }

            set
            {
                this.generalTasks = value;
            }
        }

        /// <summary>
        /// Sends a custom command to the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SendCustomCommand_Click(object sender, RoutedEventArgs e)
        {
            MainWindow.GetDataContext(this).UsbInterface.Read(this.CustomCommand);
        }

        /// <summary>
        /// Sends a custom command by pressing return/enter.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SendCustomCommand_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.Return:
                    this.SendCustomCommand_Click(sender, null);
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Sets general settings of an interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetInterfaceSettings(object sender, RoutedEventArgs e)
        {
            try
            {
                this.GeneralTasks.SetBaudRate(this.BaudRate);
                this.GeneralTasks.SetSpiClockRate(this.SpiClockRate);

                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Die Einstellungen für das Interface konnten nicht gesetzt werden.");
            }
        }

        /// <summary>
        /// Shows status and firmware information of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ShowStatusFirmware_Click(object sender, RoutedEventArgs e)
        {
            if (!this.UsbInterface.ShowStatusFirmware())
            {
                ErrorUtils.Show("Die Statusinformationen des Interfaces konnten nicht ausgelesen werden.");
            }
        }

        /// <summary>
        /// Shows saved macro of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ShowMacroStorage_Click(object sender, RoutedEventArgs e)
        {
            const string GetMacroStorage = "u";

            try
            {
                this.UsbInterface.Read(GetMacroStorage);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Der Makrospeicher des Interfaces konnten nicht ausgelesen werden.");
            }
        }

        /// <summary>
        /// Removes saved macro of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void RemoveMacroStorage_Click(object sender, RoutedEventArgs e)
        {
            const string RemoveMacroStorage = "u v00{} u";

            try
            {
                this.UsbInterface.Write(RemoveMacroStorage);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Der Makrospeicher des Interfaces konnte nicht gelöscht werden.");
            }
        }

        /// <summary>
        /// Starts the saved macro of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void StartMacro_Click(object sender, RoutedEventArgs e)
        {
            const string StartMacro = "V00{}";

            try
            {
                this.UsbInterface.Write(StartMacro);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Der Makrospeicher des Interfaces konnte nicht von Position 0 gestartet werden.");
            }
        }

        /// <summary>
        /// Ends the saved macro of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void EndMacro_Click(object sender, RoutedEventArgs e)
        {
            const string EndMacro = "<";

            try
            {
                this.UsbInterface.Write(EndMacro);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Der Makrospeicher des Interfaces konnte nicht beendet werden.");
            }
        }

        /// <summary>
        /// Does a soft reset of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void DoSoftReset_Click(object sender, RoutedEventArgs e)
        {
            const string DoSoftReset = "zAA";

            try
            {
                this.UsbInterface.Write(DoSoftReset);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Der Auslieferungszustand des Interfaces konnte nicht wiederhergestellt werden.");
            }
        }

        /// <summary>
        /// Does a hard reset of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void DoHardReset_Click(object sender, RoutedEventArgs e)
        {
            const string DoSoftReset = "z4B";

            try
            {
                this.UsbInterface.Write(DoSoftReset);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Die Y-Parameter des Interfaces konnten nicht zurücksetzt werden.");
            }
        }

        /// <summary>
        /// Resets the Y-parameter of the USB-interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ResetYParameter_Click(object sender, RoutedEventArgs e)
        {
            const string ResetYParameter = "y00 y10 y20 y30 y40 y50 y60 y70 ?";

            try
            {
                this.UsbInterface.Write(ResetYParameter);
                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Das Interface konnte nicht zurücksetzt werden.");
            }
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class TabContent_Allgemein
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for CustomCommand.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty CustomCommandProperty =
            DependencyProperty.Register("CustomCommand", typeof(string), typeof(TabContent_Allgemein), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for BaudRate.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty BaudRateProperty =
            DependencyProperty.Register("BaudRate", typeof(GeneralTasks.InterfaceBaudRate), typeof(TabContent_Allgemein), new UIPropertyMetadata(GeneralTasks.InterfaceBaudRate._38400));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SpiClockRate.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SpiClockRateProperty =
            DependencyProperty.Register("SpiClockRate", typeof(GeneralTasks.SpiClockRate), typeof(TabContent_Allgemein), new UIPropertyMetadata(GeneralTasks.SpiClockRate._250kHz));

        /// <summary>
        /// Gets or sets the current command.
        /// </summary>
        /// <value>Current command.</value>
        public string CustomCommand
        {
            get { return (string)GetValue(CustomCommandProperty); }
            set { SetValue(CustomCommandProperty, value); }
        }

        /// <summary>
        /// Gets or sets baud rate of SPI interface.
        /// </summary>
        /// <value>Baud rate of SPI interface.</value>
        public GeneralTasks.InterfaceBaudRate BaudRate
        {
            get { return (GeneralTasks.InterfaceBaudRate)GetValue(BaudRateProperty); }
            set { SetValue(BaudRateProperty, value); }
        }

        /// <summary>
        /// Gets or sets clock rate of SPI interface.
        /// </summary>
        /// <value>Clock rate of SPI interface.</value>
        public GeneralTasks.SpiClockRate SpiClockRate
        {
            get { return (GeneralTasks.SpiClockRate)GetValue(SpiClockRateProperty); }
            set { SetValue(SpiClockRateProperty, value); }
        }
    }
}
