﻿//-----------------------------------------------------------------------------
// <copyright file="TabContent_RTC.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>
// GUI-Components of the RTC-DCF.
// </summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.GUI
{
    using System;
    using System.Linq;
    using System.Timers;
    using System.Windows;
    using System.Windows.Controls;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// GUI-Components of the RTC-DCF.
    /// </summary>
    public partial class TabContent_RTC : UserControl, IStoppable
    {
        /// <summary>
        /// Timer for triggering the RTC automatically to read its current time and interrupt flags.
        /// </summary>
        private Timer readTimeAndInterruptTimer;

        /// <summary>
        /// Initializes a new instance of the TabContent_RTC class.
        /// </summary>
        public TabContent_RTC()
        {
            this.InitializeComponent();

            this.DataContext = this;
        }

        /// <summary>
        /// Gets RTC device wrapper.
        /// </summary>
        /// <value>RTC device wrapper.</value>
        public _RTC DeviceRtc
        {
            get
            {
                return MainWindow.DeviceRTC;
            }
        }

        #region IStoppable Member
        /// <summary>
        /// Stopps device communication before closing the ComPort-connection.
        /// </summary>
        public void StoppDeviceCommunication()
        {
            if (this.readTimeAndInterruptTimer != null && this.readTimeAndInterruptTimer.Enabled)
            {
                this.ToggleAutoReadTimeAndInterrupt_Click(this.btnToggleAutoRead, null);
            }
        }
        #endregion

        /// <summary>
        /// Sets the time of the RTC based on the GUI.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetTime_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.DeviceRtc.SetTime(this.dateTimePicker.DateTimeSelected);
            }
            catch
            {
                ErrorUtils.Show("Die Zeit des RTC konnte nicht gesetzt werden.");
            }
        }

        /// <summary>
        /// Reads the time from the RTC and sets the GUI.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadTime_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.dateTimePicker.DateTimeSelected = this.DeviceRtc.ReadTime();
            }
            catch
            {
                ErrorUtils.Show("Die Zeit des RTC konnte nicht ausgelesen werden.");
                return;
            }
        }

        /// <summary>
        /// Reads and resets interrupt flags of the RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadAndResetInterrupts_Click(object sender, RoutedEventArgs e)
        {
            bool interruptFlagAIF, interruptFlagPIF, interruptFlagDCFIF;
            try
            {
                this.DeviceRtc.ReadAndResetInterruptFlags(out interruptFlagAIF, out interruptFlagPIF, out interruptFlagDCFIF);
            }
            catch
            {
                ErrorUtils.Show("Die Interruptflags des RTC konnte nicht ausgelesen und resettet werden.");
                return;
            }

            this.InterruptFlagAIF = interruptFlagAIF;
            this.InterruptFlagPIF = interruptFlagPIF;
            this.InterruptFlagDCFIF = interruptFlagDCFIF;
        }

        /// <summary>
        /// Sets alarm of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetAlarm_Click(object sender, RoutedEventArgs e)
        {
            int minute, hour;
            try
            {
                this.GetAlarmTime(out minute, out hour);
            }
            catch (ArgumentException ex)
            {
                ErrorUtils.Show(ex);
                return;
            }
            catch
            {
                ErrorUtils.Show("Die Alarmzeit des RTC konnte nicht gesetzt werden.");
                return;
            }

            this.DeviceRtc.SetAlarmSettings(new _RTC.AlarmSettings()
            {
                Hour = hour,
                Minute = minute,
                Mo = this.AlarmMo,
                Di = this.AlarmDi,
                Mi = this.AlarmMi,
                Do = this.AlarmDo,
                Fr = this.AlarmFr,
                Sa = this.AlarmSa,
                So = this.AlarmSo,
                AlarmAIE = this.AlarmAIE,
                AlarmAILED = this.AlarmAILED
            });
        }

        /// <summary>
        /// Reads alarm of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadAlarm_Click(object sender, RoutedEventArgs e)
        {
            _RTC.AlarmSettings alarmSettings;
            try
            {
                alarmSettings = this.DeviceRtc.ReadAlarmSettings();
            }
            catch
            {
                ErrorUtils.Show("Die Alarmzeit des RTC konnte nicht ausgelesen werden.");
                return;
            }

            this.AlarmTime = string.Format("{0:D2}:{1:D2}", alarmSettings.Hour, alarmSettings.Minute);

            this.AlarmMo = alarmSettings.Mo;
            this.AlarmDi = alarmSettings.Di;
            this.AlarmMi = alarmSettings.Mi;
            this.AlarmDo = alarmSettings.Do;
            this.AlarmFr = alarmSettings.Fr;
            this.AlarmSa = alarmSettings.Sa;
            this.AlarmSo = alarmSettings.So;

            this.AlarmAIE = alarmSettings.AlarmAIE;
            this.AlarmAILED = alarmSettings.AlarmAILED;
        }

        /// <summary>
        /// Gets a valid alarm time from the GUI.
        /// </summary>
        /// <param name="minute">Minute alarm value.</param>
        /// <param name="hour">Hour alarm value.</param>
        private void GetAlarmTime(out int minute, out int hour)
        {
            const char TimeSplitter = ':';

            if (!this.AlarmTime.Contains(TimeSplitter))
            {
                throw new ArgumentException("Die Alarmzeit enthält keinen Doppelpunkt.");
            }

            string[] split = this.AlarmTime.Split(TimeSplitter);
            if (split.Length != 2)
            {
                throw new ArgumentException("Die Alarmzeit enthält keinen oder mehr als einen Doppelpunkt.");
            }

            try
            {
                hour = int.Parse(split[0]);
                minute = int.Parse(split[1]);
            }
            catch (Exception)
            {
                throw new ArgumentException("Der Minuten- oder Stundenwert der Alarmzeit ist ungültig.");
            }

            if (!(hour >= 0 && hour <= 23))
            {
                throw new ArgumentException(string.Format("Der Stundenwert der Alarmzeit ist ungültig."));
            }
            else if (!(minute >= 0 && minute <= 59))
            {
                throw new ArgumentException(string.Format("Der Minutenwert der Alarmzeit ist ungültig."));
            }
        }

        /// <summary>
        /// Sets periodical interrupt of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetPeriodicalInterrupt_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.DeviceRtc.SetPeriodicalInterrupt(this.PeriodicalInterruptFrequency, this.AlarmPILED, this.AlarmPIE);
            }
            catch
            {
                ErrorUtils.Show("Der periodische Interrupt des RTC konnte nicht gesetzt werden.");
            }
        }

        /// <summary>
        /// Reads periodical interrupt of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadPeriodicalInterrupt_Click(object sender, RoutedEventArgs e)
        {
            _RTC.PeriodicalInterruptFrequency periodicalInterruptFrequency;
            bool alarmPILED, alarmPIE;

            try
            {
                this.DeviceRtc.ReadPeriodicalInterrupt(out periodicalInterruptFrequency, out alarmPILED, out alarmPIE);
            }
            catch
            {
                ErrorUtils.Show("Der periodische Interrupt des RTC konnte nicht ausgelesen werden.");
                return;
            }

            this.PeriodicalInterruptFrequency = periodicalInterruptFrequency;
            this.AlarmPILED = alarmPILED;
            this.AlarmPIE = alarmPIE;
        }

        /// <summary>
        /// Sets dcf settings of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetDcf_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                this.DeviceRtc.SetDcf(this.EnableDcf, this.DCFLED, this.DCFIE);
            }
            catch
            {
                ErrorUtils.Show("Die DCF-Einstellungen des RTC konnten nicht gesetzt werden");
            }
        }

        /// <summary>
        /// Reads dcf settings of RTC.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadDcf_Click(object sender, RoutedEventArgs e)
        {
            bool enableDcf, dcfLed, dcfIe;

            try
            {
                this.DeviceRtc.ReadDcf(out enableDcf, out dcfLed, out dcfIe);
            }
            catch
            {
                ErrorUtils.Show("Die DCF-Einstellungen des RTC konnten nicht ausgelesen werden");
                return;
            }

            this.EnableDcf = enableDcf;
            this.DCFLED = dcfLed;
            this.DCFIE = dcfIe;
        }

        /// <summary>
        /// Toggles automatic reading of time and interrupt flags.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ToggleAutoReadTimeAndInterrupt_Click(object sender, RoutedEventArgs e)
        {
            const int MinAutoReadTimeAndInterruptIntervall = 750; // ms

            Button btn = (Button)sender;

            if (this.readTimeAndInterruptTimer == null || !this.readTimeAndInterruptTimer.Enabled)
            {
                if (this.AutoReadTimeAndInterruptIntervall < MinAutoReadTimeAndInterruptIntervall)
                {
                    ErrorUtils.Show(string.Format("Das Intervall zum automatischen Auslesen von Uhrzeit und Interrupts muss mindestens {0} ms betragen.", MinAutoReadTimeAndInterruptIntervall));
                    return;
                }

                this.readTimeAndInterruptTimer = new Timer(this.AutoReadTimeAndInterruptIntervall);
                this.readTimeAndInterruptTimer.Elapsed += new ElapsedEventHandler(this.ReadTimeAndInterruptTimer_Elapsed);
                this.readTimeAndInterruptTimer.Start();

                btn.Content = "beenden";
            }
            else
            {
                this.readTimeAndInterruptTimer.Stop();

                btn.Content = "starten";
            }
        }

        /// <summary>
        /// Timer-method of automatic reading of time and interrupt flags.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ReadTimeAndInterruptTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            if (!this.uiElement.Dispatcher.CheckAccess())
            {
                this.uiElement.Dispatcher.BeginInvoke(
                    new Action<object, ElapsedEventArgs>(this.ReadTimeAndInterruptTimer_Elapsed),
                    new object[] { sender, e });
                return;
            }

            this.ReadTime_Click(null, null);
            this.ReadAndResetInterrupts_Click(null, null);
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class TabContent_RTC
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for InterruptFlagAIF.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty InterruptFlagAIFProperty =
            DependencyProperty.Register("InterruptFlagAIF", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for InterruptFlagPIF.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty InterruptFlagPIFProperty =
            DependencyProperty.Register("InterruptFlagPIF", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for InterruptFlagDCFIF.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty InterruptFlagDCFIFProperty =
            DependencyProperty.Register("InterruptFlagDCFIF", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmTime.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmTimeProperty =
            DependencyProperty.Register("AlarmTime", typeof(string), typeof(TabContent_RTC), new UIPropertyMetadata("13:37"));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmMo.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmMoProperty =
            DependencyProperty.Register("AlarmMo", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmDi.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmDiProperty =
            DependencyProperty.Register("AlarmDi", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmMi.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmMiProperty =
            DependencyProperty.Register("AlarmMi", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmDo.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmDoProperty =
            DependencyProperty.Register("AlarmDo", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmFr.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmFrProperty =
            DependencyProperty.Register("AlarmFr", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmSa.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmSaProperty =
            DependencyProperty.Register("AlarmSa", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmSo.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmSoProperty =
            DependencyProperty.Register("AlarmSo", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmAILED.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmAILEDProperty =
            DependencyProperty.Register("AlarmAILED", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmAIE.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmAIEProperty =
            DependencyProperty.Register("AlarmAIE", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for PeriodicalInterruptFrequency.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty PeriodicalInterruptFrequencyProperty =
            DependencyProperty.Register("PeriodicalInterruptFrequency", typeof(_RTC.PeriodicalInterruptFrequency), typeof(TabContent_RTC), new UIPropertyMetadata(_RTC.PeriodicalInterruptFrequency.EachDay));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmPILED.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmPILEDProperty =
            DependencyProperty.Register("AlarmPILED", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AlarmPIE.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AlarmPIEProperty =
            DependencyProperty.Register("AlarmPIE", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for EnableDcf.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty EnableDcfProperty =
            DependencyProperty.Register("EnableDcf", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for DCFLED.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty DCFLEDProperty =
            DependencyProperty.Register("DCFLED", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for DCFIE.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty DCFIEProperty =
            DependencyProperty.Register("DCFIE", typeof(bool), typeof(TabContent_RTC), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Time.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty TimeProperty =
            DependencyProperty.Register("Time", typeof(DateTime), typeof(TabContent_RTC), new UIPropertyMetadata(DateTime.Now));

        /// <summary>
        /// Using a DependencyProperty as the backing store for AutoReadTimeAndInterruptIntervall.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty AutoReadTimeAndInterruptIntervallProperty =
            DependencyProperty.Register("AutoReadTimeAndInterruptIntervall", typeof(int), typeof(TabContent_RTC), new UIPropertyMetadata(1000));

        /// <summary>
        /// Gets or sets a value indicating whether the interrupt AIF flag is set or not.
        /// </summary>
        /// <value>A value indicating whether the interrupt AIF flag is set or not.</value>
        public bool InterruptFlagAIF
        {
            get { return (bool)GetValue(InterruptFlagAIFProperty); }
            set { SetValue(InterruptFlagAIFProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the interrupt PIF flag is set or not.
        /// </summary>
        /// <value>A value indicating whether the interrupt PIF flag is set or not.</value>
        public bool InterruptFlagPIF
        {
            get { return (bool)GetValue(InterruptFlagPIFProperty); }
            set { SetValue(InterruptFlagPIFProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the interrupt DCFIF flag is set or not.
        /// </summary>
        /// <value>A value indicating whether the interrupt DCFIF flag is set or not.</value>
        public bool InterruptFlagDCFIF
        {
            get { return (bool)GetValue(InterruptFlagDCFIFProperty); }
            set { SetValue(InterruptFlagDCFIFProperty, value); }
        }

        /// <summary>
        /// Gets or sets current alarm time.
        /// </summary>
        /// <value>Current alarm time.</value>
        public string AlarmTime
        {
            get { return (string)GetValue(AlarmTimeProperty); }
            set { SetValue(AlarmTimeProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for monday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for monday is set or not.</value>
        public bool AlarmMo
        {
            get { return (bool)GetValue(AlarmMoProperty); }
            set { SetValue(AlarmMoProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for tuesday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for tuesday is set or not.</value>
        public bool AlarmDi
        {
            get { return (bool)GetValue(AlarmDiProperty); }
            set { SetValue(AlarmDiProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for wednesday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for wednesday is set or not.</value>
        public bool AlarmMi
        {
            get { return (bool)GetValue(AlarmMiProperty); }
            set { SetValue(AlarmMiProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for thursday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for thursday is set or not.</value>
        public bool AlarmDo
        {
            get { return (bool)GetValue(AlarmDoProperty); }
            set { SetValue(AlarmDoProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for friday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for friday is set or not.</value>
        public bool AlarmFr
        {
            get { return (bool)GetValue(AlarmFrProperty); }
            set { SetValue(AlarmFrProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for saturday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for saturday is set or not.</value>
        public bool AlarmSa
        {
            get { return (bool)GetValue(AlarmSaProperty); }
            set { SetValue(AlarmSaProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag for sunday is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag for sunday is set or not.</value>
        public bool AlarmSo
        {
            get { return (bool)GetValue(AlarmSoProperty); }
            set { SetValue(AlarmSoProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag AILED is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag AILED is set or not.</value>
        public bool AlarmAILED
        {
            get { return (bool)GetValue(AlarmAILEDProperty); }
            set { SetValue(AlarmAILEDProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag AIE is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag AIE is set or not.</value>
        public bool AlarmAIE
        {
            get { return (bool)GetValue(AlarmAIEProperty); }
            set { SetValue(AlarmAIEProperty, value); }
        }

        /// <summary>
        /// Gets or sets periodical interrupt frequency.
        /// </summary>
        /// <value>Periodical interrupt frequency.</value>
        public _RTC.PeriodicalInterruptFrequency PeriodicalInterruptFrequency
        {
            get { return (_RTC.PeriodicalInterruptFrequency)GetValue(PeriodicalInterruptFrequencyProperty); }
            set { SetValue(PeriodicalInterruptFrequencyProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag PILED is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag PILED is set or not.</value>
        public bool AlarmPILED
        {
            get { return (bool)GetValue(AlarmPILEDProperty); }
            set { SetValue(AlarmPILEDProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the alarm flag PIE is set or not.
        /// </summary>
        /// <value>A value indicating whether the alarm flag PIE is set or not.</value>
        public bool AlarmPIE
        {
            get { return (bool)GetValue(AlarmPIEProperty); }
            set { SetValue(AlarmPIEProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether dcf is enabled or not.
        /// </summary>
        /// <value>A value indicating whether dcf is enabled or not.</value>
        public bool EnableDcf
        {
            get { return (bool)GetValue(EnableDcfProperty); }
            set { SetValue(EnableDcfProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the dcf LED flag is set or not.
        /// </summary>
        /// <value>A value indicating whether the dcf LED flag is set or not.</value>
        public bool DCFLED
        {
            get { return (bool)GetValue(DCFLEDProperty); }
            set { SetValue(DCFLEDProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the dcf IE flag is set or not.
        /// </summary>
        /// <value>A value indicating whether the dcf IE flag is set or not.</value>
        public bool DCFIE
        {
            get { return (bool)GetValue(DCFIEProperty); }
            set { SetValue(DCFIEProperty, value); }
        }

        /// <summary>
        /// Gets or sets read time of the RTC.
        /// </summary>
        /// <value>Read time of the RTC.</value>
        public DateTime Time
        {
            get { return (DateTime)GetValue(TimeProperty); }
            set { SetValue(TimeProperty, value); }
        }

        /// <summary>
        /// Gets or sets intervall of automatic reading of time and interrupt flags.
        /// </summary>
        /// <value>Intervall of automatic reading of time and interrupt flags.</value>
        public int AutoReadTimeAndInterruptIntervall
        {
            get { return (int)GetValue(AutoReadTimeAndInterruptIntervallProperty); }
            set { SetValue(AutoReadTimeAndInterruptIntervallProperty, value); }
        }
    }
}
