﻿//-----------------------------------------------------------------------------
// <copyright file="InfoDialog.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Dialog for showing version information.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool
{
    using System;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Globalization;
    using System.Windows;
    using Microsoft.Win32;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Dialog for showing version information.
    /// </summary>
    public partial class InfoDialog : Window
    {
        /// <summary>
        /// UsbInterface for communication with the interface.
        /// </summary>
        private UsbInterface usbInterface;

        /// <summary>
        /// Initializes a new instance of the InfoDialog class.
        /// </summary>
        /// <param name="usbInterface">UsbInterface for communication with the interface.</param>
        /// <param name="isConnected">A value indicating whether a connection to a USB interface is established or not.</param>
        public InfoDialog(UsbInterface usbInterface, bool isConnected)
        {
            this.InitializeComponent();

            this.usbInterface = usbInterface;
            this.IsConnected = isConnected;

            this.RefreshVersionStrings();

            this.DataContext = this;
        }

        /// <summary>
        /// Refreshes the strings for software and firmware version.
        /// </summary>
        private void RefreshVersionStrings()
        {
            this.FirmwareVersionString = string.Format(
                "Firmware-Version: {0}",
                this.usbInterface != null ? this.usbInterface.Version : "-");

            this.SoftwareVersionString = string.Format(
                "Software-Version: {0} ({1:dd.MM.yyyy})",
                VersionInfo.Version.ToString("0.0", CultureInfo.CreateSpecificCulture("en-US")),
                VersionInfo.Date);
        }

        /// <summary>
        /// Closes the info dialog.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void CloseInfo_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Handles the navigation of the hyperlink.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void Hyperlink_RequestNavigate(object sender, System.Windows.Navigation.RequestNavigateEventArgs e)
        {
            Process.Start(new ProcessStartInfo(e.Uri.AbsoluteUri));
            e.Handled = true;
        }

        /// <summary>
        /// Starts the bootloader of the USB interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void StartBootloader_Click(object sender, RoutedEventArgs e)
        {
            var result = MessageBox.Show(
                "Wollen Sie den Bootloader wirklich starten?\nWenn der Bootloader gestartet ist, können Sie nur über Trennung und nachfolgender Wiederherstellung der Spannungsversorgung oder durch ein Update der Firmware das Gerät wieder benutzen.",
                "Warnung",
                MessageBoxButton.OKCancel,
                MessageBoxImage.Warning);

            if (result != MessageBoxResult.OK)
            {
                return;
            }

            this.usbInterface.EnterBootloader();
        }

        /// <summary>
        /// Starts a firmware update of the USB interface.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void StartFirmwareUpdate_Click(object sender, RoutedEventArgs e)
        {
            var result = MessageBox.Show(
                "Um ein Firmwareupdate für das USB-Interface zu starten, muss das Gerät sich im Bootloader befinden. Entfernen Sie bitte vor dem Update alle Komponenten von den zwei SPI-Bus-Buchsen.\nWollen Sie den Vorgang wirklich fortsetzen?",
                "Warnung",
                MessageBoxButton.OKCancel,
                MessageBoxImage.Warning);

            if (result != MessageBoxResult.OK)
            {
                return;
            }

            OpenFileDialog openFileDialog = new OpenFileDialog()
            {
                Filter = "Firmware-Dateien|*.enc",
                Title = "Öffnen einer Firmware-Datei",
                Multiselect = false
            };

            if (openFileDialog.ShowDialog() != true)
            {
                return;
            }

            new FirmareUpdateProgressWindow(this.usbInterface, openFileDialog.FileName).ShowDialog();
            this.RefreshVersionStrings();
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class InfoDialog
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for IsConnected.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty IsConnectedProperty =
            DependencyProperty.Register("IsConnected", typeof(bool), typeof(InfoDialog), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SoftwareVersionString.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SoftwareVersionStringProperty =
            DependencyProperty.Register("SoftwareVersionString", typeof(string), typeof(InfoDialog), new UIPropertyMetadata("-"));

        /// <summary>
        /// Using a DependencyProperty as the backing store for FirmwareVersionString.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty FirmwareVersionStringProperty =
            DependencyProperty.Register("FirmwareVersionString", typeof(string), typeof(InfoDialog), new UIPropertyMetadata("-"));

        /// <summary>
        /// Gets or sets a value indicating whether a connection to a USB interface is established or not.
        /// </summary>
        /// <value>A value indicating whether a connection to a USB interface is established or not.</value>
        public bool IsConnected
        {
            get { return (bool)GetValue(IsConnectedProperty); }
            set { SetValue(IsConnectedProperty, value); }
        }

        /// <summary>
        /// Gets or sets displayed software version.
        /// </summary>
        /// <value>Displayed software version.</value>
        public string SoftwareVersionString
        {
            get { return (string)GetValue(SoftwareVersionStringProperty); }
            set { SetValue(SoftwareVersionStringProperty, value); }
        }

        /// <summary>
        /// Gets or sets displayed firmware version.
        /// </summary>
        /// <value>Displayed firmware version.</value>
        public string FirmwareVersionString
        {
            get { return (string)GetValue(FirmwareVersionStringProperty); }
            set { SetValue(FirmwareVersionStringProperty, value); }
        }
    }
}
