﻿//-----------------------------------------------------------------------------
// <copyright file="MainWindow.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>MainWindow of the GUI.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.IO.Ports;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Media;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.GUI;
    using UsbInterfaceTesttool.SerialCommunication;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// MainWindow of the GUI.
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Initializes a new instance of the MainWindow class.
        /// </summary>
        public MainWindow()
        {
            this.InitializeComponent();
            this.InitializeTabContents();

            this.Title = string.Format("ELV {0}-Testtool", App.UsbInterfaceType);
            this.SerialPortSettings = SerialPortSettings.GetInstance();

            this.RefreshComPortList();

            this.DataContext = this;
        }

        /// <summary>
        /// Delegate used for the AddressChanged event.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Address information.</param>
        public delegate void AddressChangedHandler(object sender, AddressEvent e);

        /// <summary>
        /// Event triggered when the chosen address in the GUI is changed.
        /// </summary>
        public event AddressChangedHandler AddressChanged;

        /// <summary>
        /// Gets or sets wrapper for communication with 3D-BS.
        /// </summary>
        /// <value>Wrapper for communication with 3D-BS.</value>
        public static _3DBS Device3DBS { get; set; }

        /// <summary>
        /// Gets or sets wrapper for communication with 6D-BS.
        /// </summary>
        /// <value>Wrapper for communication with 6D-BS.</value>
        public static _6DBS Device6DBS { get; set; }

        /// <summary>
        /// Gets or sets wrapper for communication with RTC.
        /// </summary>
        /// <value>Wrapper for communication with RTC.</value>
        public static _RTC DeviceRTC { get; set; }

        /// <summary>
        /// Gets or sets UsbInterface for communication with the interface.
        /// </summary>
        /// <value>UsbInterface for communication with the interface.</value>
        public UsbInterface UsbInterface { get; set; }

        /// <summary>
        /// Gets or sets GUI-wrapper of 3D-BS.
        /// </summary>
        /// <value>GUI-wrapper of 3D-BS.</value>
        public TabContent_3DBS TabContent_3DBS { get; set; }

        /// <summary>
        /// Gets or sets GUI-wrapper of 6D-BS.
        /// </summary>
        /// <value>GUI-wrapper of 6D-BS.</value>
        public TabContent_6DBS TabContent_6DBS { get; set; }

        /// <summary>
        /// Gets or sets GUI-wrapper of RTC-DCF.
        /// </summary>
        /// <value>GUI-wrapper of RTC-DCF.</value>
        public TabContent_RTC TabContent_RTC { get; set; }

        /// <summary>
        /// Gets the data context of a MainWindow from a UserControl.
        /// </summary>
        /// <param name="userControl">UserControl to use.</param>
        /// <returns>MainWindow parent.</returns>
        public static MainWindow GetDataContext(UserControl userControl)
        {
            var parent = userControl.Parent;
            do
            {
                parent = VisualTreeHelper.GetParent(parent);
            }
            while (!(parent is MainWindow));

            return (MainWindow)parent;
        }

        /// <summary>
        /// Initializes content of different tab pages.
        /// </summary>
        private void InitializeTabContents()
        {
            this.TabContent_3DBS = new TabContent_3DBS();
            this.TabContent_6DBS = new TabContent_6DBS();
            this.TabContent_RTC = new TabContent_RTC();
        }

        /// <summary>
        /// Sets communication parameter for the current chosen device.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SetDeviceInterfaceParameter_Click(object sender, RoutedEventArgs e)
        {
            List<string> interfaceParameter = new List<string>();

            switch ((string)((TabItem)this.tcMain.SelectedItem).Header)
            {
                case "3D-BS":
                    interfaceParameter.Add("Y11");
                    interfaceParameter.Add("Y21");
                    interfaceParameter.Add("T00625");
                    break;

                case "6D-BS":
                    interfaceParameter.Add("Y11");
                    interfaceParameter.Add("Y21");
                    interfaceParameter.Add("T01250");
                    break;

                case "RTC-DCF":
                    interfaceParameter.Add("Y10");
                    interfaceParameter.Add("Y21");
                    interfaceParameter.Add("T05000");
                    break;
            }

            try
            {
                foreach (var parameter in interfaceParameter)
                {
                    this.UsbInterface.Write(parameter);
                }

                this.UsbInterface.Read();
            }
            catch
            {
                ErrorUtils.Show("Die Parameter für das Gerät konnten nicht korrekt initialisiert werden.");
            }
        }

        /// <summary>
        /// Opens a dialog for showing version information.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void OpenInfoDialog_Click(object sender, RoutedEventArgs e)
        {
            InfoDialog window = new InfoDialog(this.UsbInterface, this.IsConnected);
            window.Owner = this;
            window.ShowDialog();
        }

        #region COM-Port operations
        /// <summary>
        /// Refreshes the list of ComPorts.
        /// </summary>
        private void RefreshComPortList()
        {
            List<string> comPorts = new List<string>(SerialPort.GetPortNames());
            comPorts.Sort();

            this.ComPorts = new ObservableCollection<string>(comPorts);
            if (this.ComPorts.Count != 0)
            {
                this.SelectedComPort = this.ComPorts[0];
            }
        }

        /// <summary>
        /// Connects to a ComPort.
        /// </summary>
        /// <returns>Result of the action.</returns>
        private bool Connect()
        {
            try
            {
                this.UsbInterface = new UsbInterface(this.SelectedComPort, this.SerialPortSettings);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Fehler", MessageBoxButton.OK, MessageBoxImage.Error);
                return false;
            }

            this.UsbInterface.DataRead += new UsbInterface.DataHandler(this.ShowReadData);
            this.UsbInterface.DataWritten += new UsbInterface.DataHandler(this.ShowWroteData);
            this.UsbInterface.OnError += new UsbInterface.OnErrorHandler(this.UsbInterface_OnError);

            this.IsConnected = true;
            this.InitDevices();

            if (!this.UsbInterface.ShowStatusFirmware())
            {
                this.Disconnect();
                ErrorUtils.Show(string.Format("Auf {0} wurde kein USB-Interface erkannt.", this.SelectedComPort));

                return false;
            }

            return true;
        }

        /// <summary>
        /// Disconnects from a ComPort.
        /// </summary>
        /// <returns>Result of the action.</returns>
        private bool Disconnect()
        {
            this.StopAutomaticTriggering();

            this.UsbInterface.Disconnect();
            this.IsConnected = false;

            return true;
        }

        /// <summary>
        /// Stops automatic triggering of devices.
        /// </summary>
        private void StopAutomaticTriggering()
        {
            IStoppable[] tabContents = new IStoppable[]
            {
                this.TabContent_3DBS,
                this.TabContent_6DBS,
                this.TabContent_RTC
            };

            foreach (var tabContent in tabContents)
            {
                tabContent.StoppDeviceCommunication();
            }
        }

        /// <summary>
        /// Toggles ComPort connection.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void BtnComPortConnection_Click(object sender, RoutedEventArgs e)
        {
            Button btn = (Button)sender;

            if (this.IsConnected)
            {
                if (this.Disconnect())
                {
                    btn.Content = "verbinden";
                }
            }
            else
            {
                if (this.Connect())
                {
                    btn.Content = "trennen";
                }
            }
        }

        /// <summary>
        /// Disconnects from the ComPort before the window gets closed.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (this.IsConnected)
            {
                this.Disconnect();
            }
        }

        /// <summary>
        /// Shows read data in the fitting list box.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ShowReadData(object sender, StreamEvent e)
        {
            if (string.IsNullOrEmpty(e.Data))
            {
                return;
            }
            else if (!this.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(new UsbInterface.DataHandler(this.ShowReadData), new object[] { sender, e });
                return;
            }

            this.ReadData.Add(e.Data);
            this.ScrollReadDataToEnd();
        }

        /// <summary>
        /// Shows written data in the fitting list box.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ShowWroteData(object sender, StreamEvent e)
        {
            if (string.IsNullOrEmpty(e.Data))
            {
                return;
            }
            else if (!this.Dispatcher.CheckAccess())
            {
                this.Dispatcher.BeginInvoke(new UsbInterface.DataHandler(this.ShowWroteData), new object[] { sender, e });
                return;
            }

            this.WroteData.Add(e.Data);
            this.ScrollWroteDataToEnd();
        }

        /// <summary>
        /// Scrolls to the end of the list box of read data.
        /// </summary>
        private void ScrollReadDataToEnd()
        {
            var scrollViewer = GuiUtils.GetScrollViewer(this.lbReceive);
            scrollViewer.ScrollToEnd();
        }

        /// <summary>
        /// Scrolls to the end of the list box of wrote data.
        /// </summary>
        private void ScrollWroteDataToEnd()
        {
            var scrollViewer = GuiUtils.GetScrollViewer(this.lbTransmit);
            scrollViewer.ScrollToEnd();
        }

        /// <summary>
        /// Shows an error during ComPort using.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="error">Error message.</param>
        private void UsbInterface_OnError(object sender, UsbInterfaceError error)
        {
            if (!this.IsConnected)
            {
                return;
            }

            if (error.ShouldDisconnect)
            {
                this.BtnComPortConnection_Click(this.btnConnect, null);
                this.RefreshComPortList();
            }
        }

        /// <summary>
        /// Clears the list box which contains read or written commands.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ClearReadWroteData_Click(object sender, RoutedEventArgs e)
        {
            Button btn = (Button)sender;
            switch (btn.Tag as string)
            {
                case "read":
                    this.ReadData.Clear();
                    break;
                case "wrote":
                    this.WroteData.Clear();
                    break;
                default:
                    break;
            }
        }

        /// <summary>
        /// Edits the current settings to connect to a ComPort.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void EditSerialPortSettings_Click(object sender, RoutedEventArgs e)
        {
            EditSerialPortSettingsWindow window = new EditSerialPortSettingsWindow(this.SerialPortSettings);
            if (window.ShowDialog() == true)
            {
                this.SerialPortSettings = window.SerialPortSettings;
            }

            Console.WriteLine(this.SerialPortSettings);
        }

        /// <summary>
        /// Refreshes the list of ComPorts if the appropriate combo box is opened.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void ComboBox_DropDownOpened(object sender, EventArgs e)
        {
            this.RefreshComPortList();
        }
        #endregion

        #region get address from GUI
        /// <summary>
        /// Toggles selection of spi address for 6D-BS gyro and acc.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void RadioButtonGetSpiAdressfor6DBS_Checked(object sender, RoutedEventArgs e)
        {
            RadioButton rb = (RadioButton)sender;
            if (this.rdGyroA == null)
            {
                return;
            }

            if (rb.GroupName == "ACC")
            {
                if (this.rdAccA.IsChecked == true)
                {
                    this.rdGyroB.IsChecked = true;

                    this.SpiAddressAcc = UsbInterface.SpiAddress.A;
                    this.SpiAddressGyro = UsbInterface.SpiAddress.B;
                }
                else
                {
                    this.rdGyroA.IsChecked = true;

                    this.SpiAddressAcc = UsbInterface.SpiAddress.B;
                    this.SpiAddressGyro = UsbInterface.SpiAddress.A;
                }
            }
            else
            {
                if (this.rdGyroA.IsChecked == true)
                {
                    this.rdAccB.IsChecked = true;

                    this.SpiAddressAcc = UsbInterface.SpiAddress.B;
                    this.SpiAddressGyro = UsbInterface.SpiAddress.A;
                }
                else
                {
                    this.rdAccA.IsChecked = true;

                    this.SpiAddressAcc = UsbInterface.SpiAddress.A;
                    this.SpiAddressGyro = UsbInterface.SpiAddress.B;
                }
            }

            if (this.AddressChanged != null)
            {
                this.AddressChanged(this, new AddressEvent(typeof(_6DBS), new string[] { this.SpiAddressAcc.ToString(), this.SpiAddressGyro.ToString() }));
            }
        }

        /// <summary>
        /// Triggers an event if the current SPI address is changed.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void SelectSingleSpiAddress_Checked(object sender, RoutedEventArgs e)
        {
            Type relevantType;

            TabItem currentTabItem = (TabItem)this.tcMain.SelectedItem;
            switch (currentTabItem.Tag.ToString())
            {
                case "common":
                    return;

                case "3D-BS":
                    relevantType = typeof(_3DBS);
                    break;
                case "RTC":
                    relevantType = typeof(_RTC);
                    break;

                default:
                    throw new ArgumentException();
            }

            if (this.AddressChanged != null)
            {
                this.AddressChanged(this, new AddressEvent(relevantType, this.SpiAddress.ToString()));
            }
        }

        /// <summary>
        /// Refreshes address selection if the current device is changed.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void MainTabControl_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Type relevantType;

            TabItem currentTabItem = (TabItem)this.tcMain.SelectedItem;
            switch (currentTabItem.Tag.ToString())
            {
                case "common":
                    return;

                case "3D-BS":
                    relevantType = typeof(_3DBS);
                    if (App.UsbInterfaceType == UsbInterface.Type.SPI && this.AddressChanged != null)
                    {
                        this.AddressChanged(this, new AddressEvent(relevantType, this.SpiAddress.ToString()));
                    }

                    break;

                case "6D-BS":
                    relevantType = typeof(_6DBS);

                    string[] addresses = new string[]
                    {
                        this.SpiAddressAcc.ToString(),
                        this.SpiAddressGyro.ToString()
                    };

                    if (App.UsbInterfaceType == UsbInterface.Type.SPI && this.AddressChanged != null)
                    {
                        this.AddressChanged(this, new AddressEvent(relevantType, addresses));
                    }

                    break;

                case "RTC":
                    relevantType = typeof(_RTC);

                    if (App.UsbInterfaceType == UsbInterface.Type.SPI && this.AddressChanged != null)
                    {
                        this.AddressChanged(this, new AddressEvent(relevantType, this.SpiAddress.ToString()));
                    }

                    break;

                default:
                    throw new InvalidOperationException();
            }
        }
        #endregion

        #region init devices
        /// <summary>
        /// Initializes all communication wrapper of devices.
        /// </summary>
        private void InitDevices()
        {
            MainWindow.Device3DBS = new _3DBS(this.UsbInterface);
            this.AddressChanged += MainWindow.Device3DBS.AddressChanged;

            MainWindow.Device6DBS = new _6DBS(this.UsbInterface);
            this.AddressChanged += MainWindow.Device6DBS.AddressChanged;

            MainWindow.DeviceRTC = new _RTC(this.UsbInterface);
            this.AddressChanged += MainWindow.DeviceRTC.AddressChanged;
        }
        #endregion
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class MainWindow
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for IsConnected.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty IsConnectedProperty =
            DependencyProperty.Register("IsConnected", typeof(bool), typeof(MainWindow), new PropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ComPorts.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ComPortsProperty =
            DependencyProperty.Register("ComPorts", typeof(ObservableCollection<string>), typeof(MainWindow), new UIPropertyMetadata(new ObservableCollection<string>()));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SelectedComPort.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SelectedComPortProperty =
            DependencyProperty.Register("SelectedComPort", typeof(string), typeof(MainWindow), new UIPropertyMetadata(string.Empty));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ReadData.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ReadDataProperty =
            DependencyProperty.Register("ReadData", typeof(ObservableCollection<string>), typeof(MainWindow), new UIPropertyMetadata(new ObservableCollection<string>()));

        /// <summary>
        /// Using a DependencyProperty as the backing store for WroteData.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty WroteDataProperty =
            DependencyProperty.Register("WroteData", typeof(ObservableCollection<string>), typeof(MainWindow), new UIPropertyMetadata(new ObservableCollection<string>()));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SerialPortSettings.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SerialPortSettingsProperty =
            DependencyProperty.Register("SerialPortSettings", typeof(SerialPortSettings), typeof(MainWindow), new UIPropertyMetadata(null));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SpiAddress.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SpiAddressProperty =
            DependencyProperty.Register("SpiAddress", typeof(UsbInterface.SpiAddress), typeof(MainWindow), new UIPropertyMetadata(UsbInterface.SpiAddress.A));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SpiAddressAcc.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SpiAddressAccProperty =
            DependencyProperty.Register("SpiAddressAcc", typeof(UsbInterface.SpiAddress), typeof(MainWindow), new UIPropertyMetadata(UsbInterface.SpiAddress.A));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SpiAddressGyro.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SpiAddressGyroProperty =
            DependencyProperty.Register("SpiAddressGyro", typeof(UsbInterface.SpiAddress), typeof(MainWindow), new UIPropertyMetadata(UsbInterface.SpiAddress.A));

        /// <summary>
        /// Using a DependencyProperty as the backing store for SelectedDeviceControlIndex.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty SelectedDeviceControlIndexProperty =
            DependencyProperty.Register("SelectedDeviceControlIndex", typeof(int), typeof(MainWindow), new UIPropertyMetadata(0));

        /// <summary>
        /// Gets or sets a value indicating whether there is a connection to a ComPort or not.
        /// </summary>
        /// <value>A value indicating whether there is a connection to a ComPort or not.</value>
        public bool IsConnected
        {
            get { return (bool)GetValue(IsConnectedProperty); }
            set { SetValue(IsConnectedProperty, value); }
        }

        /// <summary>
        /// Gets or sets list of all available ComPorts.
        /// </summary>
        /// <value>List of all available ComPorts.</value>
        public ObservableCollection<string> ComPorts
        {
            get { return (ObservableCollection<string>)GetValue(ComPortsProperty); }
            set { SetValue(ComPortsProperty, value); }
        }

        /// <summary>
        /// Gets or sets selected index of the list of available ComPorts.
        /// </summary>
        /// <value>Selected index of the list of available ComPorts.</value>
        public string SelectedComPort
        {
            get { return (string)GetValue(SelectedComPortProperty); }
            set { SetValue(SelectedComPortProperty, value); }
        }

        /// <summary>
        /// Gets or sets list of read data.
        /// </summary>
        /// <value>List of read data.</value>
        public ObservableCollection<string> ReadData
        {
            get { return (ObservableCollection<string>)GetValue(ReadDataProperty); }
            set { SetValue(ReadDataProperty, value); }
        }

        /// <summary>
        /// Gets or sets list of written data.
        /// </summary>
        /// <value>List of written data.</value>
        public ObservableCollection<string> WroteData
        {
            get { return (ObservableCollection<string>)GetValue(WroteDataProperty); }
            set { SetValue(WroteDataProperty, value); }
        }

        /// <summary>
        /// Gets or sets current settings to connect to a ComPort.
        /// </summary>
        /// <value>Current settings to connect to a ComPort.</value>
        public SerialPortSettings SerialPortSettings
        {
            get { return (SerialPortSettings)GetValue(SerialPortSettingsProperty); }
            set { SetValue(SerialPortSettingsProperty, value); }
        }

        /// <summary>
        /// Gets or sets currently chosen spi address.
        /// </summary>
        /// <value>Currently chosen spi address.</value>
        public UsbInterface.SpiAddress SpiAddress
        {
            get { return (UsbInterface.SpiAddress)GetValue(SpiAddressProperty); }
            set { SetValue(SpiAddressProperty, value); }
        }

        /// <summary>
        /// Gets or sets currently chosen spi address for 6D-BS acc.
        /// </summary>
        /// <value>Currently chosen spi address for 6D-BS acc.</value>
        public UsbInterface.SpiAddress SpiAddressAcc
        {
            get { return (UsbInterface.SpiAddress)GetValue(SpiAddressAccProperty); }
            set { SetValue(SpiAddressAccProperty, value); }
        }

        /// <summary>
        /// Gets or sets currently chosen spi address for 6D-BS gyro.
        /// </summary>
        /// <value>Currently chosen spi address for 6D-BS gyro.</value>
        public UsbInterface.SpiAddress SpiAddressGyro
        {
            get { return (UsbInterface.SpiAddress)GetValue(SpiAddressGyroProperty); }
            set { SetValue(SpiAddressGyroProperty, value); }
        }

        /// <summary>
        /// Gets or sets selected index of currently chosen device control.
        /// </summary>
        /// <value>Selected index of currently chosen device control.</value>
        public int SelectedDeviceControlIndex
        {
            get { return (int)GetValue(SelectedDeviceControlIndexProperty); }
            set { SetValue(SelectedDeviceControlIndexProperty, value); }
        }
    }
}