﻿//-----------------------------------------------------------------------------
// <copyright file="FirmareUpdateProgressWindow.xaml.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Window for showing progress of firmware update.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.SerialCommunication
{
    using System;
    using System.ComponentModel;
    using System.Threading;
    using System.Windows;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Window for showing progress of firmware update.
    /// </summary>
    public partial class FirmareUpdateProgressWindow : Window
    {
        /// <summary>
        /// Info string for transmitting data for firmware upgrade.
        /// </summary>
        private const string DoFirmwareUpdate = "Daten werden übertragen, aktueller Fortschritt:";

        /// <summary>
        /// Info string for notifying that the USB interface is usable again.
        /// </summary>
        private const string WaitForDevice = "Bitte warten Sie, bis das USB-Interface wieder verfügbar ist.";

        /// <summary>
        /// UsbInterface for communication with the interface.
        /// </summary>
        private UsbInterface usbInterface;

        /// <summary>
        /// File path of the firmware file.
        /// </summary>
        private string firmwareFile;

        /// <summary>
        /// Indicates whether the action is completed and the window can be closed or not.
        /// </summary>
        private bool actionCompleted;

        /// <summary>
        /// Background worker for updating the firmware.
        /// </summary>
        private BackgroundWorker checkFirmwareUpdateWorker;

        /// <summary>
        /// Initializes a new instance of the FirmareUpdateProgressWindow class.
        /// </summary>
        /// <param name="usbInterface">UsbInterface for communication with the interface.</param>
        /// <param name="firmwareFile">File path of the firmware file.</param>
        public FirmareUpdateProgressWindow(UsbInterface usbInterface, string firmwareFile)
        {
            this.InitializeComponent();

            this.usbInterface = usbInterface;
            this.firmwareFile = firmwareFile;
            this.InitFirmwareUpdate();

            this.actionCompleted = false;

            this.DataContext = this;
        }

        /// <summary>
        /// Initializes the firmware update.
        /// </summary>
        private void InitFirmwareUpdate()
        {
            this.usbInterface.FirmwareUpdateProgress += new ProgressChangedEventHandler(this.UsbInterface_FirmwareUpdateProgress);
            this.usbInterface.FirmwareUpdateCompleted += new RunWorkerCompletedEventHandler(this.UsbInterface_FirmwareUpdateCompleted);
            this.usbInterface.DoFirmwareUpdate(this.firmwareFile);
        }

        /// <summary>
        /// Shows current progress of the firmware update in the GUI.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Event for passing the progress.</param>
        private void UsbInterface_FirmwareUpdateProgress(object sender, ProgressChangedEventArgs e)
        {
            this.uiElement.Dispatcher.BeginInvoke((Action)(() => { this.Progress = e.ProgressPercentage; }));
        }

        /// <summary>
        /// Shows the result of the firmware update and closes the window.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Result of the firmware update.</param>
        private void UsbInterface_FirmwareUpdateCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (this.actionCompleted)
            {
                return;
            }

            if (e.Result.ToString() == FirmwareUpdater.Success)
            {
                this.usbInterface.Disconnect();
                this.usbInterface.Connect();
            }

            this.uiElement.Dispatcher.BeginInvoke((Action)(() =>
            {
                if (e.Result != null && e.Result is string)
                {
                    string result = e.Result.ToString();
                    if (result == FirmwareUpdater.Success)
                    {
                        MessageBox.Show(result, "Info", MessageBoxButton.OK, MessageBoxImage.Information);
                        this.ProgressBarIsIndeterminate = true;
                        this.Info = FirmareUpdateProgressWindow.WaitForDevice;

                        this.checkFirmwareUpdateWorker = new BackgroundWorker();
                        this.checkFirmwareUpdateWorker.DoWork += new DoWorkEventHandler(CheckFirmwareUpdateWorker_DoWork);
                        this.checkFirmwareUpdateWorker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(CheckFirmwareUpdateWorker_RunWorkerCompleted);
                        this.checkFirmwareUpdateWorker.RunWorkerAsync();
                    }
                    else
                    {
                        ErrorUtils.Show(result);

                        this.actionCompleted = true;
                        this.Close();
                    }
                }
            }));
        }

        /// <summary>
        /// Waits for finishing to the firmware upgrade and shows the status and firmware information for read out firmware version.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void CheckFirmwareUpdateWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            const int WaitForDeviceToRespond = 5000; // ms

            Thread.Sleep(WaitForDeviceToRespond);
            this.usbInterface.ShowStatusFirmware();
        }

        /// <summary>
        /// Sets a flag that the window is closable.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void CheckFirmwareUpdateWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.actionCompleted = true;
            this.Close();
        }

        /// <summary>
        /// Closes the window.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            if (!this.actionCompleted)
            {
                e.Cancel = true;
            }
        }
    }

    /// <summary>
    /// Collection of used dependency properties.
    /// </summary>
    public partial class FirmareUpdateProgressWindow
    {
        /// <summary>
        /// Using a DependencyProperty as the backing store for Progress.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ProgressProperty =
            DependencyProperty.Register("Progress", typeof(int), typeof(FirmareUpdateProgressWindow), new UIPropertyMetadata(0));

        /// <summary>
        /// Using a DependencyProperty as the backing store for ProgressBarIsIndeterminate.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty ProgressBarIsIndeterminateProperty =
            DependencyProperty.Register("ProgressBarIsIndeterminate", typeof(bool), typeof(FirmareUpdateProgressWindow), new UIPropertyMetadata(false));

        /// <summary>
        /// Using a DependencyProperty as the backing store for Info.  This enables animation, styling, binding, etc...
        /// </summary>
        public static readonly DependencyProperty InfoProperty =
            DependencyProperty.Register("Info", typeof(string), typeof(FirmareUpdateProgressWindow), new UIPropertyMetadata(FirmareUpdateProgressWindow.DoFirmwareUpdate));

        /// <summary>
        /// Gets or sets current progress of the firmware update.
        /// </summary>
        /// <value>Current progress of the firmware update.</value>
        public int Progress
        {
            get { return (int)GetValue(ProgressProperty); }
            set { SetValue(ProgressProperty, value); }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the progressbar is in indeterminate mode or not.
        /// </summary>
        /// <value>A value indicating whether the progressbar is in indeterminate mode or not.</value>
        public bool ProgressBarIsIndeterminate
        {
            get { return (bool)GetValue(ProgressBarIsIndeterminateProperty); }
            set { SetValue(ProgressBarIsIndeterminateProperty, value); }
        }

        /// <summary>
        /// Gets or sets current progress of the firmware update.
        /// </summary>
        /// <value>Current progress of the firmware update.</value>
        public string Info
        {
            get { return (string)GetValue(InfoProperty); }
            set { SetValue(InfoProperty, value); }
        }
    }
}
