//-----------------------------------------------------------------------------
// <copyright file="FirmwareUpdater.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Updates the firmware of a ComPort device.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.SerialCommunication
{
    using System;
    using System.ComponentModel;
    using System.Globalization;
    using System.IO;
    using System.IO.Ports;
    using System.Threading;

    /// <summary>
    /// Updates the firmware of a ComPort device.
    /// </summary>
    public class FirmwareUpdater
    {
        /// <summary>
        /// String to display if the firmware update was successful.
        /// </summary>
        public const string Success = "Das Firmwareupdate wurde erfolgreich durchgefhrt.";

        /// <summary>
        /// Backgroundworker for updating the device.
        /// </summary>
        private BackgroundWorker updater;

        /// <summary>
        /// Device as SerialPort to update.
        /// </summary>
        private SerialPort serialPort;

        /// <summary>
        /// Initializes a new instance of the FirmwareUpdater class.
        /// </summary>
        /// <param name="serialPort">Device as SerialPort to update.</param>
        public FirmwareUpdater(SerialPort serialPort)
        {
            this.serialPort = serialPort;

            this.updater = new BackgroundWorker();
            this.updater.DoWork += new DoWorkEventHandler(this.Updater_DoWork);
            this.updater.WorkerReportsProgress = true;
        }

        /// <summary>
        /// Progress of firmware update changed.
        /// </summary>
        public event ProgressChangedEventHandler ProgressChanged
        {
            add
            {
                this.updater.ProgressChanged += value;
            }

            remove
            {
                this.updater.ProgressChanged -= value;
            }
        }

        /// <summary>
        /// Firmware update completed.
        /// </summary>
        public event RunWorkerCompletedEventHandler RunWorkerCompleted
        {
            add
            {
                this.updater.RunWorkerCompleted += value;
            }

            remove
            {
                this.updater.RunWorkerCompleted -= value;
            }
        }

        /// <summary>
        /// Starts the update process.
        /// </summary>
        /// <param name="firmwarefile">Name of the firmware file.</param>
        public void Start(string firmwarefile)
        {
            object[] args = new object[] { firmwarefile };

            this.updater.RunWorkerAsync(args);
        }

        /// <summary>
        /// Executes the firmware update.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void Updater_DoWork(object sender, DoWorkEventArgs e)
        {
            string firmwarefile = ((object[])e.Argument)[0].ToString();
            if (string.IsNullOrEmpty(firmwarefile))
            {
                return;
            }

            var serialPortSettings = SerialPortSettings.Parse(this.serialPort);

            this.serialPort.BaudRate = SerialPortSettings.FirmwareUpdateSettings.IntegerBaudRate;
            this.serialPort.DataBits = SerialPortSettings.FirmwareUpdateSettings.DataBits;
            this.serialPort.Parity = SerialPortSettings.FirmwareUpdateSettings.Parity;
            this.serialPort.StopBits = SerialPortSettings.FirmwareUpdateSettings.StopBits;

            byte[] filedata = this.ReadHexFile(firmwarefile);
            if (filedata == null)
            {
                e.Result = "Die Firwmaredatei konnte nicht gelesen werden.";
            }
            else
            {
                try
                {
                    if (this.serialPort.IsOpen)
                    {
                        this.serialPort.ReadTimeout = 10000;
                        this.serialPort.ReadExisting();
                        string result = string.Empty;
                        int framesize = 0;
                        for (int i = 0; i < filedata.Length; i += framesize)
                        {
                            framesize = (filedata[i] * 256) + filedata[i + 1] + 2;
                            this.serialPort.Write(filedata, i, framesize);
                            int response = this.serialPort.ReadChar();
                            if (response == 0x11)
                            {
                                this.updater.ReportProgress((i + framesize) * 100 / filedata.Length);
                            }
                            else
                            {
                                e.Result = "Die Firwmaredatei ist beschdigt (CRC Fehler).";
                                return;
                            }
                        }

                        e.Result = FirmwareUpdater.Success;
                    }
                    else
                    {
                        e.Result = "Die Verbindung zum Gert konnte nicht aufgebaut werden.";
                        return;
                    }
                }
                catch (Exception ex)
                {
                    e.Result = "Es ist ein Fehler in der Datenbertragung aufgetreten:\r\n" + ex.Message;
                }
            }
        }

        /// <summary>
        /// Reads a hex file.
        /// </summary>
        /// <param name="firmwareFile">Firmware file to read.</param>
        /// <returns>Read content as byte array.</returns>
        private byte[] ReadHexFile(string firmwareFile)
        {
            byte[] filedata = null;
            StreamReader tr = new StreamReader(firmwareFile);
            try
            {
                string data = tr.ReadToEnd();
                filedata = new byte[data.Length / 2];
                for (int i = 0; i < (data.Length / 2); i++)
                {
                    filedata[i] = byte.Parse(data.Substring(i * 2, 2), NumberStyles.AllowHexSpecifier);
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("Die Firwmaredatei konnte nicht gelesen werden.\r\n" + ex.Message);
                filedata = null;
            }
            finally
            {
                tr.Close();
            }

            return filedata;
        }
    }
}