﻿//-----------------------------------------------------------------------------
// <copyright file="SerialPortSettings.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper for settings to connect to ComPort.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.SerialCommunication
{
    using System;
    using System.ComponentModel;
    using System.IO.Ports;
    using UsbInterfaceTesttool.Devices;

    /// <summary>
    /// Wrapper for settings to connect to ComPort.
    /// </summary>
    public class SerialPortSettings : INotifyPropertyChanged
    {
        /// <summary>
        /// ComPort settings for firmware update.
        /// </summary>
        public static readonly SerialPortSettings FirmwareUpdateSettings = new SerialPortSettings()
        {
            BaudRate = GeneralTasks.InterfaceBaudRate._38400,
            Parity = Parity.None,
            DataBits = 8,
            StopBits = StopBits.One
        };

        /// <summary>
        /// Default ComPort settings for USB-I2C.
        /// </summary>
        private static readonly SerialPortSettings standardI2cInterface = new SerialPortSettings()
        {
            BaudRate = GeneralTasks.InterfaceBaudRate._38400, // 115200
            DataBits = 8,
            Parity = Parity.None,
            StopBits = StopBits.One
        };

        /// <summary>
        /// Default ComPort settings for USB-SPI.
        /// </summary>
        private static readonly SerialPortSettings standardSpiInterface = new SerialPortSettings()
        {
            BaudRate = GeneralTasks.InterfaceBaudRate._38400,
            DataBits = 8,
            Parity = Parity.None,
            StopBits = StopBits.One
        };

        /// <summary>
        /// Baud rate of the serial port.
        /// </summary>
        private GeneralTasks.InterfaceBaudRate baudRate;

        /// <summary>
        /// Data bits of the serial port.
        /// </summary>
        private int dataBits;

        /// <summary>
        /// Parity of the serial port.
        /// </summary>
        private Parity parity;

        /// <summary>
        /// Stop bits of the serial port.
        /// </summary>
        private StopBits stopBits;

        /// <summary>
        /// Event for notifying about changed property values.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Gets or sets baud rate of the serial port.
        /// </summary>
        /// <value>Baud rate of the serial port.</value>
        public GeneralTasks.InterfaceBaudRate BaudRate
        {
            get
            {
                return this.baudRate;
            }

            set
            {
                this.baudRate = value;
                this.NotifyPropertyChanged("BaudRate");
            }
        }

        /// <summary>
        /// Gets the current baud rate as integer.
        /// </summary>
        /// <value>The current baud rate as integer.</value>
        public int IntegerBaudRate
        {
            get
            {
                return GeneralTasks.GetBaudRate(this.BaudRate);
            }
        }

        /// <summary>
        /// Gets or sets data bits of the serial port.
        /// </summary>
        /// <value>Data bits of the serial port.</value>
        public int DataBits
        {
            get
            {
                return this.dataBits;
            }

            set
            {
                this.dataBits = value;
                this.NotifyPropertyChanged("DataBits");
            }
        }

        /// <summary>
        /// Gets or sets parity of the serial port.
        /// </summary>
        /// <value>Parity of the serial port.</value>
        public Parity Parity
        {
            get
            {
                return this.parity;
            }

            set
            {
                this.parity = value;
                this.NotifyPropertyChanged("Parity");
            }
        }

        /// <summary>
        /// Gets or sets stop bits of the serial port.
        /// </summary>
        /// <value>Stop bits of the serial port.</value>
        public StopBits StopBits
        {
            get
            {
                return this.stopBits;
            }

            set
            {
                this.stopBits = value;
                this.NotifyPropertyChanged("StopBits");
            }
        }

        /// <summary>
        /// Gets a instance based on the set USB-interface type.
        /// </summary>
        /// <returns>SerialPortSettings instance.</returns>
        public static SerialPortSettings GetInstance()
        {
            switch (App.UsbInterfaceType)
            {
                case UsbInterface.Type.I2C:
                    return standardI2cInterface;
                case UsbInterface.Type.SPI:
                    return standardSpiInterface;
                default:
                    throw new ArgumentException();
            }
        }

        /// <summary>
        /// Gets settings of a existing SerialPort.
        /// </summary>
        /// <param name="serialPort">SerialPort to parse.</param>
        /// <returns>Settings of a existing SerialPort.</returns>
        public static SerialPortSettings Parse(SerialPort serialPort)
        {
            return new SerialPortSettings()
            {
                BaudRate = GeneralTasks.GetIntegerBaudRate(serialPort.BaudRate),
                DataBits = serialPort.DataBits,
                Parity = serialPort.Parity,
                StopBits = serialPort.StopBits
            };
        }

        /// <summary>
        /// Notifies about changed property values.
        /// </summary>
        /// <param name="propertyName">Name of changed property.</param>
        public void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}