﻿//-----------------------------------------------------------------------------
// <copyright file="UsbInterface.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Wrapper for communication to a USB-interface.</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.SerialCommunication
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.IO;
    using System.IO.Ports;
    using System.Text;
    using System.Threading;
    using UsbInterfaceTesttool.Devices;
    using UsbInterfaceTesttool.Utils;

    /// <summary>
    /// Wrapper for communication to a USB-interface.
    /// </summary>
    public class UsbInterface
    {
        /// <summary>
        /// Object for locking ComPort-usage (thread safety).
        /// </summary>
        private readonly object comPortLocker = new object();

        /// <summary>
        /// Timeout for reading operation.
        /// </summary>
        private const int ReadTimeout = 250;

        /// <summary>
        /// Device as SerialPort to use.
        /// </summary>
        private SerialPort serialPort;

        /// <summary>
        /// Stream of the ComPort.
        /// </summary>
        private Stream baseStream;

        /// <summary>
        /// Reader of the ComPort-stream.
        /// </summary>
        private StreamReader reader;

        /// <summary>
        /// Writer of the ComPort-stream.
        /// </summary>
        private StreamWriter writer;

        /// <summary>
        /// BackgroundWorker for firmware update.
        /// </summary>
        private BackgroundWorker workerFirmwareUpdate;

        /// <summary>
        /// Chosen ComPort of the connection.
        /// </summary>
        private string comPort;

        /// <summary>
        /// Settings of the serial port.
        /// </summary>
        private SerialPortSettings serialPortSettings;

        /// <summary>
        /// Initializes a new instance of the UsbInterface class.
        /// </summary>
        /// <param name="comPort">ComPort identification (f.e. "COM1").</param>
        /// <param name="settings">Connection settings.</param>
        public UsbInterface(string comPort, SerialPortSettings settings)
        {
            this.comPort = comPort;
            this.serialPortSettings = settings;

            this.Connect();
        }

        /// <summary>
        /// Finalizes an instance of the UsbInterface class.
        /// </summary>
        ~UsbInterface()
        {
            this.Disconnect();
        }

        /// <summary>
        /// Delegate for DataRead and DataWrote event.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="e">Read/Wrote data.</param>
        public delegate void DataHandler(object sender, StreamEvent e);

        /// <summary>
        /// Delegate for OnError event.
        /// </summary>
        /// <param name="sender">Sender of the event.</param>
        /// <param name="error">Error wrapper.</param>
        public delegate void OnErrorHandler(object sender, UsbInterfaceError error);

        /// <summary>
        /// Event gets triggered when new data was read.
        /// </summary>
        public event DataHandler DataRead;

        /// <summary>
        /// Event gets triggered when new data was written.
        /// </summary>
        public event DataHandler DataWritten;

        /// <summary>
        /// Event gets triggered when a error during ComPort using appears.
        /// </summary>
        public event OnErrorHandler OnError;

        /// <summary>
        /// Progress of firmware update changed.
        /// </summary>
        public event ProgressChangedEventHandler FirmwareUpdateProgress;

        /// <summary>
        /// Firmware update was completed.
        /// </summary>
        public event RunWorkerCompletedEventHandler FirmwareUpdateCompleted;

        /// <summary>
        /// Type of the USB-interface.
        /// </summary>
        public enum Type
        {
            /// <summary>
            /// Application should be used for I2C-USB-interface.
            /// </summary>
            I2C,

            /// <summary>
            /// Application should be used for SPI-USB-interface.
            /// </summary>
            SPI
        }

        /// <summary>
        /// Indicates whether a read or write operation was executed.
        /// </summary>
        public enum Operation
        {
            /// <summary>
            /// A read operation was executed.
            /// </summary>
            Read,

            /// <summary>
            /// A write operation was executed.
            /// </summary>
            Write
        }

        /// <summary>
        /// Indicates whether spi address a or b should be used.
        /// </summary>
        public enum SpiAddress
        {
            /// <summary>
            /// The device should use spi address a.
            /// </summary>
            A = 0,

            /// <summary>
            /// The device should use spi address b.
            /// </summary>
            B = 1
        }

        /// <summary>
        /// Module to trigger with a command.
        /// Some devices supports auto incrementing of register addresses or different addresses for reading or writing operations.
        /// </summary>
        public enum Module
        {
            /// <summary>
            /// 3D-BS to trigger.
            /// </summary>
            _3DBS,

            /// <summary>
            /// 6D-BS to trigger.
            /// </summary>
            _6DBS,

            /// <summary>
            /// RTC-DCF to trigger.
            /// </summary>
            _RTC,

            /// <summary>
            /// A non supported device should get triggered.
            /// </summary>
            NoSpecialDevice
        }

        /// <summary>
        /// Gets the firmware version of the USB-interface.
        /// </summary>
        /// <value>Firmware version of the USB-interface.</value>
        public string Version { get; private set; }

        /// <summary>
        /// Shows the current status and firmware information.
        /// </summary>
        /// <returns>Action was successfull or not.</returns>
        public bool ShowStatusFirmware()
        {
            const string GetStatusFirmware = "?";

            try
            {
                if (string.IsNullOrEmpty(this.Read(GetStatusFirmware)))
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Connects to the serial port.
        /// </summary>
        public void Connect()
        {
            this.serialPort = new SerialPort(
                this.comPort,
                GeneralTasks.GetBaudRate(this.serialPortSettings.BaudRate),
                this.serialPortSettings.Parity,
                this.serialPortSettings.DataBits,
                this.serialPortSettings.StopBits);
            this.serialPort.Open();
            this.baseStream = this.serialPort.BaseStream;
            this.baseStream.ReadTimeout = UsbInterface.ReadTimeout;

            this.reader = new StreamReader(this.baseStream, new ASCIIEncoding());
            this.writer = new StreamWriter(this.baseStream, new ASCIIEncoding());

            this.workerFirmwareUpdate = new BackgroundWorker();
            this.workerFirmwareUpdate.DoWork += new DoWorkEventHandler(this.WorkerFirmwareUpdate_DoWork);
        }

        /// <summary>
        /// Disconnects from the ComPort.
        /// </summary>
        public void Disconnect()
        {
            try
            {
                this.serialPort.DiscardInBuffer();
                this.serialPort.DiscardOutBuffer();
                this.serialPort.Dispose();
            }
            catch (Exception)
            {
            }

            Thread.Sleep(1000);
        }

        /// <summary>
        /// Enters the bootloader of the USB-interface.
        /// </summary>
        public void EnterBootloader()
        {
            const string CommandEnterBootloader = "ZFF";

            var serialPortSettings = SerialPortSettings.Parse(this.serialPort);

            this.serialPort.ReadExisting();
            this.Write(CommandEnterBootloader);
        }

        /// <summary>
        /// Does a firmware update of the USB-interface.
        /// </summary>
        /// <param name="file">Firmware file to use.</param>
        public void DoFirmwareUpdate(string file)
        {
            if (this.workerFirmwareUpdate.IsBusy)
            {
                ErrorUtils.Show("Es kann nur ein Firmware-Update zur selben Zeit gestartet werden.");
                return;
            }
            else if (!File.Exists(file))
            {
                ErrorUtils.Show("Die angegebene Firmware-Datei existiert nicht.");
            }

            this.workerFirmwareUpdate.RunWorkerAsync(new object[] { file });
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="command">Custom command.</param>
        public void Write(string command)
        {
            try
            {
                lock (this.comPortLocker)
                {
                    this.writer.Write(command);

                    try
                    {
                        this.writer.Flush();
                    }
                    catch (ObjectDisposedException ex)
                    {
                        Console.WriteLine(ex.StackTrace);
                    }
                }
            }
            catch
            {
                if (this.OnError != null)
                {
                    UsbInterfaceError error = new UsbInterfaceError("Beim Schreiben eines Befehls ist ein Fehler aufgetreten.", true);
                    this.OnError(this, error);
                }
            }

            this.DataWritten(this, new StreamEvent(command));
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(byte i2cAddress, byte register, byte data, UsbInterface.Module module)
        {
            this.Write(i2cAddress.ToString("X2"), register, new byte[] { data }, module);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        public void Write(byte i2cAddress, byte register, byte data)
        {
            this.Write(i2cAddress.ToString("X2"), register, new byte[] { data }, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(byte i2cAddress, byte register, byte[] data, UsbInterface.Module module)
        {
            this.Write(i2cAddress.ToString("X2"), register, data, module);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        public void Write(byte i2cAddress, byte register, byte[] data)
        {
            this.Write(i2cAddress.ToString("X2"), register, data, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(UsbInterface.SpiAddress spiAddress, byte register, byte data, UsbInterface.Module module)
        {
            this.Write(spiAddress.ToString(), register, new byte[] { data }, module);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        public void Write(UsbInterface.SpiAddress spiAddress, byte register, byte data)
        {
            this.Write(spiAddress.ToString(), register, new byte[] { data }, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(UsbInterface.SpiAddress spiAddress, byte register, byte[] data, UsbInterface.Module module)
        {
            this.Write(spiAddress.ToString(), register, data, module);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        public void Write(UsbInterface.SpiAddress spiAddress, byte register, byte[] data)
        {
            this.Write(spiAddress.ToString(), register, data, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(string address, byte register, byte data, UsbInterface.Module module)
        {
            this.Write(address, register, new byte[] { data }, module);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Single byte to write.</param>
        public void Write(string address, byte register, byte data)
        {
            this.Write(address, register, new byte[] { data }, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Single register to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read byte.</returns>
        public byte Read(byte i2cAddress, byte register, UsbInterface.Module module)
        {
            return this.Read(i2cAddress.ToString("X2"), register, 1, module)[0];
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">Single register to read.</param>
        /// <returns>Read byte.</returns>
        public byte Read(byte i2cAddress, byte register)
        {
            return this.Read(i2cAddress.ToString("X2"), register, 1, Module.NoSpecialDevice)[0];
        }

        /// <summary>
        /// Reads bytes from several registers from the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">First register to read.</param>
        /// <param name="bytes">Amount of registers to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read bytes.</returns>
        public byte[] Read(byte i2cAddress, byte register, int bytes, UsbInterface.Module module)
        {
            return this.Read(i2cAddress.ToString("X2"), register, bytes, module);
        }

        /// <summary>
        /// Reads bytes from several registers from the device.
        /// </summary>
        /// <param name="i2cAddress">I2C-address of the device.</param>
        /// <param name="register">First register to read.</param>
        /// <param name="bytes">Amount of registers to read.</param>
        /// <returns>Read bytes.</returns>
        public byte[] Read(byte i2cAddress, byte register, int bytes)
        {
            return this.Read(i2cAddress.ToString("X2"), register, bytes, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Single register to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read byte.</returns>
        public byte Read(UsbInterface.SpiAddress spiAddress, byte register, UsbInterface.Module module)
        {
            return this.Read(spiAddress.ToString(), register, 1, module)[0];
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">Single register to read.</param>
        /// <returns>Read byte.</returns>
        public byte Read(UsbInterface.SpiAddress spiAddress, byte register)
        {
            return this.Read(spiAddress.ToString(), register, 1, Module.NoSpecialDevice)[0];
        }

        /// <summary>
        /// Reads bytes from several registers from the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">First register to read.</param>
        /// <param name="bytes">Amount of registers to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read bytes.</returns>
        public byte[] Read(UsbInterface.SpiAddress spiAddress, byte register, int bytes, UsbInterface.Module module)
        {
            return this.Read(spiAddress.ToString(), register, bytes, module);
        }

        /// <summary>
        /// Reads bytes from several registers from the device.
        /// </summary>
        /// <param name="spiAddress">SPI-address of the device.</param>
        /// <param name="register">First register to read.</param>
        /// <param name="bytes">Amount of registers to read.</param>
        /// <returns>Read bytes.</returns>
        public byte[] Read(UsbInterface.SpiAddress spiAddress, byte register, int bytes)
        {
            return this.Read(spiAddress.ToString(), register, bytes, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read byte.</returns>
        public byte Read(string address, byte register, UsbInterface.Module module)
        {
            return this.Read(address, register, 1, module)[0];
        }

        /// <summary>
        /// Reads a byte from one register from the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to read.</param>
        /// <returns>Read byte.</returns>
        public byte Read(string address, byte register)
        {
            return this.Read(address, register, 1, Module.NoSpecialDevice)[0];
        }

        /// <summary>
        /// Reads all available data from the device.
        /// </summary>
        /// <returns>Read data.</returns>
        public string Read()
        {
            return this.Read(string.Empty);
        }

        /// <summary>
        /// Reads all available data from the device.
        /// </summary>
        /// <param name="command">Command which gets written to the device before reading.</param>
        /// <returns>Read data.</returns>
        public string Read(string command)
        {
            this.Write(command);

            string[] excludedReadResult = new string[] { "SPI:" };
            List<string> rawReadData = this.ReadComPort();

            StringBuilder sb = new StringBuilder();
            foreach (var readData in rawReadData)
            {
                this.DataRead(this, new StreamEvent(readData));
                if (!this.IsReadStringRelevant(readData))
                {
                    continue;
                }

                sb.Append(string.Format("{0}\r\n", readData));
            }

            return sb.ToString();
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        /// <param name="module">Module to trigger.</param>
        public void Write(
            string address,
            byte register,
            byte[] data,
            UsbInterface.Module module)
        {
            switch (module)
            {
                case Module._3DBS:
                    BitUtils.SetBit(ref register, _3DBS.ReadWriteBit, false);
                    break;
                case Module._6DBS:
                    BitUtils.SetBit(ref register, _6DBS.ReadWriteBit, false);
                    BitUtils.SetBit(ref register, _6DBS.OperationAmountBit, data.Length > 1);
                    break;
                case Module._RTC:
                    // BitUtils.SetBit(ref register, _RTC.SingleBurstBit, data.Length == 1);
                    BitUtils.SetBit(ref register, _RTC.ReadWriteBit, false);
                    break;
                case Module.NoSpecialDevice:
                    break;
                default:
                    throw new ArgumentException();
            }

            StringBuilder dataToString = new StringBuilder();
            foreach (var b in data)
            {
                dataToString.Append(string.Format("{0} ", b.ToString("X2")));
            }

            // length
            string command = string.Format(
                "S{0} W{1} {2} N{0}",
                address,
                register.ToString("X2"),
                dataToString.ToString().Trim());

            this.Write(command);
        }

        /// <summary>
        /// Writes a command to the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">Register to write.</param>
        /// <param name="data">Bytes to write.</param>
        public void Write(
            string address,
            byte register,
            byte[] data)
        {
            this.Write(address, register, data, Module.NoSpecialDevice);
        }

        /// <summary>
        /// Reads bytes from several registers from the device.
        /// </summary>
        /// <param name="address">Address of the device.</param>
        /// <param name="register">First register to read.</param>
        /// <param name="bytes">Amount of registers to read.</param>
        /// <param name="module">Module to trigger.</param>
        /// <returns>Read bytes.</returns>
        public byte[] Read(string address, byte register, int bytes, UsbInterface.Module module)
        {
            switch (module)
            {
                case Module._3DBS:
                    BitUtils.SetBit(ref register, _3DBS.ReadWriteBit, true);
                    break;
                case Module._6DBS:
                    BitUtils.SetBit(ref register, _6DBS.ReadWriteBit, true);
                    BitUtils.SetBit(ref register, _6DBS.OperationAmountBit, bytes > 1);
                    break;
                case Module._RTC:
                    BitUtils.SetBit(ref register, _RTC.ReadWriteBit, true);
                    break;
                case Module.NoSpecialDevice:
                    break;
                default:
                    throw new ArgumentException();
            }

            string command = string.Format(
                "S{0} W{1} R{2} N{0}",
                address,
                register.ToString("X2"),
                bytes.ToString("X2"));

            this.Write(command);
            List<string> rawReadData = this.ReadComPort();

            if (rawReadData.Count == 0)
            {
                return new byte[] { };
            }

            List<byte> result = new List<byte>();
            foreach (var readData in rawReadData)
            {
                if (!this.IsReadStringRelevant(readData))
                {
                    continue;
                }

                this.DataRead(this, new StreamEvent(readData));
                foreach (var readByte in readData.Split(' '))
                {
                    if (string.IsNullOrEmpty(readByte))
                    {
                        continue;
                    }

                    result.Add(byte.Parse(readByte, System.Globalization.NumberStyles.AllowHexSpecifier));
                }
            }

            byte[] readBytes = result.ToArray();
            if (readBytes.Length == 0)
            {
                throw new InterfaceException(string.Format("Für den Befehl {0} konnte keine Antwort gelesen werden.", command));
            }

            return readBytes;
        }

        /// <summary>
        /// Reads all available lines of a ComPort stream.
        /// </summary>
        /// <returns>Read lines from ComPort.</returns>
        private List<string> ReadComPort()
        {
            List<string> rawReadData = new List<string>();

            try
            {
                lock (this.comPortLocker)
                {
                    while (!this.reader.EndOfStream)
                    {
                        rawReadData.Add(this.reader.ReadLine());
                    }
                }
            }
            catch (TimeoutException)
            {
            }
            catch (UnauthorizedAccessException)
            {
                if (this.OnError != null)
                {
                    UsbInterfaceError error = new UsbInterfaceError("Die Verbindung zum USB-Interface wurde zwischenzeitlich getrennt.", true);
                    this.OnError(this, error);
                }
            }
            catch (ObjectDisposedException)
            {
                if (this.OnError != null)
                {
                    UsbInterfaceError error = new UsbInterfaceError("Die Verbindung zum USB-Interface wurde zwischenzeitlich getrennt.", true);
                    this.OnError(this, error);
                }
            }
            catch
            {
                if (this.OnError != null)
                {
                    UsbInterfaceError error = new UsbInterfaceError("Beim Auslesen ist ein Fehler aufgetreten.", true);
                    this.OnError(this, error);
                }
            }

            return rawReadData;
        }

        /// <summary>
        /// Checks whether a read string should be displayed or not.
        /// </summary>
        /// <param name="readString">Read string to check.</param>
        /// <returns>Result of the check.</returns>
        private bool IsReadStringRelevant(string readString)
        {
            const string VersionIdentifier = "Interface v";

            if (string.IsNullOrEmpty(readString))
            {
                return false;
            }

            if (readString.Contains("Interface v"))
            {
                var split = readString.Split(new string[] { VersionIdentifier }, StringSplitOptions.RemoveEmptyEntries);
                if (split.Length >= 2)
                {
                    this.Version = split[1];
                }
            }

            string[] excludeReadResult = new string[] { "SPI:" };
            foreach (var exclude in excludeReadResult)
            {
                if (readString.Contains(exclude))
                {
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Starts a firmware updater and updates the firmware.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void WorkerFirmwareUpdate_DoWork(object sender, DoWorkEventArgs e)
        {
            object[] arguments = (object[])e.Argument;
            string file = (string)arguments[0];

            FirmwareUpdater updater = new FirmwareUpdater(this.serialPort);
            updater.ProgressChanged += new ProgressChangedEventHandler(this.FirmwareUpdater_ProgressChanged);
            updater.RunWorkerCompleted += new RunWorkerCompletedEventHandler(this.FirmwareUpdater_RunWorkerCompleted);
            updater.Start(file);
        }

        /// <summary>
        /// Passes received events for changed progress of the firmware update.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void FirmwareUpdater_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            if (this.FirmwareUpdateProgress != null)
            {
                this.FirmwareUpdateProgress(this, e);
            }
        }

        /// <summary>
        /// Passes received events for a completed firmware update.
        /// </summary>
        /// <param name="sender">Standard parameter sender.</param>
        /// <param name="e">Standard parameter e.</param>
        private void FirmwareUpdater_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (this.FirmwareUpdateCompleted != null)
            {
                this.FirmwareUpdateCompleted(this, e);
            }
        }
    }
}