﻿//-----------------------------------------------------------------------------
// <copyright file="BcdUtils.cs" company="eQ-3 Entwicklung GmbH">
//  Copyright (c) 2013 eQ-3 Entwicklung GmbH
// </copyright>
// <summary>Utility methods for handling numbers represented as binary coded decimals (BCD).</summary>
//-----------------------------------------------------------------------------
namespace UsbInterfaceTesttool.Utils
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using UsbInterfaceTesttool.Devices;

    /// <summary>
    /// Utility methods for handling numbers represented as binary coded decimals (BCD).
    /// </summary>
    public static class BcdUtils
    {
        /// <summary>
        /// Converts a DateTime to a array of bytes/BCDs.
        /// </summary>
        /// <param name="time">DateTime to convert.</param>
        /// <returns>Converted array of bytes/BCDs.</returns>
        public static byte[] ConvertDateTimeToBytes(DateTime time)
        {
            byte[] timeBytes = new byte[_RTC.TimeBytesAmount];
            timeBytes[0] = BcdUtils.ConvertIntToBcd(time.Second);
            timeBytes[1] = BcdUtils.ConvertIntToBcd(time.Minute);
            timeBytes[2] = BcdUtils.ConvertIntToBcd(time.Hour);
            timeBytes[3] = BcdUtils.ConvertDayOfWeekToBcd(time.DayOfWeek);
            timeBytes[4] = BcdUtils.ConvertIntToBcd(time.Day);
            timeBytes[5] = BcdUtils.ConvertIntToBcd(time.Month);
            timeBytes[6] = BcdUtils.ConvertIntToBcd(time.Year % 100);

            return timeBytes;
        }

        /// <summary>
        /// Converts a array of bytes/BCDs to a DateTime.
        /// </summary>
        /// <param name="timeBytes">Array of bytes/BCDs.</param>
        /// <returns>Converted DateTime.</returns>
        public static DateTime ConvertBytesToDateTime(byte[] timeBytes)
        {
            if (timeBytes.Length != _RTC.TimeBytesAmount)
            {
                throw new ArgumentException();
            }

            try
            {
                int[] values = new int[]
                {
                    BcdUtils.ConvertBcdToInt(timeBytes[6]),
                    BcdUtils.ConvertBcdToInt(timeBytes[5]),
                    BcdUtils.ConvertBcdToInt(timeBytes[4]),
                    BcdUtils.ConvertBcdToInt(timeBytes[2]),
                    BcdUtils.ConvertBcdToInt(timeBytes[1]),
                    BcdUtils.ConvertBcdToInt(timeBytes[0])
                };

                // DateTime cannot manage year 0, but year 100
                if (values[0] == 0)
                {
                    values[0] = 100;
                }

                return new DateTime(values[0], values[1], values[2], values[3], values[4], values[5]);
            }
            catch (Exception)
            {
                return DateTime.FromBinary(10000);
            }
        }

        /// <summary>
        /// Converts a number to a byte/BCD.
        /// </summary>
        /// <param name="number">Number to convert.</param>
        /// <returns>Converted byte/BCD.</returns>
        public static byte ConvertIntToBcd(int number)
        {
            number %= 100;
            return byte.Parse(number.ToString(), System.Globalization.NumberStyles.AllowHexSpecifier);
        }

        /// <summary>
        /// Converts a byte/BCD to a number.
        /// </summary>
        /// <param name="bcd">Byte/BCD to convert.</param>
        /// <returns>Converted number.</returns>
        public static int ConvertBcdToInt(byte bcd)
        {
            int high = bcd >> 4;
            int low = bcd & 0xF;
            return (10 * high) + low;
        }

        /// <summary>
        /// Converts a DayOfWeek (enum used in DateTime) to a byte/BCD.
        /// </summary>
        /// <param name="dayOfWeek">DayOfWeek to convert.</param>
        /// <returns>Converted byte/BCD.</returns>
        public static byte ConvertDayOfWeekToBcd(DayOfWeek dayOfWeek)
        {
            switch (dayOfWeek)
            {
                case DayOfWeek.Monday:
                    return 0x01;
                case DayOfWeek.Tuesday:
                    return 0x02;
                case DayOfWeek.Wednesday:
                    return 0x03;
                case DayOfWeek.Thursday:
                    return 0x04;
                case DayOfWeek.Friday:
                    return 0x05;
                case DayOfWeek.Saturday:
                    return 0x06;
                case DayOfWeek.Sunday:
                    return 0x07;
                default:
                    throw new ArgumentException();
            }
        }
    }
}
